function Bn=bernoull(n)
%BERNOULL Bernoulli numbers of indexes specified in n

%	Written by I. Kollar
%	Last Modified: 13-Dec-1994.

%	Reference: Korn, G. A. and Korn, T. M. (1968), Mathematical handbook
%	for scientists and engineers; definitions, theorems, and formulas for
%	reference and review. 2nd ed., New York, McGraw-Hill. Section 21-5.2.

Bn=n(:);
if any(rem(Bn,1)~=0), error('Indexes are not all integers'), end
if any(~finite(Bn)), error('Some indexes are infinite or NaN'), end
if any(imag(Bn)~=0), error('Some indexes are complex'), end
if any(Bn<0), error('Some indexes are negative'), end
%
nmax=max(Bn);
Bnv=zeros(nmax+1,1); %all Bernoulli numbers
binom=ones(1,nmax+1); binom(2)=2; %binomial coefficients
Bnv(1)=1; Bnv(2)=-0.5;
for k=2:nmax
  binom(2:k+1)=binom(1:k)+binom(2:k+1); %recursion for binomial coefficients
  if rem(k,2)==0 %k even
    Bnv(k+1)=-1/(k+1)*(binom(1:k)*Bnv(1:k));
  end
end
Bn=zeros(size(n));
Bn(:)=Bnv(n+1);
%
%End of bernoull
%1.0000   -0.5000    0.1250         0    0.0417         0   -0.1979
