function problem_19_24

disp('Solving problem 19.24 ...')

close all
fc=10; fs=1e3; dt=1/fs; di=round(2.56*fs); 
x=mlbs(10)';
for ii=1:round(fs/fc)-1, x=[x;x(1,:)]; end
x=x(:); x=[x;zeros(2*di,1)];
N=length(x);
Pr=zeros(size(x));
Pr(1+di:end)=(x(1:end-di)+1)/2;

%figure(5)
%stairs([1:length(x)]/fs,x)
%title('Pseudorandom signal')
%if exist('saveprobfig.m'), saveprobfig5,['figp_16_20a.eps'])

sig2=10^(20/10); df=1e6;
Ps=1; Pn=sig2; stdPn=2*sig2^2/(2*df*dt);

Prn=Pr+stdPn*randn(size(Pr))+Pn;
mPrn=mean(Prn); stdPrn=std(Prn);

figure(6)
plot([0:N-1]/fs,Prn(1:N),'.','markersize',0.3), %'color',0.9*[0.3,0.3,1]
set(gca,'xlim',[0,25])
hold on
ver=version;
if ver(1)>'6'
  stairs([0:N-1]/fs,Pr(1:N)+Pn,'linewidth',1.7,'color','black')
else
  stairs([0:N-1]/fs,Pr(1:N)+Pn,'color','black')
end
plot([0,N-1]/fs,(Pn+stdPrn)*[1,1],':','linewidth',0.7,'color','black')
plot([0,N-1]/fs,Pn-stdPrn*[1,1],':','linewidth',0.7,'color','black')
hold off
title('Measured power with useful power buried in noise')
shg
if exist('saveprobfig.m'), saveprobfig(6,'figp_16_20f.eps'), end

for qn=[4,3,2,1]
  if qn==4, c=[-1,0,1]*stdPrn; l=[-1.5,-0.5,0.5,1.5];
  elseif qn==3, c=[-0.5,0.5]*stdPrn; l=[-1,0,1];
  elseif qn==2, c=[0]*stdPrn; l=[-1,1];
  elseif qn==1, c=[-127:127]/128*3*stdPrn; l=[-127.5:127.5]/128*3*stdPrn;
  end
  c=[-inf,c,inf]; Prnq=zeros(size(Prn));
  for li=1:length(l)
    ind=find((Prn-mPrn>=c(li))&(Prn-mPrn<c(li+1)));
    if ~isempty(ind), Prnq(ind)=l(li)*ones(size(ind)); end
  end
  Pc=xcov(Prnq,x,'unbiased');
  Pccentr=Pc(N+[-N/2:N/2]);
  figure(qn)
  tau=[-N/2:N/2]/fs;
  plot(tau,Pccentr), set(gca,'xlim',[-5,5]), shg
  m=max(Pccentr);
  set(gca,'ylim',m*[-0.2,1.1])
  if qn==1, qnt=256; else qnt=qn; end
  title(sprintf('Cross-correlation, quantization levels: %.0f',qnt))
  if exist('saveprobfig.m'), saveprobfig(qn,['figp_16_20',setstr('a'+qn-1),'.eps']), end
end

function [bitseries,nextstnum]=mlbs(log2N,varargin)
%MLBS   Maximum length binary sequence (pseudo-random binary sequence, PRBS).
%
%       [bitseries,nextstnum]=MLBS(log2N,runmod)
%
%       Output arguments:
%       bitseries = generated bit series (values +1,-1).
%           If runmod is a structure, or 'tiddata', bitseries is a tiddata object.
%           Else it is a column vector, length: bitno, default length: 2^log2N-1
%       nextstnum = startnum to be used if continued sequence generation
%           follows.
%
%       Input arguments:
%       log2N = bit length of the shift register: integer number 1<=log2N<=30
%       runmod = run modifier structure
%         fs = frequency of samples
%         bitno = number of bits to be generated (length of bitseries)
%           default: 2^log2N-1
%         inverserepeat = 'on' for inverse repeat mlbs (odd spectral lines)
%         startnum = digital equivalent of the start value of the
%           shift register, considered as a regular binary number containing
%           0-s and 1-s.  1<=startnum<2^log2N
%
%       Usage: [bitseries,nextstnum]=mlbs(log2N,bitno,startnum);
%       Example: bitseries=mlbs(10);
%
%       Algorithm:
%         K. R. Godfrey, ed.: Perturbation Signals for System Identification.
%         Englewood Cliffs, Prentice-Hall, 1993.

error(nargchk(1,3,nargin))
if rem(log2N,1)~=0, error('log2N is not integer'), end
%
newcall=0;
if nargin>1
  runmod=varargin{1};
  if isstruct(runmod)|isa(runmod,'tiddata')|strncmp(runmod,'tiddata',5)
    newcall=1;
  else
    newcall=0;
  end
end
if ~newcall
  if nargin<2, bitno=[]; else bitno=varargin{1}; end
  if nargin<3, startnum=[]; else startnum=varargin{2}; end
  invr=0; fs=1;
else
  bitno=[]; startnum=[]; fs=1; invr=0;
  if isstruct(runmod)
    fn=fieldnames(runmod);
    for ii=1:length(fn)
      if strncmp(fn{ii},'bitno',4), bitno=getfield(runmod,fn{ii});
      elseif strncmp(fn{ii},'startnum',6), startnum=getfield(runmod,fn{ii});
      elseif strncmp(fn{ii},'fs',6)|strncmp(fn{ii},'fc',6)
        fs=getfield(runmod,fn{ii});
      elseif strncmp(fn{ii},'inverserepeat',3)
        invrstr=getfield(runmod,fn{ii});
        if strcmpi(invrstr,'on'), invr=1;
        elseif strcmpi(invrstr,'off')|isempty(invrstr)
        else error(['Invalid field ''',fn{ii},''''])
        end
      else error(['Unknown runmod field ''',fn{ii},''''])
      end
    end %for ii
  end
end
reglen=2^log2N-1;
if isempty(bitno), bitno=(invr+1)*reglen; end
if bitno>(invr+1)*reglen
  disp('WARNING! bitno exceeds maximum length of PRBS in mlbs')
end
if bitno<0, error('negative value of bitno'), end
%
if isempty(startnum), startnum=pow2(log2N)-1; end
%
if (startnum<1)|(startnum>pow2(log2N)-1), error('startnum out of range'), end
startnum=round(rem(abs(startnum-1),2^log2N-1))+1;
stn=startnum; reg=zeros(log2N,1);
for i=1:log2N
  reg(i)=rem(stn,2); stn=(stn-reg(i))/2;
end
zind=find(reg==0); reg(zind)=-1*ones(length(zind),1); %values +1,-1
%
if     log2N==1, multind=[1];
elseif log2N==2, multind=[1,2];
elseif log2N==3, multind=[2,3];
elseif log2N==4, multind=[3,4];
elseif log2N==5, multind=[3,5];
elseif log2N==6, multind=[5,6];
elseif log2N==7, multind=[4,7];
elseif log2N==8, multind=[4,5,6,8];
elseif log2N==9, multind=[5,9];
elseif log2N==10, multind=[7,10];
elseif log2N==11, multind=[9,11];
elseif log2N==12, multind=[6,8,11,12];
elseif log2N==13, multind=[9,10,12,13];
elseif log2N==14, multind=[4,8,13,14];
elseif log2N==15, multind=[14,15];
elseif log2N==16, multind=[4,13,15,16];
else error(sprintf('log2N=%.0f is not allowed',log2N))
end
bitseries=zeros(min(reglen,bitno),1);
for i=1:length(bitseries)
  bitseries(i)=-prod(reg(multind));
  reg=[bitseries(i);reg(1:log2N-1)];
  %if (rem(i,100)==0)|(i==2^log2N-1)
    %fprintf('mlbs ready with %.0f bits of %.0f\n',i,2^log2N-1)
  %end
end
if (length(reg)==1)&(length(bitseries)>0), bitseries=abs(bitseries); end
if invr
  bitseries=[bitseries;bitseries];
  bitseries(2:2:end)=-bitseries(2:2:end);
end
while length(bitseries)<bitno, bitseries=[bitseries;bitseries]; end
bitseries=bitseries(1:bitno);
if newcall
  bitseries=tiddata([],bitseries,1/fs);
end
%
nextstnum=sum((reg/2+1/2).*(2.0.^[0:log2N-1]'));
%
%end of mlbs
%%%%%%%%%%%%%%%%%%%%%%%% end of mlbs %%%%%%%%%%%%%%%%%%%%%%%%
