function z = abs(a)
%ABS  Absolute value of @qdata objects.
%   ABS(X) is the absolute value of the elements of X. When
%   X is complex, ABS(X) is the complex modulus (magnitude) of
%   the elements of X. <-- *** currently not supported due to the lack of SQRT
%   function ***
%
%   See also QDATA/SIGN.

% Copyright (C) 2004 by JUHASZ Laszlo

name = a.name;
a = a.data;
% filtering out empty matrix
if isempty(a), out = []; return, end

orsiza = size(a);
% calculating the number of segments in 'a'
segna = orsiza(1);
% calculating the real size WITHOUT segments - the length of size is
% minimum 2 so the '2:end' expression is always correct.
orsiza = orsiza(2:end);
% convert to two dimension (for the ABS operations on complex
% numbers)
a = reshape(a,[segna,prod(orsiza)]);

% deciding whether input are pure real, pure imag or complex
re = real(a);
im = imag(a);
isre = any(re(:));
isim = any(im(:));

if ~isre & ~isim
   % full zero
   y = a;
elseif isre & ~isim
   % pure real
   % generating index vector for positive and negative numbers. 'sum' is needed
   % due to phisically denormalized numbers
   pos = sign(sum(sign(re)))>=0;
   neg = sign(sum(sign(re)))<0;
   % extend the row-vectors to 'a's size
   pos = pos(ones(segna,1),:);
   neg = neg(ones(segna,1),:);
   % generating output
   y = pos.*re-(neg.*re);
elseif ~isre & isim
   % pure imaginary
   % generating index vector for positive and negative numbers. 'sum' is needed
   % due to phisically denormalized numbers
   pos = sign(sum(sign(im)))>=0;
   neg = sign(sum(sign(im)))<0;
   % extend the row-vectors to 'a's size
   pos = pos(ones(segna,1),:);
   neg = neg(ones(segna,1),:);
   % generating output
   y = pos.*im-(neg.*im);
else
   % complex numbers
   error('ABS on complex numbers currently not supported due to the lack of SQRT function.')
end

% ***** TRAILER *****
% reshape to original size
y = reshape(y,[segna,orsiza]);

% generating output object
z.name = name;
z.data = y;
z = class(z,'qdata');
