function [ki,longstr] = b_divide(a,b,prec)
%B_DIVIDE  Division through conversion to bit sequences
%
%   ki = b_divide(a,b) %a,b: qdata objects

%   ki = b_divide(a,b,prec,dummyobj) %a=obj.data
%      Here dummyobj is used to be able to call with vectors

if isa(a,'qdata')
  avect=a.data; bvect=b.data; Q=a.name;
  if nargin<3, prec=qget(a.name,'precision'); end
else
  avect=a; bvect=b; Q='double';
  if nargin<3, prec=53; end
end
sign_a = sign(avect(1));
sign_b = sign(bvect(1));
avect = abs(avect);
bvect = abs(bvect);
sign_k = sign_a / sign_b;

[a_binary e_a]=num2binsegm(avect,22,prec);
[b_binary e_b]=num2binsegm(bvect,22,prec);

a_binary=[a_binary,zeros(1,2*prec)];
result=[]; lB=length(b_binary);
while any(a_binary)&(length(a_binary)>=length(b_binary))
  if a_binary(1)==0
    result=[result,0]; 
    a_binary(1)=[];
  else %leading 1 or 2
    amod=a_binary; amod(1:lB)=amod(1:lB)-b_binary;
    while ~isempty(amod)&(amod(1)==0), amod(1)=[]; end
    if ~isempty(amod)&(amod(1)<0)
      result=[result,0]; 
      a_binary(2)=a_binary(2)+2*a_binary(1);
      a_binary(1)=[];
    else %possible to subtract
      result=[result,1];
      a_binary(1:lB)=a_binary(1:lB)-b_binary;
      %eliminate -1 values
      ind=find(a_binary==-1);
      while ~isempty(ind)
        a_binary(ind)=a_binary(ind)+2;
        a_binary(ind-1)=a_binary(ind-1)-1;
        ind=find(a_binary==-1);
      end
      if a_binary(1)==1, a_binary(2)=a_binary(2)+2*a_binary(1);
      elseif a_binary(1)~=0, error('Error during binary division')
      end
      a_binary(1)=[];
    end
  end
end
if nargout>=2
  [result,emod,longstr]=fixbin(result,Q,sign_k,e_a+e_b);
else
  [result,emod]=fixbin(result,Q);
end
while (length(result)>0)&(result(1)==0), result(1)=[]; emod=emod-1; end
if length(result)<prec, result=[result,zeros(1,prec-length(result))]; end
e=e_a-e_b+emod;

%javitva, s most szegmensels
N=length(avect);
szam = zeros(N,1);
L=22;

for k = 1 : N
  for n = 1 : L
    szam(k) = 2^(-(n-1)) * result((k-1)*L+n) + szam(k);
    if (k-1)*L+n==length(result), break, end
  end
end

szam = szam * 2 ^ (e);
ki = sign_k * szam;
if isa(a,'qdata')
  num.name=a.name; num.data=ki; 
  ki=qdata(num);
end
