function disp(x)
%DISP  Display @qdata objects without printing object name.
%   DISP(X) displays the object, without printing the object name.  In
%   all other ways it's the same as leaving the semicolon off an
%   expression.
%
%   See also INT2STR, NUM2STR, SPRINTF, RATS, FORMAT.

%   DISP performs minor format checks related to storage of objects too.

% Copyright (C) 2003,2004 by JUHASZ Laszlo

% group number - controls how many bits are in a group separated with
% spaces from each other
gn = 0;

% argument check
error(nargchk(1,1,nargin))

siz = size(x);
len = max(size(x));
name = x.name;
x = x.data;
% obtaining real and imag part
re = real(x);
im = imag(x);
% indicates that any nonzero data exist in either real or imag parts
isre = any(re(:));
isim = any(im(:));
% filtering out empty and zero objects
if isempty(name)
   if isempty(x)
      % empty object
      %fprintf('     []\n')
      return
   else
      error(['Internal error. Wrong object: it doesn''t contain ',...
            'quantizer name, but isn''t proper [] or 0.'])
   end
else
   % reverse check
   if isempty(x)
      error('Internal error. Wrong object: it contains quantizer name, but is empty.')
   end
end

% request quantizer properties
[code,err] = qdspdemo('quantizers','get',name); error(err)
[code,err] = qdspdemo('quantizers',code.name); error(err)
eval(code)

% correct segnum in case when the quantizer in memory has changed
SEGNUM=size(x,1);

% filter out when the object is not vector
if prod(siz)~=len
   % generating the size string
   csiz = '';
   for k = 1:length(siz)-1
      csiz = [csiz,sprintf('%dx',siz(k))];
   end
   csiz = [csiz,sprintf('%d',siz(end))];
   fprintf('[%s qdata]\n\n',csiz)
   return
end

% collect the displayed strings into cell
rec = cell(len,1);
imc = cell(len,1);
for k = 1:len
   rec{k} = intern_disp(real(x(:,k).'),code,gn);
   imc{k} = intern_disp(imag(x(:,k).'),code,gn);
end

% obtain screen width
scr = get(0,'ScreenSize');
scr = scr(3);
% available width in chars (optimized for 800x600, screen font: Fixedsys 11)
scr = fix(scr*93/800);

spa = ' ';
% display
for k = 1:len
   t = rec{k};
   if (length(t)+6)>scr
      u = round(length(t)/2);
      point = findstr(t,'.');
      % search for a group separator space at the screen
      sepa = findstr(t(u-gn:u+gn),' ');
      % select separator
      sepa = round(length(sepa)/2);
      % which space is the sepa
      sepa = length(findstr(t(1:u-gn-1),' '))+sepa;
      % which position;
      sepa2 = findstr(t,' ');
      sepa = sepa2(sepa);
      % 1st row
      fprintf('      %-s...\n',t(1:sepa-1))
      % 2nd row
      fprintf('%s%-s\n',spa(ones(point+6,1)),t(sepa+1:end))
   else
      fprintf('      %-s\n',t)
   end
   if isim
      t = imc{k};
      if length(t)>scr
         % search for a group separator space at the end of screen
         sepa = findstr(t(scr-2*gn:scr),' ');
         % select separator
         sepa = sepa(ceil(length(sepa)/2));
         % which space is the sepa
         sepa = length(findstr(t(1:scr-2*gn-1),' '))+sepa;
         % which position;
         sepa2 = findstr(t,' ')
         sepa = sepa2(sepa);
         % 1st row
         fprintf('  +j*{%-s...\n\n',t(1:sepa-1))
         % 2nd row
         fprintf('         %-s}\n',t(sepa+1:end))
      else
         fprintf('  +j*{%-s}\n\n',imc{k})
      end
   end
end

fprintf('\n')
