function s=dec2bin(qdata_obj,nums)
%DEC2BIN  converts the data of a qdata object to a binary string

if nargin==0
   % called without argument
   s = '';
   return
end

if (nargin<2)
  nums=22; % Need at least one digit even for 0.
end

if (nargin==1)&isa(qdata_obj,'qdata');
    d = qdata_obj.data;
    [f_d,e_d] = log2(d);
    e_common = e_d(1);
    s = dec2bin_inner((abs(f_d)*2^nums).*(2.^(e_d-e_common)));
    return
   
else 
    s = dec2bin_inner(qdata_obj,nums);
    error('QDATA:DEC2BIN:MAIN','The input is not an QDATA object');
    
end
    
%-------------------------------------------------------
%Do not change below this line, this is the real dec2bin
%-------------------------------------------------------

function s=dec2bin_inner(d,n)
%DEC2BIN Convert decimal integer .
%   DEC2BIN(D) returns the binary representation of D as a string.
%   D must be a non-negative integer smaller than 2^52. 
%
%   DEC2BIN(D,N) produces a binary representation with at least
%   N bits.
%
%   Example
%      dec2bin(23) returns '10111'
%
%   See also BIN2DEC, DEC2HEX, DEC2BASE.

%   Hans Olsson, hanso@dna.lth.se 3-23-95
%   Copyright 1984-2002 The MathWorks, Inc. 
%   $Revision: 1.13 $  $Date: 2002/04/09 00:33:33 $

%
% Input checking
%
if nargin==0, error('Not enough input arguments.'); end
if isempty(d), s = ''; return, end

d = d(:); % Make sure d is a column vector.

if (nargin<2)
  n=1; % Need at least one digit even for 0.
else
  if prod(size(n))~=1 | n<0, error('N must be a positive scalar.'); end
  n = round(n); % Make sure n is an integer.
end;

%
% Actual algorithm
%
[f,e]=log2(max(d)); % How many digits do we need to represent the numbers?
s=setstr(rem(floor(d*pow2(1-max(n,e):0)),2)+'0');
