function C = ge(a,b)
%GE (>=)  Greater than or equal.
%    A >= B does element by element comparisons between A and B
%    and returns a matrix of the same size with elements set to one
%    where the relation is true and elements set to zero where it is
%    not.  A and B must have the same dimensions unless one is a
%    scalar. A scalar can be compared with anything.
% 
%    C = GE(A,B) is called for the syntax 'A >= B' when A or B is an
%    object.
%
%	(C) 2004 Rvsz Sndor

% ***** PREAMBLE *****
switch class(a)
   case 'qdata'
      if isempty(a.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsiza = size(a.data);
         % calculating the number of segments in 'a'
         segna = orsiza(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsiza = orsiza(2:end);
         % convert to two dimension
         a.data = reshape(a.data,[segna,prod(orsiza)]);
      end
   case 'double'
      if isempty(a)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'a' argument to segments
         % save original size of 'a'
         orsiza = size(a);
         % generating segnum - it's always 3 in case of 'double'
         segna = 3;
         % convert 'a' to column vector
         a = a(:);
         % fill up the variable 'z' with segment-transformed values of 'a'
         for k = 1:length(a)
            z(:,k) = double2seg(a(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'a' and put back the data of 'z' to the
         % 'data' field
         a = [];
         a.data = z;
         a.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end

switch class(b)
   case 'qdata'
      if isempty(b.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsizb = size(b.data);
         % calculating the number of segments in 'b'
         segnb = orsizb(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsizb = orsizb(2:end);
         % convert to two dimension
         b.data = reshape(b.data,[segnb,prod(orsizb)]);
      end
   case 'double'
      if isempty(b)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'b' argument to segments
         % save original size of 'b'
         orsizb = size(b);
         % generating segnum - it's always 3 in case of 'double'
         segnb = 3;
         % convert 'b' to column vector
         b = b(:);
         % fill up the variable 'z' with segment-transformed values of 'b'
         for k = 1:length(b)
            z(:,k) = double2seg(b(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'b' and put back the data of 'z' to the
         % 'data' field
         b = [];
         b.data = z;
         b.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end

% download quantizer properties - the first argument ('a') has a bigger
% priority.
if ~isempty(a.name)
   qcode = qdspdemo('quantizers',a.name);
elseif ~isempty(b.name)
   qcode = qdspdemo('quantizers',b.name);
else
   error('One of the inputs must have a quantizer name.');
end   
eval(qcode)

% extract values from structs - the name is unnecessary from now on
a = a.data;
b = b.data;

% scalar extension and check incorrect lengths
% 'a' and 'b' are both 2-dim arrays - compatibility with Matlab 5.3!!!
if prod(orsiza)==1
   % the 'a' object is scalar so its variable is a column vector
   % extend 'a' to the size of 'b'
   a = a(1:size(a,1),ones(size(b,2),1));
   orsiza = orsizb;
elseif prod(orsizb)==1
   % the same case with 'b'
   b = b(1:size(b,1),ones(size(a,2),1));
   orsizb = orsiza;
elseif ~isequal(orsiza,orsizb)
   error('Matrix dimensions must agree.')
end

orsiz(1:length(orsiza)) = orsiza;
orsiz(1:length(orsizb)) = orsizb;

% from now size(a,2)==size(b,2)

% balancing number of segments
if segna~=segnb
   if segna>segnb
      % extend number of rows of 'b' to the number of rows of 'a'
      b(segna,:) = 0;
   else
      % extend number of rows of 'a' to the number of rows of 'b'
      a(segnb,:) = 0;
   end
end

% from now size(a,1)==size(b,1) so size(a)==size(b)
% ***** end of PREAMBLE *****



C = zeros(1,size(a,2));

for k = 1:size(a,2)
    i=1;
    while(i <= size(a,1))
        if(a(i,k)==b(i,k))
            i = i + 1;
            continue;	        
        else
            if(a(i,k)>=b(i,k))
                C(k) = 1;
            end
            break;
        end
    end
end



% ***** TRAILER *****
% reshape to original size
if~(prod(size(C))==1)
    %akkor nem skalr szm volt
    C = reshape(C,orsiz);
end
