function [lsbmag,value]=lsb(x,dummy)
%LSB  Least significant bit (LSB) value in a (usually floating-point) number
%
%       [lsbmag,value]=lsb(x)
%
%   The magnitude of bit 1 at the LSB place and value of the LSB (0 or 1).
%
%   Examples: 
%       Q=qmake('Qd','double'); Qs=qmake('Qs','single');
%       qmake('Qf','Q16bit');
%       one=qdata(1,Q); two=qdata(2,Q); one_sing=qdata(1,'Qs');
%       [mag,lsbit]=lsb(one)  %is [eps = 2^(-52) = 2.2204e-016,0]
%       [mag,lsbit]=lsb(two)  %is [2*eps = 2*2^(-52) = 4.4409e-016,0]
%       [mag,lsbit]=lsb(one_sing+2^(-23))  %is [eps = 2^(-23) = 1.1921e-007,1]
%       [mag,lsbit]=lsb(qdata(0,'Qd'))  %is [realmin*eps = 2^(-1022-52) = 4.9407e-324,0]
%       [mag,lsbit]=lsb(qdata(2^16-1,'Qf')) %is [1,1]

%	Copyright (c) I. Kollar, 2006, all rights reserved
%	Last modified: 4-May-2006

if all(size(x)==[1 1])
    qcode = qdspdemo('quantizers',x.name);
    eval(qcode); %set parameters as variables
    if strcmp(TYPE,'floating-point')
      [xm,xe] = log2(x.data(1));
      %M = 2 ^  (xe - (SEGNUM * 22));
      if (xm==0)|(xe<-1021), xe=log2(realmin)+1; end
      lsbmag=pow2(xe-PRECISION);
      datai=x.data(floor((PRECISION-1)/22)+1);
      value=floor((rem(abs(datai),2*pow2(lsbmag,22*floor((PRECISION-1)/22))))/...
        (pow2(lsbmag,22*floor((PRECISION-1)/22))));
    else %fixed-point
      lsbmag=pow2(-FRACTIONBITS);
      datai=x.data(floor((PRECISION-1)/22)+1);
      value=rem(abs(datai)/pow2(floor((PRECISION-1)/22)),2*lsbmag)/lsbmag;
    end
else
    error('LSB must used with scalar qdata objects');
end
if lsbmag<=0, error('lsbmag is not positive'), end
if value<0, error('value is negative'), end
