function y = plus(a,b)
%PLUS(+)       Add @qdata objects.
%     X + Y adds qdata objects X and Y.  X and Y must have the same
%     dimensions unless one is a scalar (a 1-by-1 object).
%     A scalar can be added to anything. Either of X or Y can be also a double array.

% Copyright (C) 2004 by JUHASZ Laszlo

%y=b_add(a,b); return %temporary bypass

% ***** PREAMBLE *****
switch class(a)
   case 'qdata'
      if isempty(a.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsiza = size(a.data);
         %asize=size(a);
         %if (length(orsiza)==length(asize))&(orsiza(end)~=asize(end))
         %  orsiza(end+1)=orsiza(end); orsiza(end-1)=1;
         %end
         % calculating the number of segments in 'a'
         segna = orsiza(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsiza = orsiza(2:end); 
         if length(orsiza)==1, orsiza=[orsiza,1]; end %KI
         % convert to two dimension
         a.data = reshape(a.data,[segna,prod(orsiza)]);
      end
   case 'double'
      if isempty(a)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'a' argument to segments
         % save original size of 'a'
         orsiza = size(a);
         % generating segnum - it's always 3 in case of 'double'
         segna = 3;
         % convert 'a' to column vector
         a = a(:);
         % fill up the variable 'z' with segment-transformed values of 'a'
         for k = 1:length(a)
            z(:,k) = double2seg(a(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'a' and put back the data of 'z' to the
         % 'data' field
         a = [];
         a.data = z;
         a.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end

switch class(b)
   case 'qdata'
      if isempty(b.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsizb = size(b.data);
         % calculating the number of segments in 'b'
         segnb = orsizb(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsizb = orsizb(2:end);
         if length(orsizb)==1, orsizb=[orsizb,1]; end %KI
         % convert to two dimension
         b.data = reshape(b.data,[segnb,prod(orsizb)]);
      end
   case 'double'
      if isempty(b)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'b' argument to segments
         % save original size of 'b'
         orsizb = size(b);
         % generating segnum - it's always 3 in case of 'double'
         segnb = 3;
         % convert 'b' to column vector
         b = b(:);
         % fill up the variable 'z' with segment-transformed values of 'b'
         for k = 1:length(b)
            z(:,k) = double2seg(b(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'b' and put back the data of 'z' to the
         % 'data' field
         b = [];
         b.data = z;
         b.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end

%nulla ellenrzs
%if (iszeros(a.data))
if ~any(a.data(:))
  %a csak nulla, visszatrnk B-vel
    if isa(b,'qdata')
      if isa(a,'qdata')|isa(a,'embedded-fi'), y=qdata(b,a.name);
      else y = b;
      end
    else
      y=a; 
      code=qdspdemo('quantizers',a.name);
      y.data=seground(b.data,code);
      %y=qdata(y,y.name); %az eredmnyt kvantlni kellene
      
    end
    return
else
    %if ~any(b.data(:))
        %B csak null visszatrnk A-val
    %    y = a;
    %    return
    %end
end

% download quantizer properties - the first argument ('a') has a bigger
% priority.
if ~isempty(a.name)
   qcode = qdspdemo('quantizers',a.name);
elseif ~isempty(b.name)
   qcode = qdspdemo('quantizers',b.name);
else
   error('One of the inputs must have a quantizer name.');
end   
eval(qcode)

% extract values from structs - the name is unnecessary from now on
a = a.data;
b = b.data;

% scalar extension and check incorrect lengths
% 'a' and 'b' are both 2-dim arrays - compatibility with Matlab 5.3!!!
if prod(orsiza)==1
   % the 'a' object is scalar so its variable is a column vector
   % extend 'a' to the size of 'b'
   a = a(1:size(a,1),ones(size(b,2),1));
   orsiza = orsizb;
elseif prod(orsizb)==1
   % the same case with 'b'
   b = b(1:size(b,1),ones(size(a,2),1));
   orsizb = orsiza;
elseif ~isequal(orsiza,orsizb)
   error('Matrix dimensions must agree.')
end

orsiz(1:length(orsiza)) = orsiza;
orsiz(1:length(orsizb)) = orsizb;

% from now size(a,2)==size(b,2)

% balancing number of segments
if segna~=segnb
   if segna>segnb
      % extend number of rows of 'b' to the number of rows of 'a'
      b(segna,:) = 0;
   else
      % extend number of rows of 'a' to the number of rows of 'b'
      a(segnb,:) = 0;
   end
end

% from now size(a,1)==size(b,1) so size(a)==size(b)
% ***** end of PREAMBLE *****

% deciding real and imag operations
% as default both adding is not necessary
reop = 0; imop = 0;
% extract real and imag parts
rea = real(a);
reb = real(b);
ima = imag(a);
imb = imag(b);
% deciding which parts are nonzero
isare = any(rea(:));
isaim = any(ima(:));
isbre = any(reb(:));
isbim = any(imb(:));
% allocating space for the result
% 15 segments is enough big and the output length of addBFP is unpredictable by me
reres = zeros(15,size(a,2));
imres = zeros(15,size(a,2));

% decision from the necessary operations
if isaim | isbim
   % no imag part in either of the arguments
   imop = 1;
end
if isare | isbre
   % no real part in either of the arguments
   reop = 1;
end

if reop
   % adding real
   for k = 1:size(a,2)
      % calling core function with only one number at once, with segment size of
      % 22 and with segnum of size(a,1)
      t = addbfp(rea(:,k),reb(:,k),22,size(a,1)).'; 
      %t = addbfp(rea(:,k),reb(:,k),22,size(a,1),1).'; %multiply with info
      reres(1:length(t),k) = t;
   end
   % rounding
   reres_in=reres;
   [reres,of,uf,inv,warn,err] = seground(reres_in,qcode); 
   if ~isempty(warn)&0 %*%*%*%
     QC=qget('Qdsp','QCODE');
     indM1=findstr('MAXIMUM',QC);indM2=findstr('MINIMUM',QC); MAXC=QC(indM1:indM2-1)
     1;
   end
   error(err); warning(warn);
   ofs = of; ufs = uf; ins = inv;
end

if imop
   % adding imag
   for k = 1:size(a,2)
      % calling core function with only one number at once, with segment size of
      % 22 and with segnum of size(a,1)
      t = addbfp(ima(:,k),imb(:,k),22,size(a,1)).';
      imres(1:length(t),k) = t;
   end
   % rounding
   [imres,of,uf,inv,warn,err] = seground(imres,qcode); error(err); warning(warn);
   ofs = ofs | of; ufs = ufs | uf; ins = ins | inv;
end

% extend results to seven segments (six is the maximum output segment number of the core
% function)
reres(6,1) = 0; reres = reres(1:5,:);
imres(6,1) = 0; imres = imres(1:5,:);

% ***** TRAILER *****
% interleaving real and imag part
t = reres+i.*imres;
% cut number of segs to SEGNUM
t = t(1:SEGNUM,:);
% reshape to original size
t = reshape(t,[SEGNUM,orsiz]);

% generating output object
if ~isempty(t(:))
   y.name = NAME;
else
   y.name = '';
end
y.data = t;
y = class(y,'qdata');

% store statistical info
if any(ofs(:)) | any(ufs(:)) | any(ins(:))
   % reshape stat info variables into size of 'data' without leading
   % segment dimensions
   if size(orsiza)>1
     ofs = reshape(ofs,orsiza);
     ufs = reshape(ufs,orsiza);
     ins = reshape(ins,orsiza);
   end
   st = dbstack;
   % clear the top of the stack containing info about current function
   st(1) = [];
   % prevent from storing empty 'dbstack' struct with complete fieldnames
   % (in case when command line event occured and the former operation cleared
   % this)
   if isempty(st), st = []; end
   % fill up struct
   stat.dbstack = st;
   stat.ofmask = ofs;
   stat.ufmask = ufs;
   stat.invmask = ins;
   stat.arg1 = []; %v; %error???
   stat.arg2 = [];
   stat.result = y;
   % send to 'quantizers'
   qdspdemo('quantizers','event',stat);
end
