function y = bitinseg(exp,seg)
%BITINSEG          Returns the bit number in specified segment against actual exponent.
%  Y = BITINSEG(EXP,SEG) Returns that how many bit long the SEG segment is in
%        case of EXP exponent.
%
%  Plot of transfer function in case of SEG is 1:
%                EXP=-1053
%  Y=22 _____________
%                    \
%                     \
%                      \_______________ Y=0
%
%                    EXP=-1075
%
%  BITINSEG is used by STR2SEG_CORE and INTERN_DISP.
%  BITINSEG can process arrays too. EXP and SEG must have same size.

% Copyright (C) 2004 by JUHASZ Laszlo

error(nargchk(2,2,nargin))

% check input
if isempty(exp) | isempty(seg) | prod(size(exp))~=1 | prod(size(seg))~=1 | ...
      rem(exp,1) | rem(seg,1) | seg<1
   error('Internal error. Bad arguments.')
end   

if exp>=(-1031-22*seg)
   y = 22;
elseif exp<=(-1053-22*seg)
   y = 0;
else
   y = exp+1053+22*seg;
end
