function [mant,exp] = bitovfgen(sign,code)
%BITOVFGEN         Generates bit pattern for overflowed numbers.
%  [MANT,EXP] = BITOVFGEN(SIGN,CODE) generates clipped overflow pattern and/or
%     segments for fixed-point and floating-point numbers. 
%  Input arguments:
%     SIGN - 1 for positive and -1 for negative (only matters by CODING 'twos-comp'
%            and Inf)
%     CODE - is in the 'short-code' format what is the output of the quantizer-manager
%            core function 'quantizers'. The following properties have effect: TYPE,
%            CODING, PRECISION, FRACTION and OVERFLOW.
%  Output arguments:
%     MANT - Bit pattern vector containing only ones
%     EXP  - Exponent - scalar integer or +Inf

% Copyright (C) 2004 by JUHASZ Laszlo

error(nargchk(2,2,nargin))

% input check
if isempty(sign) | isempty(code) | ~isa(sign,'double') | ~ischar(code) | ...
      size(code,1)~=1 | prod(size(sign))~=1 | ~(sign==1 | sign==-1)
   error('Bad arguments.')
end

eval(code)

switch TYPE
case 'floating-point'
   switch OVERFLOW
   case 'inf'
      % assuming that pow2(1,Inf) is Inf
      mant = 1;
      mant = mant(1,ones(SEGNUM,1));
      exp = Inf;
   case 'clip'
      switch CODING
      case {'ones-comp' 'sign-mag'}
         % the simplest case -> fill with ones
         mant = ones(1,FRACTION+1);
         exp = EMAX;
      case 'twos-comp'
         if ~sign
            % positive numbers -> fill with ones
            mant = ones(1,FRACTION+1);
            exp = EMAX;
         else
            % negative numbers -> 1@(emax+1) is allowed
            mant = 1;
            exp = EMAX+1;
         end
      otherwise error(['Unsupported CODING ',CODING])
      end
   otherwise error(['Unsupported OVERFLOW ',OVERFLOW])
   end
case 'fixed-point'
   switch OVERFLOW
   case 'clip'
      switch CODING
      case 'unsign'
         mant = ones(1,PRECISION);
         exp = PRECISION-FRACTION-1;
      case 'ones-comp'
         mant = ones(1,PRECISION-1);
         exp = PRECISION-FRACTION-2;
      case 'twos-comp'
         if ~sign
            % positive numbers
            mant = ones(1,PRECISION-1);
            exp = PRECISION-FRACTION-2;
         else
            % negative numbers
            mant = 1;
            exp = PRECISION-FRACTION-1;
         end
      otherwise error(['Unsupported CODING ',CODING])
      end
   otherwise error(['Unsupported OVERFLOW ',OVERFLOW])
   end
otherwise error(['Unsupported TYPE ',TYPE])
end

% internal bit pattern check
if isempty(mant) | isempty(exp) | ~all(mant)
   error('Internal error at output check')
end
