function [mant,exp] = bitup(mant,exp,limit)
%BITUP             Bit rounding upwards.
%  BITUP Performs a binary incrementation with 1 at the position where rounding point
%  (LIMIT) has been specified. Incrementation only occurs when after the rounding point 
%  any 'one' is (minimum 1 piece).
%
%  Usage:
%  [MANT,EXP] = BITUP(MANT,EXP,LIMIT)
%  Arguments: MANT  - mantissa in vector form with ones and zeros
%             EXP   - exponent as decimal number, icrementation occurs when needed.
%             LIMIT - after which bit is the rounding point
%
%BITUP is used by 'bitround' and 'intern_display'

% Copyright (C) 2003, 2004 by JUHASZ Laszlo

error(nargchk(3,3,nargin))

% check input
if isempty(mant) | isempty(exp) | isempty(limit) | (~isa(mant,'double')&~isa(mant,'logical')) | ...
      ~isa(exp,'double') | ~isa(limit,'double') | prod(size(mant))~=length(mant) | ...
      prod(size(exp))~=1 | prod(size(limit))~=1 | ~all(isfinite(mant)) | ...
      ~isfinite(exp) | ~isfinite(limit) | any(rem(mant,1)) | rem(exp,1) | ...
      rem(limit,1) | ~all(mant==(~~mant))
   error('Internal error. Bad arguments.')
end

if limit >= length(mant), return, end  % filter out if the rounding point is behind mant
                                       % everything remains as it was

if limit < 1                           % if rounding point is before the whole mant
   if any(mant)
      % if mant has a one then the whole vector overflows before the new r.p.
      mant = 1;
      exp = exp-limit+1;
   else
      % if mant hasn't a one then the whole vector remain zero
      mant = 0;
      exp = 0;
   end
   return
end

if any(mant(limit+1:end))              % if number is greater than 0|000... or 1|000...
   if mant(limit)                         % number before r.point is 1
      % try to increment the bit before with recursive call and decrementation of 'limit'
      [mant,exp] = bitup(mant,exp,limit-1);
   else                                   % number before r.point is 0
      % simply set the bit before r.p.
      mant(limit) = 1;
      mant = mant(1:limit);
   end
end
