function s = decbin(d,n)
%DECBIN            Modified version of Matlab's dec2bin.
%   DECBIN(D) returns the binary representation of D as an
%   0-1 vector.
%   D must be a non-negative integer smaller than 2^52. 
%
%   DECBIN(D,N) produces a binary representation with at least
%   N bits.
%
%   Example
%      decbin(23) returns '10111'
%
%   See also BINDEC.
%
%   DECBIN is used by INTERN_DISPLAY.

% Copyright (C) 2004 by JUHASZ Laszlo

%
% Input checking
%
error(nargchk(1,2,nargin))
if isempty(d), s = ''; return, end

d = d(:); % Make sure d is a column vector.

if (nargin<2)
  n=1; % Need at least one digit even for 0.
else
  if prod(size(n))~=1 | n<0, error('N must be a positive scalar.'); end
  n = round(n); % Make sure n is an integer.
end;

%
% Actual algorithm
%
[f,e] = log2(max(d)); % How many digits do we need to represent the numbers?
s = rem(floor(d*pow2(1-max(n,e):0)),2);
