function y = double2seg(x)
%DOUBLE2SEG Converts a double number to segment form (3 segments)
%
%  Usage:
%     Y = DOUBLE2SEG(A)
%
%  Y will be a vector with size [1,3]. REAL(Y) gives back the 'real'
%  segments while IMAG(Y) gives back the 'imaginary' segments.

% Copyright (C) 2004 by JUHASZ Laszlo, corrected by IK

% emptyness check
if isempty(x)
   re = [];
   im = [];
   return
end

% NaN check
if isnan(x)
   n = nan;
   y = n(1,ones(3,1));
   return
end

if isinf(real(x))
   t = real(x);
   y(1,:) = t(1,ones(3,1));
else
   [f,e] = log2(real(x));
   y(1,1) = fix(f*2^22)/2^22;
   y(1,2) = fix((f-y(1,1))*2^44)/2^22;
   y(1,3) = rem(f*2^44,1);
   y = pow2(y,e);
end

if isinf(imag(x))
   % Inf in imag. part produces NaN according to Matlab's behaviour
   n = nan;
   y = n(1,ones(3,1));
   return
else
   [f,e] = log2(imag(x));
   y(2,1) = fix(f*2^22)/2^22;
   y(2,2) = fix((f-y(2,1))*2^44)/2^22;
   y(2,3) = rem(f*2^44,1);
   y(2,:) = pow2(y(2,:),e);
end

% finally compress the two rows of 'y' into one row
y = y(1,:)+j*y(2,:);

% internal check
if sum(pow2(real(y),[0:-22:-44])) + j*sum(pow2(imag(y),[0:-22:-44])) ~= x
   error(sprintf(['Internal error while transforming\n',...
         'double ''%g'' to\nsegments\nre\t''%g,%g,%g''\nim\t''%g,%g,%g'''],x,...
      real(y(1)),real(y(2)),real(y(3)),imag(y(1)),imag(y(2)),imag(y(3))))
end
