function y = groups(x,n,dir)
%GROUPS            Slice the input string into groups separated with spaces
%
%  Usage:
%     Y = GROUPS(X,N,DIR), where X is a string, N is the size of groups
%                          and direction has logical meaning such as 1 or 0.
%
%  Examples:
%   - groups('abcdefghijklmnopqrstubxyz',6,1) will be
%     'abcdef ghijkl mnopqr stubxy z'
%   - groups('abcdefghijklmnopqrstubxyz',6,0) will be
%     'a bcdefg hijklm nopqrs tubxyz'

% Copyright (C) 2003,2004 by JUHASZ Laszlo

% check inputs
error(nargchk(3,3,nargin))
if ~ischar(x) | ~prod(size(x))==length(x)
   error('Internal error. Bad string.')
end
if ~(isa(n,'double')|isa(n,'logical')) | ~isfinite(n) | rem(n,1) | n<0
   error('Internal error. Bad group size.')
end
if(n==0)
    y = x;
    return
end
dir = ~~dir;



y = '';
for k = 1:length(x)
   if dir
      % forward direction
      r = k;
   else
      % backward direction
      r = length(x)-k;
   end
   if rem(r,n)
      % not in separator position
      y = [y,x(k)];
   elseif k~=length(x)
      % separator position but not at end
      y = [y,x(k),' '];
   else
      % separator position at end -> not insert space
      % because then a space would be at the end
      y = [y,x(k)];
   end
end
