function y = intern_disp(x,code,gn)
%INTERN_DISP       Core function of '@qdata/disp.m' method.
%  Usage:
%     Y = INTERN_DISP(X,CODE,GN)
%  Input arguments:
%     X    - Segments of a number in row vector.
%     CODE - Quantizer properties in evaluateable string.
%     GN   - group number
%  Output argument:
%     Y - String containing human readable (? - in most cases) form of number.

% INTERN_DISPLAY performs minor format checks related to the
% storage of segments too.

% Copyright (C) 2003,2004 by JUHASZ Laszlo

% zero and one in temporary variable for 'repmat-like' indexing
nul = 0;
one = 1;

% check input
error(nargchk(3,3,nargin))
if isempty(x) | prod(size(x))~=length(x) | ~isa(x,'double') | ...
    isempty(gn) | prod(size(gn))~=length(gn) | ~isa(gn,'double')
  error('Internal error. Bad arguments.')
end

eval(code)
% filtering out common values
if isnan(x(1))                                  % NaN
  if ~all(isnan(x))
    error('Internal error. Inconsistent NaN segments.')
  end
  y = 'NaN';
  return
elseif isinf(x(1))                              % Inf
  if ~all(isinf(x))
    error('Internal error. Inconsistent Inf segments.')
  elseif ~all(sign(x)==sign(x(1)))
    error('Internal error. Inconsistent Inf signs.')
  elseif sign(x(1))==1
    y = 'Inf';
  elseif sign(x(1))==-1
    y = '-Inf';
  else
    error('Internal error. Invalid sign of Inf.')
  end
  return
elseif all(x==0)                                % zero
  %y = '  0';
  y=setstr('0'*ones(1,PRECISION));
  if FRACTIONBITS>0
    y=[y(1:end-FRACTIONBITS),'(.)',y(end-FRACTIONBITS+1:end)];
    for ii=fliplr(length(y)-FRACTIONBITS+5:4:length(y)-1), y=[y(1:ii-1),' ',y(ii:end)]; end
    ind=findstr('(.)',y);
    for ii=ind-4:-4:2, y=[y(1:ii-1),' ',y(ii:end)]; end
  else
    for ii=length(y)-4:-4:2, y=[y(1:ii),' ',y(ii+1:end)]; end
  end
  if strcmp(TYPE,'fixed-point')
    if strcmp(CODING,'sign-mag'), y=['+',y];
    else y=['0',y];
    end
  else
    if strncmp(LEADINGBIT,'h',1), y=['(',y(1),')',y(2:end)]; end
    y=['+',y];
    if rem(log2(abs(EMIN)+2),1)==0
      n2b=setstr('0'*ones(1,log2(abs(EMIN)+2)+1));
      for ii=fliplr(5:4:length(n2b)-1), n2b=[n2b(1:ii-1),' ',n2b(ii:end)]; end
    else
      n2b='';
    end
    y=[y,' (*2^)',n2b,sprintf(' [%.0fd]',EMIN)];
  end
  return
end

% generating char-form sign notation
% The 'min(find(x))' expression works also with physically denormalized numbers
% where the first segment(s) can be zero.
s = sign(x(min(find(x))));
if s==1
  csign = '+';
elseif s==-1
  csign = '-';
else
  error('Internal error 1')
end
x = abs(x);

% obtain mantissas and exponents
[f,e] = log2(x);

%correction on 2004-10-24 by RS%
%original code does not count in, that segment(i) had been changed
%by it's exponenet lowered by i*22
%i = (0:length(e)-1)*-22;
e_correcter = e - e(1);
%e_correcter = e_correcter + i;


% number of the first notnull segment
fs = min(find(f));
if f(1) & e(1)>=-1052
  % physically normalized numbers
  % correct mantissas (avoid automatic normalization)
  f = pow2(f,e_correcter);
  % main exponent
  e = e(1);
else
  % physically denormalized numbers
  % how many digits is in that segment?
  % the 'bitinseg' is calibrated for 'on-page' notation that's why e(fs)-1
  d = bitinseg(e(fs)-22*(fs-1)-1,fs);
  % exponent corrector values - It's 0 in the place of 1st notnull segment
  % and increments with 22 towards the lesser significant segments.
  expcorr = (-22*(fs-1):22:22*(size(f,2)-fs));
  % correct mantissas - the 1st notnull segment wouldn't be corrected
  f = pow2(f,e(fs)-e+expcorr);
  % main exponent
  e = e(fs)-22*(fs-1);
  % make sure f is enough long
  f(length(f)+1) = 0;
  % correct denormalized values
  for k = fs:length(f)-1
    % Add the slice from the next mantissa to the current mantissa
    % (for the first time the current mantissa is the 1st)
    % The size of slice is 22-d.
    % The upcorrector 22-d and fix selects the 22-d bits.
    % The downcorrector -22 transform slice AFTER the former
    % mantissa segment.
    f(k) = f(k) + pow2(fix(pow2(f(k+1),22-d)),-22);
    % slide up the remained slice
    % (for the first time the remained slice is the slice after the 1st)
    f(k+1) = rem(pow2(f(k+1),22-d),1);
  end
  f(end) = 0;
end

% scale 'mant' and 'exp' to conform to theoretical notation (like on paper)
% with 1.xxx mantissa
f = f*2;
e = e-1;
% for now mantissa is in 'mant' and the main exponent is in 'exp'

switch TYPE
  case 'floating-point'
    % ****** FLOATING-POINT NUMBERS ******
    % generating exponent in char form
    if strcmp(EXPCODING,'biased')
      % add EXPBIAS to logical exponent to obtain physical exponent
      EXPBITS=ceil(log2(EMAX-EMIN));
      cexp = groups(char(decbin(abs(e+EXPBIAS),EXPBITS)+'0'),gn,1);
    elseif e>=0
      % positive exponent - common to all format
      if strcmp(EXPCODING,'sign-mag'), cexp = '+'; else cexp = ''; end
      cexp = [cexp,groups(char(decbin(abs(e),EXPBITS)+'0'),gn,1)];
    else
      % negative exponent - diverse along formats
      % The max(e,EMIN) is needed to represent denormalized numbers
      % with EMIN as exponent and a downscaled mantissa.
      switch EXPCODING
        case 'ones-comp'
          % simply logical negation
          cexp = groups(char(~decbin(abs(max(e,EMIN)),EXPBITS)+'0'),gn,1);
        case 'twos-comp'
          % generating twos complement format like on paper: negating then adding 1
          % vector-form adding performed by 'bitup'
          cexp = bitup([~decbin(abs(max(e,EMIN)),EXPBITS),1],0,EXPBITS);
          % make sure 'cexp' has enough bits in case of 'bitup' has cleared them
          cexp(EXPBITS+1)=0;
          cexp = cexp(1:EXPBITS);
          % transforming to char
          cexp = groups(char(cexp+'0'),gn,1);
        case 'sign-mag'
          % putting out minus sign else equivalent with positive
          cexp = ['-',groups(char(decbin(abs(max(e,EMIN)),EXPBITS)+'0'),gn,1)];
        otherwise error(['Unsupported EXPCODING ',EXPCODING])
      end
    end
    cexp = strrep(cexp,' ',' '); %_???
    % generating mantissa
    switch CODING
      case 'sign-mag'
        % sign-magnitude mantissa like IEEE
        if e<EMIN
          % logical denormalized
          switch LEADINGBIT
            case 'hidden'
              y = [csign,'(0)(.)'];
            case 'shown'
              y = [csign,'0(.)'];
            otherwise error(['Unsupported LEADINGBIT ',LEADINGBIT])
          end
          % load mantissa leading zeros to 'yy'
          cnul = '0';
          yy = cnul(1,ones(EMIN-e,1));
        else
          % logical normalized
          switch LEADINGBIT
            case 'hidden'
              y = [csign,'(1)(.)'];
            case 'shown'
              y = [csign,'1(.)'];
            otherwise error(['Unsupported LEADINGBIT ',LEADINGBIT])
          end
          % there aren't mantissa zeros
          yy = '';
        end
        for k = fs:length(f)
          % putting the char-form of mant-segments to 'yy'
          % 'mant' needs only 21 bit upscale because it's in the form: 1.xxxx
          % so the 1st bit is already before the radix.
          yy = [yy,dec2bin(pow2(f(k),21),22)];
        end
        while length(yy)<PRECISION, yy = [yy,dec2bin(pow2(0,21),22)]; end
        % generating the full number string by concatenating 'y' with 'yy'
        % The first bit of 'yy' is stored already in 'y' with possible
        % brackets that't why the 2: expression.
        yyhid=yy(FRACTION+2:end);
        if all(ismember(yyhid,'0'))
          strhid='';
        else
          while(strcmp(yyhid(end),'0')), yyhid(end)=''; end
          strhid=[' (',groups([setstr('2'*ones(1,rem(FRACTION,gn))),yyhid],gn,1),')'];
          ind=findstr('2',strhid); if ~isempty(ind), strhid(ind)=''; end
        end
        y = deblank([y,groups(yy(2:FRACTION+1),gn,1),strhid,' (*2^)',cexp,' [',...
          num2str(max(e,EMIN)),'d]']);
      case 'twos-comp'
        % twos-complement mantissa like TMS320C4x
        yy = '';
        for k = 1:length(f)
          % concatenating all the segments in vector-form
          % Vector-form is needed because later vector-form adding
          % needs to be performed on negative mantissa.
          % 'decbin' produces vector-form istead of the char-form
          % of Matlab's 'dec2bin'.
          yy = [yy,decbin(pow2(f(k),21),22)];
        end
        if s==1
          % positive
          % decide denorm
          if e<EMIN
            yy = [nul(1,ones(EMIN-e+1,1)),yy];
          else
            yy = [0,yy];
          end
        else
          % filtering out special cases
          if yy(1)==1 & sum(yy)==1 & e<=EMIN
            % in case of twos-comp -1@emin is already denormalized
            yy = [one(1,ones(EMIN-e,1)),1,1];
          elseif yy(1)==1 & sum(yy)==1
            % normal one within border
            e = e-1;
            % regenerate cexp
            if strcmp(EXPCODING,'biased')
              % add EXPBIAS to logical exponent to obtain physical exponent
              cexp = groups(char(decbin(abs(e+EXPBIAS),EXPBITS)+'0'),gn,1);
            elseif e>=0
              % positive exponent - common to all format
              if strcmp(EXPCODING,'sign-mag'), cexp = '+'; else cexp = ''; end
              cexp = [cexp,groups(char(decbin(abs(e),EXPBITS)+'0'),gn,1)];
            else
              % negative exponent - diverse along formats
              % The max(e,EMIN) is needed to represent denormalized numbers
              % with EMIN as exponent and a downscaled mantissa.
              switch EXPCODING
                case 'ones-comp'
                  % simply logical negation
                  cexp = groups(char(~decbin(abs(max(e,EMIN)),EXPBITS)+'0'),gn,1);
                case 'twos-comp'
                  % generating twos complement format like on paper: negating then adding 1
                  % vector-form adding performed by 'bitup'
                  cexp = bitup([~decbin(abs(max(e,EMIN)),EXPBITS),1],0,EXPBITS);
                  % make sure 'cexp' has enough bits in case of 'bitup' has cleared them
                  cexp(EXPBITS+1)=0;
                  cexp = cexp(1:EXPBITS);
                  % transforming to char
                  cexp = groups(char(cexp+'0'),gn,1);
                case 'sign-mag'
                  % putting out minus sign else equivalent with positive
                  cexp = ['-',groups(char(decbin(abs(max(e,EMIN)),EXPBITS)+'0'),gn,1)];
                otherwise error(['Unsupported EXPCODING ',EXPCODING])
              end
            end
            cexp = strrep(cexp,' ','_');
          else
            % more than one pieces of one in mantissa
            % allocating place for the so-called 'sign' bit but it isn't really
            % decide denorm
            if e<EMIN
              yy = [nul(1,ones(EMIN-e+1,1)),yy];
            else
              yy = [0,yy];
            end
            yy(FRACTION+3) = 0;
            yy = ~yy;
            yy = bitup(yy,0,FRACTION+2);
            % restore original length
          end
        end
        yy(FRACTION+3) = 0;
        yy = yy(1:FRACTION+2);
        % converitng to char
        yy = char(yy+'0');
        % inserting the brackets posteriorly
        switch LEADINGBIT
          case 'hidden'
            yy = [yy(1),'(',yy(2),').',groups(yy(3:end),gn,1)];
          case 'shown'
            yy = [yy(1:2),'.',groups(yy(3:end),gn,1)];
          otherwise error(['Unsupported LEADINGBIT ',LEADINGBIT])
        end
        y = deblank([yy,' *2^(',cexp,')[',num2str(max(e,EMIN)),'d]']);
      otherwise error(['Unsupported CODING ',CODING])
    end
  case 'fixed-point'
    % ****** FIXED POINT NUMBERS ******
    y = '';
    % generating emax and emin
    EMAX = PRECISION+1-FRACTION-1;
    EMIN = -FRACTION;
    % collecting segment bits into 'y'
    for k = 1:length(f)
      y = [y,decbin(pow2(f(k),21),22)];
    end
    switch CODING
      case 'unsign'
        % unsigned number
        % generating zeros before the leading one
        y = char([nul(1,ones(EMAX-e,1)),y]+'0');
        % grouping the integer part in right-to-left order and the
        % fractional part in left-to-right order
        y = [groups(y(1:PRECISION-FRACTION),gn,0),'.',...
          groups(y(PRECISION-FRACTION+1:PRECISION),gn,1)];
      case 'ones-comp'
        % ones-complement number
        % transform sign in the form: 0 - positive, 1 - negative
        s = ~(s+1);
        % if s=1 (negative) then negate bits but leave unchanged if positive
        y = char(xor(s,[nul(1,ones(EMAX-e,1)),y])+'0');
        if strcmp(y(1),'1'), y(1)='o'; end
        %
        % grouping the integer part in right-to-left order and the
        % fractional part in left-to-right order
        y = [groups(y(1:PRECISION-FRACTION),gn,0),'.',...
          groups(y(PRECISION-FRACTION+1:PRECISION),gn,1)];
      case 'twos-comp'
        % twos-complement number
        if s==1
          % positive numbers
          % normal, like unsigned
          y = [char([nul(1,ones(EMAX-e,1)),y]+'0')];
          1;
        else
          % negative numbers
          % generating leading ones and negating mantissa
          y = ~[nul(1,ones(EMAX-e,1)),y];
          % make y enough long and make sure bitup
          y(PRECISION+2) = 1;
          y = bitup(y,0,PRECISION+1);
          % restore original length
          y(PRECISION+2) = 0;
          y = y(1:PRECISION+1);
          % transform to char-form
          y = char(y+'0');
          if strcmp(y(1),'1'), y(1)='t'; end
        end
        % grouping the integer part in right-to-left order and the
        % fractional part in left-to-right order
        y = [groups(y(1:PRECISION+1-FRACTION),gn,0),'.',...
          groups(y(PRECISION+1-FRACTION+1:PRECISION+1),gn,1)];
      otherwise
        error(['Unsupported CODING ',CODING])
    end
  otherwise
    error(['Unsupported TYPE ',TYPE])
end
