function [mant,expon] = num2binsegm(segms,L,MSB,prec)
%NUM2BINSEGM   Simple segment representation to binary string conversion.
%
%       [mant expon] = num2binsegm(segms,L)
%
% segms may be vector of a few segments, L is the number of bits in a segment.
% The result will have binary digits each in a number, minimum 3
% if there is imag part and real part of the input argument then the size
% will be : size(result) = [size(segms,1:end-1) L*semno 2]
%   First position: MSB
%   Last position:  LSB
% example(1)
%   num2binsegm(343,22) -> [[1 0 1 0 1 0 1 1 1 zeros(1,13)],-13]
% example(2)
%   num2binsegm(343,22,[],18) -> [[1 0 1 0 1 0 1 1 1 zeros(1,9)],-13]
%
% Written by Sndor Rvsz and Istvn Kollr

if nargin<4, prec=[]; end, if isempty(prec), prec=length(segms)*L; end
if nargin<3, MSB=[]; end
%First determining the sizes
ndimssegms = ndims(segms);
Sizesegms = size(segms);
N=Sizesegms(end);
%determining if there are any imag and real part
imagsegms = imag(segms);
realsegms = real(segms);

mant=zeros(1,min(Sizesegms(1:end-1)*Sizesegms(end)*L,prec));
expon=zeros(Sizesegms(1:end-1),1);

for ii=1:length(expon(:))
  %seg=segms((ii-1)*L+1); 
  seg=segms(ii);
  s=sign(seg);
  if ii==1, if isempty(MSB), MSB=pow2(floor(log2(abs(seg)))); end, end
  sub=s*MSB;
  for iL=1:L
    if s>0
      if sub<=seg, seg=seg-sub; mant((ii-1)*L+iL)=s; end
    else
      if sub>=seg, seg=seg-sub; mant((ii-1)*L+iL)=s; end
    end
    sub=sub/2;
  end %for iL
  expon(ii)=log2(MSB);
end %for ii
if any(diff(expon)), error('expon changes'), else expon=expon(1); end
%