function [mant,exp] = ovfgen(s,code)
%OVFGEN            Generates segments for overflowed numbers.
%  [MANT,EXP] = OVFGEN(SIGN,CODE) generates clipped or Inf segments for fixed-point and 
%      floating-point numbers. 
%  Input arguments:
%     SIGN - 0 for positive and 1 for negative (only matters by CODING 'twos-comp')
%            If it's matrix then the outputs will be also matrix with same size.
%     CODE - It's in the 'short-code' format what is the output of the quantizer-manager
%            core function 'quantizers'. The following properties have effect: TYPE,
%            CODING, PRECISION, FRACTION and OVERFLOW.
%  Output argument:
%     MANT - Mantissa matrix
%     EXP  - Exponent matrix

% Copyright (C) 2004 by JUHASZ Laszlo

eval(code)

% check for Inf overflow - only available by floating-point
if strcmp(OVERFLOW,'inf')
   % temp variable to allow multipying with function 'ones'
   v = Inf
   % generating temporary index matrix to multiply columns of 's' with SEGNUM
   t = 1:size(s,2)
   % and multiply 's' with converting to real sign values
   ss = (-1).^s(:,t(ones(SEGNUM,1),:))
   % generating 'mant' with multiply with 'ss'
   mant = ss.*v(ones(size(s,1),SEGNUM*size(s,2)))
   % generating 'exp' - exp is always positive Inf apart from 'sign'
   exp = v(ones(size(s)))
   return
end

fullone = sum(pow2(-22:-1))

switch TYPE
case 'floating-point'
   partone = sum(pow2(-(FRACTION+1-(22*(SEGNUM-1))):-1))
   % only clip overflow remains
   switch CODING
   case {'ones-comp' 'sign-mag'}
      % the simplest case -> fill with ones : 1.111...@EMAX = 0.111...@EMAX+1
      % generating pattern - row vector with segments of one number
      % SEGNUM-1 piece of fullone and one piece of partone
      pat = [fullone(:,ones(1,SEGNUM-1)),partone]
      % generating temporary index matrix to multiply columns of 's' with SEGNUM
      t = 1:size(s,2)
      % and multiply 's' with converting to real sign values
      ss = (-1).^s(:,t(ones(SEGNUM,1),:))
      % generating temporary index matrices to multiply 'pat'
      t = 1:SEGNUM
      % and multiply 'pat' and also multiply with 'ss' sign matrix to generate 'mant'
      mant = ss.*pat(ones(size(s,1),1),t(ones(size(s,2),1),:)')
      % generating 'exp'
      exp = (EMAX+1)*ones(size(s))
   case 'twos-comp'
      % positive numbers -> fill with ones : 1.111...@EMAX = 0.111...@EMAX+1
      % generating pattern - row vector with segments of one positive number
      % SEGNUM-1 piece of fullone and one piece of partone
      pospat = [fullone(:,ones(1,SEGNUM-1)),partone]
      % negative numbers -> 2@EMAX = 1@EMAX+1 = 0.1@EMAX+2
      % generating pattern - row vector with segments of one negative number
      % one piece of 0.5 and SEGNUM-1 piece of zeros
      negpat = [0.5,zeros(1,SEGNUM-1)]
      % generating temporary index matrix to multiply columns of 's' with SEGNUM
      t = 1:size(s,2)
      % and multiply 's'
      ss = s(:,t(ones(SEGNUM,1),:))
      % generating temporary index matrices to multiply 'pospat' and 'negpat'
      t = 1:SEGNUM
      % generating 'mant'
      mant = (-1).^ss.*((~ss).*pospat(ones(size(s,1),1),t(ones(size(s,2),1),:)')+ss.*negpat(ones(size(s,1),1),t(ones(size(s,2),1),:)'))
      % generating 'exp'
      exp = (EMAX+1)*ones(size(s)).*(~s)+(EMAX+2)*ones(size(s)).*s
   otherwise error(['Internal error. Unsupported CODING ',CODING])
   end
case 'fixed-point'
   partone = sum(pow2(-(PRECISION-(22*(SEGNUM-1))):-1))
   switch OVERFLOW
   case 'clip'
      switch CODING
      case 'unsign'
         % the simplest case -> fill with ones : 0.111...
         % generating pattern - row vector with segments of one number
         % SEGNUM-1 piece of fullone and one piece of partone
         pat = [fullone(:,ones(1,SEGNUM-1)),partone]
         % generating temporary index matrices to multiply 'pat'
         t = 1:SEGNUM
         % and generating 'mant' with multiply 'pat'
         mant = pat(ones(size(s,1),1),t(ones(size(s,2),1),:)')
         % generating 'exp' with temp variable 'v'
         v = PRECISION-FRACTION
         exp = v(ones(size(s)))
      case 'ones-comp'
         % the simplest case -> fill with ones : 0.111...
         % generating pattern - row vector with segments of one number
         % SEGNUM-1 piece of fullone and one piece of partone
         pat = [fullone(:,ones(1,SEGNUM-1)),partone]
         % generating temporary index matrix to multiply columns of 's' with SEGNUM
         t = 1:size(s,2)
         % and multiply 's' with converting to real sign values
         ss = (-1).^s(:,t(ones(SEGNUM,1),:))
         % generating temporary index matrices to multiply 'pat'
         t = 1:SEGNUM
         % and multiply 'pat' and also multiply with 'ss' sign matrix to generate 'mant'
         mant = ss.*pat(ones(size(s,1),1),t(ones(size(s,2),1),:)')
         % generating 'exp' with temp variable 'v'
         v = PRECISION-FRACTION-1
         exp = v(ones(size(s)))
      otherwise error(['Internal error. Unsupported CODING ',CODING])
      end
   otherwise error(['Internal error. Unsupported OVERFLOW ',OVERFLOW])
   end
otherwise error(['Internal error. Unsupported TYPE ',TYPE])
end
