function [exp2,err] = procexp(exp)
%PROCEXP           Process exponent.
%  Usage: [EXP,ERR] = PROCEXP(EXP)
%  Input argument: string containing exponent.
%  Output arguments:
%     EXP - scalar decimal exponent
%     ERR - error message

% Copyright (C) 2004 by JUHASZ Laszlo

error(nargchk(1,1,nargin))

exp2 = []; err = '';

if ~ischar(exp) | (~isempty(exp) & size(exp,1)~=1)
   error('Bad arguments.')
end

if isempty(exp)
   err = 'Empty exponent.';
   return
end

% filter out zeros
if all(exp=='0')
   exp2 = 0;
   return
end

format = exp(end-1);
exp(end) = '';

exp = strrep(exp, '[', '');
exp = strrep(exp, ']', '');

zaro = findstr(exp,')');
exp=exp(zaro+1:end);

switch format
case 'd'                            % decimal
   % check
   if isempty(exp)
      err = 'Missing exponent before decimal format indicator.';
      return
   end
   err = strchk(exp,'0123456789^()d','decimal exponent');
   if ~isempty(err), return, end
end

exp2 = exp;
