function [sign,mant2,expcorr,err] = procmant(mant)
%PROCMANT          Process mantissa.
%  Usage: [SIGN,MANT,EXPCORR,ERR] = PROCMANT(MANT)

% Copyright (C) 2004 by JUHASZ Laszlo

error(nargchk(1,1,nargin))

sign = []; mant2 = []; expcorr = []; err = '';

if ~ischar(mant) | (~isempty(mant) & size(mant,1)~=1)
   error('Bad arguments.')
end

if isempty(mant)
   err = 'Empty mantissa.';
   return
end

% filter out zeros
if all(mant=='0')
   sign = 1;
   mant2 = 0;
   expcorr = 0;
   return
end

format = mant(end);
mant(end) = '';

switch format
case 'b'                            % binary
   % check
   if isempty(mant)
      err = 'Missing mantissa before binary format indicator.';
      return
   end
   err = strchk(mant,'.+-0123456789/\()','binary mantissa');
   if ~isempty(err), return, end
   mant(ismember(mant,'()'))='';
   % replace / and \
   [mant,err] = strreplace(mant);
   if ~isempty(err), return, end
   err = strchk(mant,'.+-01/\','binary mantissa');
   if ~isempty(err), return, end
case 'x'                            % hexadecimal
   % check
   if isempty(mant)
      err = 'Missing mantissa before hexadecimal format indicator.';
      return
   end
   err = strchk(mant,'.+-0123456789abcdef/\','hexadecimal mantissa');
   if ~isempty(err), return, end
   % replace / and \
   [mant,err] = strreplace(mant);
   if ~isempty(err), return, end
   % replace
   mant = strrep(mant, '0', '0000');
   mant = strrep(mant, '1', '0001');
   mant = strrep(mant, '2', '0010');
   mant = strrep(mant, '3', '0011');
   mant = strrep(mant, '4', '0100');
   mant = strrep(mant, '5', '0101');
   mant = strrep(mant, '6', '0110');
   mant = strrep(mant, '7', '0111');
   mant = strrep(mant, '8', '1000');
   mant = strrep(mant, '9', '1001');
   mant = strrep(mant, 'a', '1010');
   mant = strrep(mant, 'b', '1011');
   mant = strrep(mant, 'c', '1100');
   mant = strrep(mant, 'd', '1101');
   mant = strrep(mant, 'e', '1110');
   mant = strrep(mant, 'f', '1111');
case 'o'                            % octal
   % check
   if isempty(mant)
      err = 'Missing mantissa before octal format indicator.';
      return
   end
   err = strchk(mant,'.+-0123456789/\','octal mantissa');
   if ~isempty(err), return, end
   % replace / and \
   [mant,err] = strreplace(mant);
   if ~isempty(err), return, end
   err = strchk(mant,'.+-01234567/\','octal mantissa');
   if ~isempty(err), return, end
   % replace
   mant = strrep(mant, '0', '000');
   mant = strrep(mant, '1', '001');
   mant = strrep(mant, '2', '010');
   mant = strrep(mant, '3', '011');
   mant = strrep(mant, '4', '100');
   mant = strrep(mant, '5', '101');
   mant = strrep(mant, '6', '110');
   mant = strrep(mant, '7', '111');
otherwise
   err = ['Incorrect format indicator ''',format,''' at the end of mantissa.'];
   return
end

% analizing sign
signpos = find(mant=='+' | mant=='-');
if isempty(signpos)                          % no sign character
   sign = 1;                                 % sign is plus
elseif length(signpos)>1
   err = ['More than one ''',mant(signpos),''' sign characters in mantissa.'];
   return
elseif signpos~=1
   err = ['Sign isn''t at start of mantissa.'];
   return
elseif signpos==length(mant)
   err = 'Mantissa is only a sign character.';
   return
elseif all(mant(signpos+1:end)=='.')
   err = 'Mantissa contains only comma(s) after sign.';
   return
else
   if mant(1) == '+'
      sign = 1;
   elseif mant(1) == '-'
      sign = -1;
   else
      error('Internal error 1')
   end
   mant(1) = '';                    % cutting sign from start
end

% analizing comma position
commapos = findstr(mant,'.');      % commapos is the position before what the comma is.
mant(commapos)='';
if isempty(commapos)
   commapos = length(mant)+1;
elseif length(commapos)>1
   err = 'More than one commas (.) in mantissa.';
   return
elseif isempty(mant)
   err = 'Mantissa is only one comma.';
   return
end

% convert to numeric format
mant = double(mant-'0');
expcorr = commapos-2;
[mant,expcorr] = rmnull(mant,expcorr);

mant2 = mant;
