function y = qdata(varargin)
%QDATA  Constructor function of the @qdata quantized object.
%
%   For information about the format of numbers that the constructor can handle, 
%   type 'help qformats'. For the help on QDSP, type 'help qdsp' or 'qdspstart', 
%   for a help on the methods, type 'help(qdata)'.
%
%   Usage (N is a string, the name of quantizer):
%   1) Calling with double scalar or array
%      Y = QDATA(D,N) where D is a double scalar or array. The size of resulting object 
%      will be the same as D. Multidimensional double arrays are supported as well.
%   2) Calling with a string
%      Y = QDATA(S,N) where S is a string such as '2x'. The resulting object will be a scalar.
%      If the string is missing, duble precision will be assumed. Empty
%      string causes to extract a quantizer 'FROMSTRING' from the string 
%   3) calling with cell array of strings
%      Y = QDATA(C,N) where C is a cell array of strings
%      such as {'1x','2x';'3x','4x'}.
%      The size of resulting object will be the same as C. 
%      Multidimensional cell arrays are supported as well.
%
%   See also QDATA/DISPLAY, QDATA/PLUS, QDATA/TIMES.

%   Written by Lszl Juhsz, 2003-2004.
%   Modified by Istvn Kollr, 3-May-2006

%   3) calling with multiple strings
%      Y = QDATA(S1,S2,...,N) where S1, S2 and all the other
%      arguments are strings pruduces an 'column-object' so the size of the
%      object will be [X,1] where X is the number of arguments.

warn='';
err='';
if nargin==0
   % called without argument
   y = emptyclass;
   y = class(y,'qdata');
   return
end

if nargin==1
  name=varargin{1};
  if isstruct(name)
    %name-ban az adatok amit mg kvantlni kell
    [code,err] = qdspdemo('quantizers','get',name.name); error(err);
    [code,err] = qdspdemo('quantizers',name.name); error(err);
    a = name.data;
    orsiza = size(a);
    SEGNUM = orsiza(1);
    orsiza = orsiza(2:end);
    a = reshape(a,[SEGNUM,prod(orsiza)]);
    eval(code);
    [re_name,of,uf,in,warn,err] = seground(real(a),code);
    [im_name,of,uf,in,warn,err] = seground(imag(a),code);
    a = re_name + j*im_name;
    if(isdscalar(orsiza))
      orsiza = [orsiza 1];
    end
    a = reshape(a,[SEGNUM,orsiza]);
    if strcmp(OPERATION,'quantize')
      name.data = a;
    else %PQN
      if ~isequal(a,name.data)
        q=lsb(name,qdata);
        noise=qdata(q*rand(1,1),'DefaultBigFPQ');
        y=class(name,'qdata');
        xq=struct(noise+y); xq.name=name.name;
        name=xq;
        %error('Not yet ready...')
      end
    end
    y=class(name,'qdata');
    return
  end
end

%Make name
if (isquant(varargin{1})|isempty(varargin{1}))&~isquant(varargin{end})
  %old call
  name=varargin{1}; varargin(1)=[];
  warning('Old-form call is detected: qdata(qname,var). Change to qdata(var,qname).')
elseif isquant(varargin{end})
  name=varargin{end}; varargin(end)=[];
elseif nargin==1
  [code,err] = qdspdemo('quantizers','get','-all'); error(err);
  %if isempty(code)|~isquant('IEEE_double'), qmake('IEEE_double'), end
  if isa(varargin{1},'qdata'), name=varargin{1}.name;
  else name='IEEE_double';
  end
elseif (nargin==2)&isstr(varargin{1})&isstr(varargin{2})&isempty(varargin{2})
  qmake('FROMSTRING',varargin{1});
  name='FROMSTRING'; varargin(2)=[];
elseif (nargin==2)&isstr(varargin{2})&strcmp(varargin{2},'DefaultBigFPQ')&~isquant('DefaultBigFPQ')
%  qmake('name','BigFPQ','precision',110);
  name='BigFPQ';
end

% obtaining quantizer properties
% the 'get' function of 'quantizers' has error correction so double call required
[code,err] = qdspdemo('quantizers','get',name); error(err)
% name is OK, we can call the unchecked fast version of function
[code,err] = qdspdemo('quantizers',name); error(err)
eval(code);

if (nargin==1)&isquant(varargin{1})
   % called with one argument - the quantizer name without data
   y = class(emptyclass,'qdata');
   return
end

if nargin>2
   % only chance to use multiple arguments is calling with multiple strings
   for k = 1:length(varargin)
      if ~ischar(varargin{k})
         error('Multiple arguments must be all strings.')
      elseif prod(size(varargin{k}))~=length(varargin{k})
         % filtering out strings with e.g. line-break
         error('Multiple strings must be all simple.')
      end
   end
   for k = 1:length(varargin)
      if isempty(varargin{k})
         warning(['Size of input has been probably modified ',...
               'due to deleted empty element.'])
         % delete empty strings
         varargin(k) = [];
      end
   end
   % the whole 'varargin' goes to 'v'
   v = varargin;
   % save size
   siz = size(v);
else
   % chances are: only one string, cell array of strings, double scalar or array
   if isempty(varargin)
      % if the only argument is an empty cell or something
      y = class(emptyclass,'qdata');
      return
   elseif prod(size(varargin))~=1
      error('Internal error 1')
   end
   % the "inside" of 'varargin' goes to 'v'
   v = varargin{1};
   switch class(v)
   case 'char'
      if prod(size(v))~=length(v)
         % filtering out string with e.g. line-break
         error('String must be simple.')
      end
      siz = [1 1];
      % convert to cell
      v = {v};
   case 'double'
      siz = size(v);
   case 'cell'
      for k = 1:prod(size(v))
         if ~ischar(v{k})
            error('Cell matrices can contain only strings.')
         elseif prod(size(v{k}))~=length(v{k})
            % filtering out strings with e.g. line-break
            error('Strings in cell must be all simple.')
         end
      end
      for k = 1:prod(size(v))
         if isempty(v{k})
            if prod(size(v))~=length(v)
               % if the cell array isn't a vector then we couldn't delete the
               % empty element without upsetting the layout of the array
               error('Non-vector cell array contains empty element.')
            else
               warning(['Size of input has been probably modified ',...
                     'due to deleted empty element.'])
            end
            % delete empty strings
            v(k) = [];
         end
      end
      siz = size(v);
     case 'qdata' %reassign quantizer
       if nargin==1, y=v; return
       elseif (nargin==2)&isstr(name)&isquant(name)
         %y=qdata(v.data,name);
         y=v; y.name=name; 
         y=y+0; %remake data to requantize
         return
       else
         error(['Cannot reassign new quantizer ''',varargin{2},''' to qdata'])
       end
     otherwise
      error(['Unsupported input type ''',class(v),'''.'])
   end
end

% if 'v' has contained only empty chars/doubles and these have been erased...
if isempty(v)
   y = class(emptyclass,'qdata');
   return
end

% convert double arrays to colum-vector and cell arrays to column-cell
v = v(:);
% initialize output
z = zeros(SEGNUM,length(v));
% initialize statistic variables
ofs = zeros(1,length(v));         % overflow index vector
ufs = zeros(1,length(v));         % underflow index vector
ins = zeros(1,length(v));         % 'unallowed NaN' index vector

switch class(v)
case 'cell'
         for k = 1:length(v)
          % transforming and rounding with 'str2seg' or 'str2seg_kp'
          
          nyito = findstr(v{k},'[');
          csuko = findstr(v{k},']');
          hatv1 = findstr(v{k},'*2^');
          hatv2 = findstr(v{k},'(2^)');
   
   if (~isempty(nyito) && ~isempty(csuko))
       if isempty(hatv1)
           err = 'Missing raising to the second power.';
           return
       end
   end
   
   if (isempty(nyito) && ~isempty(csuko))
        err = 'Missing ''['' character.';
      return
   end
   
  if (~isempty(nyito) && isempty(csuko))
        err = 'Missing '']'' character.';
      return    
  end
         
      if (isempty(nyito) && isempty(csuko) && isempty(hatv1) && isempty(hatv2))
         [segs,of,uf,inv,warn,err] = str2seg(v{k},code);
         if ~isempty(err), error(['At the number ''',v{k},''': ', err]), end
         if ~isempty(warn), warning(['At the number ''',v{k},''': ',warn]), end
        % saving of, uf and inv values into 'ofs', 'ufs' and 'ins' index vectors
        ofs(k) = of;
        ufs(k) = uf;
       ins(k) = inv;
       % put back complex data into 'z'
       % transpose operator (.') is needed instead of ctranspose (')
       z(:,k) = segs.';
      end
     
      if (~isempty(nyito) && ~isempty(csuko) && ~isempty(hatv1) )
            v2=cellstr(v);
            v3=char(v2);
           bin2numv3=bin2num(v3,'double');
           y = qdata(bin2numv3,NAME);
           return;
      end
      
      %% A SZILGYI LACI-FLE STRINGET GYIS CSAK MEGKTSEKKEL TUDNNK FELDOLGOZNI... 
      
% % % % % %        if (~isempty(nyito) && ~isempty(csuko) && ~isempty(hatv1) && isempty(hatv2))
% % % % % %           %code
% % % % % %           [y,warn,err] = str2seg_kp(v{k},code);
% % % % % %        %[segs,of,uf,inv,warn,err] = str2seg_kp(v{k},code);
% % % % % %        if ~isempty(err),   error(['At the number ''',v{k},''': ', err]), end
% % % % % %        if ~isempty(warn),warning(['At the number ''',v{k},''': ',warn]), end
% % % % % %          % saving of, uf and inv values into 'ofs', 'ufs' and 'ins' index vectors
% % % % % %          %ofs(k) = of;
% % % % % %          %ufs(k) = uf;
% % % % % %          %ins(k) = inv;
% % % % % %          % put back complex data into 'z'
% % % % % %          % transpose operator (.') is needed instead of ctranspose (')
% % % % % %          %z(:,k) = segs.';
% % % % % %          return
% % % % % %        end

         end    % Ez a case 'cell' end-je

case 'double'
   for k = 1:length(v)
      % convert double to segments
      t = double2seg(v(k)).';
      % rounding and collecting stat info - real part
      [re,of,uf,in,warn,err] = seground(real(t),code);
      if ~isempty(err),   error(['Error at the number ',num2str(v(k)),': ', err]), end
      if ~isempty(warn),warning(['At the number ',num2str(v(k)),': ',warn]), end
      % saving of, uf and inv values into 'ofs', 'ufs' and 'ins' index vectors
      ofs(k) = ofs(k)|of;
      ufs(k) = ufs(k)|uf;
      ins(k) = ins(k)|in;
      % rounding and collecting stat info - imag part
      [im,of,uf,in,warn,err] = seground(imag(t),code);
      if ~isempty(err),   error(['Error at the number ',num2str(v(k)),': ', err]), end
      if ~isempty(warn),warning(['At the number ',num2str(v(k)),': ',warn]), end
      % saving of, uf and inv values into 'ofs', 'ufs' and 'ins' index vectors
      ofs(k) = ofs(k)|of;
      ufs(k) = ufs(k)|uf;
      ins(k) = ins(k)|in;
      % put back complex data into 'z'
      % transpose operator is needed instead of ctranspose
      z(:,k) = (re+j*im);
   end
otherwise
   error('Internal error 2')
end

% restore size of 'z' with leading segment dimensions
z = reshape(z,[SEGNUM,siz]);
% make object from 'z'
y.name = name;
y.data = z;
y = class(y,'qdata');
if strcmp(qget(name,'operation'),'PQN')
  %error('PQN is not yet ready (under debugging at this moment...)')
end

% store statistical info
if any(ofs(:)) | any(ufs(:)) | any(ins(:))
   % reshape stat info variables into size of 'data' without leading
   % segment dimensions
   ofs = reshape(ofs,siz);
   ufs = reshape(ufs,siz);
   ins = reshape(ins,siz);
   st = dbstack;
   % clear the top of the stack containing info about current function
   st(1) = [];
   % prevent from storing empty 'dbstack' struct with complete fieldnames
   % (in case when command line event occured and the former operation cleared
   % this)
   if isempty(st), st = []; end
   % fill up struct
   stat.dbstack = st;
   stat.ofmask = ofs;
   stat.ufmask = ufs;
   stat.invmask = ins;
   stat.arg1 = v;
   stat.arg2 = [];
   stat.result = y;
   % send to 'quantizers'
   qdspdemo('quantizers','event',stat);
end
%