function out = rand(obj,mod)
%RAND  Uniformly distributed quantized data
%
%       out = rand(obj,mod)
%       obj: object describing random number
%       mod: 'range' - [xmin,xmax], else [0,1]

if nargin<2, mod=''; end
data=obj.data;
prec=qget(obj.name,'precision');
if strcmp(mod,'range')
  xmax=get(obj,'maximum');
  if isempty(xmax), xmax=pow2(2-2^(-prec+1),Emax); end
  xmin=get(obj,'minimum');
  if isempty(xmin), xmin=-pow2(2-2^(-prec+1),Emax); end
  xmean=(xmax/2+xmin/2);
  xscale=((1-2^(-21))*xmax(1)/2-(1-2^(-21))*xmin(1)/2);
  %xscale=xscale/4; %max random number: 4*sigma RANDN/RAND
elseif isnumeric(mod)&(size(mod)==[1,2])
  xmean=sum(mod/2); xscale=diff(mod/2);
elseif isempty(mod)
  xmean=0; xscale=1;
else error(['mod = ''',mod,''' is not allowed'])
end
out.data=inf;
while any(~isfinite(out.data))
  data=xscale*rand(size(data(:,:,:,:))+xmean/xscale); %RANDN/RAND
  out=qdata(data,obj.name,'data');
end

%if prec>53
%  out=out+pow2(xscale,-53)*rand(size(obj.data(1,:,:,:)));
%end
%if prec>106
%  out=out+pow2(xscale,-106)*rand(size(obj.data(1,:,:,:)));
%end
