function y = rdivide(a,b)
%RDIVIDE(./)     Array divide of @qdata objects.
%     A./B denotes element-by-element dividing.  A and B
%     must have the same dimensions unless one is a scalar.
%     A scalar can be multiplied into anything.
%     Either of X or Y can be also a double array.
%     C = RDIVIDE(A,B) is called for the syntax 'A ./ B' when A or B is an
%     object.
%
%     Copyright (C) 2004 by JUHASZ Laszlo and Rvsz Sndor
%
%     See also LDIVIDE, MLDIVIDE, MRDIVIDE.



% PREDEFINES
%global log_depth_debug log_depth_watch log_depth_easy log_depth_none;
%setlogdepth(log_depth_debug);

if(a==0)
    y = a;
    return
end

if(b==0)
    error('ERROR:QDSP:QDATA:RDIVIDE:Divide by zero.');
end

% ***** PREAMBLE *****
switch class(a)
   case 'qdata'
      if isempty(a.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsiza = size(a.data);
         % calculating the number of segments in 'a'
         segna = orsiza(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsiza = orsiza(2:end);
         % convert to two dimension
         a.data = reshape(a.data,[segna,prod(orsiza)]);
      end
   case 'double'
      if isempty(a)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'a' argument to segments
         % save original size of 'a'
         orsiza = size(a);
         % generating segnum - it's always 3 in case of 'double'
         segna = 3;
         % convert 'a' to column vector
         a = a(:);
         % fill up the variable 'z' with segment-transformed values of 'a'
         for k = 1:length(a)
            z(:,k) = double2seg(a(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'a' and put back the data of 'z' to the
         % 'data' field
         a = [];
         a.data = z;
         a.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end

switch class(b)
   case 'qdata'
      if isempty(b.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsizb = size(b.data);
         % calculating the number of segments in 'b'
         segnb = orsizb(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsizb = orsizb(2:end);
         % convert to two dimension
         b.data = reshape(b.data,[segnb,prod(orsizb)]);
      end
   case 'double'
      if isempty(b)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'b' argument to segments
         % save original size of 'b'
         orsizb = size(b);
         % generating segnum - it's always 3 in case of 'double'
         segnb = 3;
         % convert 'b' to column vector
         b = b(:);
         % fill up the variable 'z' with segment-transformed values of 'b'
         for k = 1:length(b)
            z(:,k) = double2seg(b(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'b' and put back the data of 'z' to the
         % 'data' field
         b = [];
         b.data = z;
         b.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end
valami = 2;
if(valami==1)
    disdghjasdghjsj=1;
end

% download quantizer properties - the first argument ('a') has a bigger
% priority.
if ~isempty(a.name)
   qcode = qdspdemo('quantizers',a.name);
elseif ~isempty(b.name)
   qcode = qdspdemo('quantizers',b.name);
else
   error('One of the inputs must have a quantizer name.');
end   
eval(qcode)

% extract values from structs - the name is unnecessary from now on
a = a.data;
%outlog('a.data',log_depth_watch);
%outlog(a,log_depth_watch);
b = b.data;
%outlog('b.data',log_depth_watch);
%outlog(b,log_depth_watch);

% scalar extension and check incorrect lengths
% 'a' and 'b' are both 2-dim arrays - compatibility with Matlab 5.3!!!
% scalar extension and check incorrect lengths
% 'a' and 'b' are both 2-dim arrays - compatibility with Matlab 5.3!!!
if prod(orsiza)==1
   % the 'a' object is scalar so its variable is a column vector
   % extend 'a' to the size of 'b'
   a = a(1:size(a,1),ones(size(b,2),1));
   orsiza = orsizb;
elseif prod(orsizb)==1
   % the same case with 'b'
   b = b(1:size(b,1),ones(size(a,2),1));
   orsizb = orsiza;
elseif ~isequal(orsiza,orsizb)
   error('Matrix dimensions must agree.')
end

orsiz(1:length(orsiza)) = orsiza;
orsiz(1:length(orsizb)) = orsizb;

% from now size(a,2)==size(b,2)

% balancing number of segments
if segna~=segnb
   if segna>segnb
      % extend number of rows of 'b' to the number of rows of 'a'
      b(segna,:) = 0;
   else
      % extend number of rows of 'a' to the number of rows of 'b'
      a(segnb,:) = 0;
   end
end



% from now size(a,1)==size(b,1) so size(a)==size(b)
% ***** end of PREAMBLE *****




% deciding real and imag operations

%       re(a)*re(b)-im(a)*im(b)    re(b)*im(a)-re(a)*im(b)
%res =  -----------------------  + -----------------------
%          re(b)^2-im(b)^2            re(b)^2-im(b)^2
%
%
%
%

% re(a)*re(b)-im(a)*im(b) + j*(re(a)*im(b)+re(b)*im(a))
do_arbr = 1;  do_aibi = 1; do_arbi = 1; do_aibr = 1;
do_rebsquare = 1; do_imbsquare = 1;
rea = real(a);
reb = real(b);
ima = imag(a);
imb = imag(b);

has_real_a_not_null = any(rea(:));
has_imag_a_not_null = any(ima(:));
has_real_b_not_null = any(reb(:));
has_imag_b_not_null = any(imb(:));

% restrict the set of necessary multiplications
if ~has_real_a_not_null, do_arbr = 0; do_arbi = 0; end
if ~has_imag_a_not_null, do_aibi = 0; do_aibr = 0; end
if ~has_real_b_not_null, do_arbr = 0; do_aibr = 0; do_rebsquare = 0; end
if ~has_imag_b_not_null, do_aibi = 0; do_arbi = 0; do_imbsquare = 0; end
%outlog('do_arbr',log_depth_watch);
%outlog(do_arbr,log_depth_watch);

%outlog('do_aibi',log_depth_watch);
%outlog(do_aibi,log_depth_watch);

%outlog('do_arbi',log_depth_watch);
%outlog(do_arbi,log_depth_watch);

%outlog('do_aibr',log_depth_watch);
%outlog(do_aibr,log_depth_watch);

%outlog('do_rebsquare',log_depth_watch);
%outlog(do_rebsquare,log_depth_watch);

%outlog('do_imbsquare',log_depth_watch);
%outlog(do_imbsquare,log_depth_watch);


if(has_real_a_not_null & has_real_b_not_null & ~has_imag_a_not_null & ~has_imag_b_not_null)
    %csak egyetlen egy oszts real/real
    reres =  zeros(1+size(rea,1),size(rea,2));
    imres =  zeros(1+size(rea,1),size(rea,2));
    for k = 1:size(rea,2)
        tempor =  osztas(rea(:,k),reb(:,k),22,size(rea,1)).';
        reres(:,k) = tempor;
    end
    %outlog('reres',log_depth_easy);
    %outlog(reres,log_depth_easy);
else
    if(~has_real_a_not_null & ~has_real_b_not_null & has_imag_a_not_null & has_imag_b_not_null)
        %csak egyetlen real/imag
        reres = zeros(1+size(ima,1),size(ima,2));
        imres = zeros(1+size(ima,1),size(ima,2));
        for k = 1:size(ima,2)
           tempor = osztas(ima(:,k),imb(:,k),22,size(rea,1)).';
           reres(:,k) = tempor;
        end
        %outlog('imres',log_depth_easy);
        %outlog(imres,log_depth_easy);
    else
        %vgig csinlunk minden osztst
        % step1:perform simple multiplications
        % pre-allocate space for result
        arbr = zeros(2*size(a,1),size(a,2));
        if do_arbr
            % re(a)*re(b)
            for k = 1:size(a,2)
                arbr(:,k) = szorzas(rea(:,k),reb(:,k),22,size(a,1)).';
          end
        end
        %outlog('arbr',log_depth_easy);
        %outlog(arbr,log_depth_easy);

        % pre-allocate space for result
        aibi = zeros(2*size(a,1),size(a,2));
        if do_aibi
            % im(a)*im(b)
            for k = 1:size(a,2)
                aibi(:,k) = szorzas(ima(:,k),imb(:,k),22,size(a,1)).';
            end
        end
        %outlog('aibi',log_depth_easy);
        %outlog(aibi,log_depth_easy);

        % pre-allocate space for result
        arbi = zeros(2*size(a,1),size(a,2));
        if do_arbi
            % re(a)*im(b)
            for k = 1:size(a,2)
                arbi(:,k) = szorzas(rea(:,k),imb(:,k),22,size(a,1)).';
           end
        end
        %outlog('arbi',log_depth_easy);
        %outlog(arbi,log_depth_easy);

        % pre-allocate space for result
        aibr = zeros(2*size(a,1),size(a,2));
        if do_aibr
            % re(b)*im(a)
            for k = 1:size(a,2)
                aibr(:,k) =  szorzas(reb(:,k),ima(:,k),22,size(a,1)).';
           end
        end
        %outlog('aibr',log_depth_easy);
        %outlog(aibr,log_depth_easy);

        % pre-allocate space for result
        rebsquare = zeros(2*size(a,1),size(a,2));
        if (do_rebsquare)
            % re(b)*im(a)
            for k = 1:size(a,2)
                rebsquare(:,k) = szorzas(reb(:,k),reb(:,k),22,size(a,1)).';
           end
        end
        %outlog('rebsquare',log_depth_easy);
        %outlog(rebsquare,log_depth_easy);

        % pre-allocate space for result
        imbsquare = zeros(2*size(a,1),size(a,2));
        if (do_imbsquare)
            % re(b)*im(a)
            for k = 1:size(a,2)
                imbsquare(:,k) =  szorzas(imb(:,k),imb(:,k),22,size(a,1)).';
            end
        end
        %outlog('imbsquare',log_depth_easy);
        %outlog(imbsquare,log_depth_easy);

        %step2:perform adding
        % adding real
        real_counter = zeros(1+size(arbr,1),size(arbr,2));
        for k = 1:size(a,2)
            t = addbfp(arbr(:,k),aibi(:,k),22,size(arbr,1)).';
            real_counter(:,k) = t(1:1+size(arbr,1));
        end
        %outlog('real_counter',log_depth_easy);
        %outlog(real_counter,log_depth_easy);

        % adding imag
        imag_counter = zeros(1+size(arbr,1),size(arbr,2));
        for k = 1:size(a,2)
            t = addbfp(arbi(:,k),aibr(:,k),22,size(arbr,1)).';
            imag_counter(:,k)= t(1:1+size(arbr,1));
        end
        %outlog('imag_counter',log_depth_easy);
        %outlog(imag_counter,log_depth_easy);

        % adding divider
        divider = zeros(1+size(arbr,1),size(arbr,2));
        for k = 1:size(a,2)
             t = addbfp(rebsquare(:,k),-imbsquare(:,k),22,size(arbr,1)).';
             divider(:,k)= t(1:1+size(arbr,1));
        end
        %outlog('divider',log_depth_easy);
        %outlog(divider,log_depth_easy);

        %step3:dividing
        reres = zeros(1+size(real_counter,1),size(arbr,2));
        for k = 1:size(a,2)
           reres(:,k) = osztas(real_counter(:,k),divider(:,k),22,size(real_counter,1)).';
        end
        %outlog('reres',log_depth_easy);
        %outlog(reres,log_depth_easy);

        imres = zeros(1+size(real_counter,1),size(arbr,2));
        for k = 1:size(a,2)
           imres(:,k) = osztas(imag_counter(:,k),divider(:,k),22,size(real_counter,1)).';
        end
        %outlog('imres',log_depth_easy);
        %outlog(imres,log_depth_easy);

    end
end
% rounding
[reres,of,uf,inv,warn,err] = seground(reres,qcode); error(err); warning(warn);
ofs = of; ufs = uf; ins = inv;


% rounding
[imres,of,uf,inv,warn,err] = seground(imres,qcode); error(err); warning(warn);
ofs = ofs | of; ufs = ufs | uf; ins = ins | inv;

% ***** TRAILER *****
% interleaving real and imag part
t = reres+i.*imres;
% cut number of segs to SEGNUM
t = t(1:SEGNUM,:);
% reshape to original size
t = reshape(t,[SEGNUM,orsiz]);

% generating output object
if ~isempty(t(:))
   y.name = NAME;
else
   y.name = '';
end
y.data = t;
y = class(y,'qdata');

% store statistical info
if any(ofs(:)) | any(ufs(:)) | any(ins(:))
   % reshape stat info variables into size of 'data' without leading
   % segment dimensions
   if length(orsiza)>1
     ofs = reshape(ofs,orsiza);
     ufs = reshape(ufs,orsiza);
     ins = reshape(ins,orsiza);
   end
   st = dbstack;
   % clear the top of the stack containing info about current function
   st(1) = [];
   % prevent from storing empty 'dbstack' struct with complete fieldnames
   % (in case when command line event occured and the former operation cleared
   % this)
   if isempty(st), st = []; end
   % fill up struct
   stat.dbstack = st;
   stat.ofmask = ofs;
   stat.ufmask = ufs;
   stat.invmask = ins;
   if ~exist('v'), v=[]; end
   stat.arg1 = v;
   stat.arg2 = [];
   stat.result = y;
   % send to 'quantizers'
   qdspdemo('quantizers','event',stat);
end
