function y = sign(a)
%SIGN  Signum function of @qdata objects.
%     For each element of X, SIGN(X) returns 1 if the element
%     is greater than zero, 0 if it equals zero and -1 if it is
%     less than zero.  For the nonzero elements of complex X,
%     SIGN(X) = X ./ ABS(X)

a = a.data;
% filtering out empty matrix
if isempty(a), out = []; return, end

orsiza = size(a);
% calculating the number of segments in 'a'
segna = orsiza(1);
% calculating the real size WITHOUT segments - the length of size is
% minimum 2 so the '2:end' expression is always correct.
orsiza = orsiza(2:end);
% convert to two dimension (for the ABS operations on complex
% numbers)
a = reshape(a,[segna,prod(orsiza)]);

% deciding whether input are pure real, pure imag or complex
re = real(a);
im = imag(a);
isre = any(re(:));
isim = any(im(:));

if ~isre & ~isim
   % full zero
   y = 0;
   elseif isre & ~isim
   % pure real
   y = sign(sum(sign(re)));
elseif ~isre & isim
   % pure imaginary
   y = j*sign(sum(sign(im)));
else
   % complex numbers
   error('SIGN on complex numbers currently not supported due to the lack of SQRT function.')
end

% ***** TRAILER *****
% reshape to original size
if length(orsiza)>1, y = reshape(y,orsiza); end
