function varargout = size(x,d)
%SIZE          Size of @qdata object.
%    D = SIZE(X), for M-by-N object X, returns the two-element
%    row vector D = [M, N] containing the number of rows and columns
%    in the object.  For N-D arrays, SIZE(X) returns a 1-by-N
%    vector of dimension lengths.  Trailing singleton dimensions
%    are ignored.
%
%    [M,N] = SIZE(X) returns the number of rows and columns in
%    separate output variables. [M1,M2,M3,...,MN] = SIZE(X)
%    returns the length of the first N dimensions of X.
%
%    M = SIZE(X,DIM) returns the length of the dimension specified
%    by the scalar DIM.  For example, SIZE(X,1) returns the number
%    of rows.
% 
%    See also QDATA/LENGTH, QDATA/NDIMS.

% Copyright (C) 2003,2004 by JUHASZ Laszlo

% filter out empty objects
if isempty(x), varargout=[0,0]; return
elseif isempty(x(1).data)
   if nargin==1
      if nargout<=1
         varargout = {[0 0]};
      elseif nargout==2
         varargout = {0 0};
      else
         varargout = cell(1,nargout)
         varargout{1} = 0;
         varargout{2} = 0;
         for k = 3:nargout
            varargout{k} = 1;
         end
      end
   elseif nargin==2
      if isempty(d) | ~isa(d,'double') | prod(size(d))~=1
         error('Dimension number must be a non-negative integer.')
      end
      if ~isfinite(d) | rem(d,1) | d<1
         error('Dimension number must be a non-negative integer.')
      end
      if nargout>1
         error('Too many output arguments.')
      end
      if d<3
         varargout = {0};
      else
         varargout = {1};
      end
   else
      error('Too many input arguments.')
   end
   return
end

siz = size(x.data);
% correct 'siz'
% clear the leading 'segment-dimension'
siz(1) = [];
% if the number of remaining dimensions is less than 2 (only 1 is possible) then
% fill this value with one (with zero if data is empty)
if length(siz)<2
   if isempty(x.data)
      siz = [siz,0];
   else
      siz = [siz,1];
   end
end

if nargin==1
   if nargout<=1
      varargout = {siz};
   else
      % create output cell array with proper size
      varargout = cell(1,nargout);
      % fill output cell with ones - for case if the number of outputs is greater than
      % the dimension of the input
      for k = 1:nargout
         varargout{k} = 1;
      end
      % fill output cell with size values along dimensions respectively
      for k = 1:length(siz)
         varargout{k} = siz(k);
      end
   end
elseif nargin==2
   if isempty(d) | ~isa(d,'double') | prod(size(d))~=1
      error('Dimension number must be a non-negative integer.')
   end
   if ~isfinite(d) | rem(d,1) | d<1
      error('Dimension number must be a non-negative integer.')
   end
   if nargout>1
      error('Too many output arguments.')
   end
   if d<=length(siz)
      varargout = {siz(d)};
   else
      varargout = {1};
   end
else
   error('Too many input arguments.')
end
