function y = times(a,b)
%TIMES(.*)     Array multiply @qdata objects.
%     X.*Y denotes element-by-element multiplication.  X and Y
%     must have the same dimensions unless one is a scalar.
%     A scalar can be multiplied into anything.
%     Either of X or Y can be also a double array.

% Copyright (C) 2004 by JUHASZ Laszlo

% ***** PREAMBLE *****
switch class(a)
   case 'qdata'
      if isempty(a.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsiza = size(a.data);
         % calculating the number of segments in 'a'
         segna = orsiza(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsiza = orsiza(2:end);
         % convert to two dimension
         a.data = reshape(a.data,[segna,prod(orsiza)]);
      end
   case 'double'
      if isempty(a)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'a' argument to segments
         % save original size of 'a'
         orsiza = size(a);
         % generating segnum - it's always 3 in case of 'double'
         segna = 3;
         % convert 'a' to column vector
         a = a(:);
         % fill up the variable 'z' with segment-transformed values of 'a'
         for k = 1:length(a)
            z(:,k) = double2seg(a(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'a' and put back the data of 'z' to the
         % 'data' field
         a = [];
         a.data = z;
         a.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end

switch class(b)
   case 'qdata'
      if isempty(b.data)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         orsizb = size(b.data);
         % calculating the number of segments in 'b'
         segnb = orsizb(1);
         % calculating the real size WITHOUT segments - the length of size is
         % minimum 2 so the '2:end' expression is always correct.
         orsizb = orsizb(2:end);
         % convert to two dimension
         b.data = reshape(b.data,[segnb,prod(orsizb)]);
      end
   case 'double'
      if isempty(b)
         % argument is empty so the result will be empty too
         y = class(emptyclass,'qdata');
         return
      else
         % convert 'b' argument to segments
         % save original size of 'b'
         orsizb = size(b);
         % generating segnum - it's always 3 in case of 'double'
         segnb = 3;
         % convert 'b' to column vector
         b = b(:);
         % fill up the variable 'z' with segment-transformed values of 'b'
         for k = 1:length(b)
            z(:,k) = double2seg(b(k)).';
         end
         % 'z' is a two-dimensional array with segments in the rows and numbers
         % in the columns
         % making a fake-object from 'b' and put back the data of 'z' to the
         % 'data' field
         b = [];
         b.data = z;
         b.name = '';
      end
   otherwise error('Only qdata and double inputs are supported.')
end

% download quantizer properties - the first argument ('a') has a bigger
% priority.
if ~isempty(a.name)
   qcode = qdspdemo('quantizers',a.name);
elseif ~isempty(b.name)
   qcode = qdspdemo('quantizers',b.name);
else
   error('One of the inputs must have a quantizer name.');
end   
eval(qcode)

% extract values from structs - the name is unnecessary from now on
a = a.data;
b = b.data;

% scalar extension and check incorrect lengths
% 'a' and 'b' are both 2-dim arrays - compatibility with Matlab 5.3!!!
if prod(orsiza)==1
   % the 'a' object is scalar so its variable is a column vector
   % extend 'a' to the size of 'b'
   a = a(1:size(a,1),ones(size(b,2),1));
   orsiza = orsizb;
elseif prod(orsizb)==1
   % the same case with 'b'
   b = b(1:size(b,1),ones(size(a,2),1));
   orsizb = orsiza;
elseif ~isequal(orsiza,orsizb)
   error('Matrix dimensions must agree.')
end

orsiz(1:length(orsiza)) = orsiza;
orsiz(1:length(orsizb)) = orsizb;

% from now size(a,2)==size(b,2)

% balancing number of segments
if segna~=segnb
   if segna>segnb
      % extend number of rows of 'b' to the number of rows of 'a'
      b(segna,:) = 0;
   else
      % extend number of rows of 'a' to the number of rows of 'b'
      a(segnb,:) = 0;
   end
end

% from now size(a,1)==size(b,1) so size(a)==size(b)
% ***** end of PREAMBLE *****

% deciding real and imag operations
% as default all the four multiplications are necessary
% re(a)*re(b)-im(a)*im(b) + j*(re(a)*im(b)+re(b)*im(a))
d = 1;           e = 1;          f = 1;       g = 1;
rea = real(a);
reb = real(b);
ima = imag(a);
imb = imag(b);
isare = any(rea(:));
isaim = any(ima(:));
isbre = any(reb(:));
isbim = any(imb(:));

% restrict the set of necessary multiplications
if ~isare, d = 0; f = 0; end
if ~isaim, e = 0; g = 0; end
if ~isbre, d = 0; g = 0; end
if ~isbim, e = 0; f = 0; end

% perform multiplications

% pre-allocate space for result
dr = zeros(2*size(a,1),size(a,2));
if d
   % re(a)*re(b)
   for k = 1:size(a,2)
      dr(:,k) = szorzas(rea(:,k),reb(:,k),22,size(a,1)).';
      bin_result = b_szorzas(rea(:,k),reb(:,k),22,size(a,1)).';
	if(bin_result(1:5)~=dr(1:5,k))
		ERROR('QDATA:TIMES','szmtsi hiba');
	end
   end
end

% pre-allocate space for result
er = zeros(2*size(a,1),size(a,2));
if e
   % im(a)*im(b)
   for k = 1:size(a,2)
      er(:,k) = szorzas(ima(:,k),imb(:,k),22,size(a,1)).';
      bin_result = b_szorzas(ima(:,k),imb(:,k),22,size(a,1)).';
	if(bin_result(1:5)~=er(1:5,k))
		ERROR('QDATA:TIMES','szmtsi hiba');
	end
   end
end

% pre-allocate space for result
fr = zeros(2*size(a,1),size(a,2));
if f
   % re(a)*im(b)
   for k = 1:size(a,2)
      fr(:,k) = szorzas(rea(:,k),imb(:,k),22,size(a,1)).';
      bin_result = b_szorzas(rea(:,k),imb(:,k),22,size(a,1)).';
	if(bin_result(1:5)~=fr(1:5,k))
		ERROR('QDATA:TIMES','szmtsi hiba');
	end
   end
end

% pre-allocate space for result
gr = zeros(2*size(a,1),size(a,2));
if g
   % re(b)*im(a)
   for k = 1:size(a,2)
      gr(:,k) = szorzas(reb(:,k),ima(:,k),22,size(a,1)).';
      bin_result = b_szorzas(reb(:,k),ima(:,k),22,size(a,1)).';
	if(bin_result(1:5)~=gr(1:5,k))
		ERROR('QDATA:TIMES','szmtsi hiba');
	end
   end
end

% adding
% adding real
reres = zeros(1+size(dr,1),size(dr,2));
for k = 1:size(a,2)
   t = addbfp(dr(:,k),-er(:,k),22,size(dr,1)).';
   reres(1:length(t),k) = t;
end
% rounding
[reres,of,uf,inv,warn,err] = seground(reres,qcode); error(err); warning(warn);
ofs = of; ufs = uf; ins = inv;

% adding imag
imres = zeros(1+size(dr,1),size(dr,2));
for k = 1:size(a,2)
   t = addbfp(fr(:,k),gr(:,k),22,size(dr,1)).';
   imres(1:length(t),k) = t;
end
% rounding
[imres,of,uf,inv,warn,err] = seground(imres,qcode); error(err); warning(warn);
ofs = ofs | of; ufs = ufs | uf; ins = ins | inv;

% ***** TRAILER *****
% interleaving real and imag part
t = reres+i.*imres;
% cut number of segs to SEGNUM
t = t(1:SEGNUM,:);
% reshape to original size
t = reshape(t,[SEGNUM,orsiz]);

% generating output object
if ~isempty(t(:))
   y.name = NAME;
else
   y.name = '';
end
y.data = t;
y = class(y,'qdata');

% store statistical info
if any(ofs(:)) | any(ufs(:)) | any(ins(:))
   % reshape stat info variables into size of 'data' without leading
   % segment dimensions
   ofs = reshape(ofs,orsiza);
   ufs = reshape(ufs,orsiza);
   ins = reshape(ins,orsiza);
   st = dbstack;
   % clear the top of the stack containing info about current function
   st(1) = [];
   % prevent from storing empty 'dbstack' struct with complete fieldnames
   % (in case when command line event occured and the former operation cleared
   % this)
   if isempty(st), st = []; end
   % fill up struct
   stat.dbstack = st;
   stat.ofmask = ofs;
   stat.ufmask = ufs;
   stat.invmask = ins;
   stat.arg1 = v;
   stat.arg2 = [];
   stat.result = y;
   % send to 'quantizers'
   qdspdemo('quantizers','event',stat);
end
