function z = ulp(x)
%ULP Returns the smallest possible increment or decrement
%   that can be made using the machine's floating-point arithmetic.
%
%   ULP(X) An ulp of a real number value is the distance between the 
%   floating-point numbers which bracket the real value. 
%   Of course since the only reals we can represent happen to be 
%   floating-point numbers, this corresponds to the distance 
%   between adjacent floating-point values. 
%
%   To avoid ambiguities at powers of two (where the distance 
%   between numbers changes), we can define an ulp of a 
%   floating-point value to be the distance to the next 
%   floating-point value larger in magnitude
%
%
%   use only with scalar qdata objects
%
%   (C) Rvsz Sndor 2005


if all(size(x)==[1 1])

    qcode = qdspdemo('quantizers',x.name);
    eval(qcode);
    [xm,xe] = log2(x.data(1));
    %M = 2 ^  (xe - (SEGNUM * 22));
    if (xm==0)|(xe<-1021), xeM=log2(realmin)+1; else xeM=xe; end
    M=pow2(xeM-PRECISION);
    if isequal(abs(xm),0.5)&(xe>-1022)
      M=M/2; 
      %warning('Adjusting is necessary'), 
    end
    
    xupper = x + M;
    xlower = x - M;

%    if ~(x+M-M-x ==0)
%        error('ERROR IN QDSP')
%    end
    xdiffupper = abs(x - xupper);
    xdifflower = abs(x - xlower);

    if(xdiffupper>xdifflower)
        z = xdiffupper;
    else
        z = xdifflower;
    end


else
    error('ULP must used with scalar qdata objects');
end