function X = FFTD(xbe,fftdft)
%FFTD   Fast Fourier Transfrom for QDSP Toolbox's Objects

%   X = FFTD(xbe,fftdft)

%x must be a vector or a skalar. if x is a skalar,then the output is the same
if(size(xbe)==[1,1])
    X = xbe;
    return 
end
if(prod(size(xbe))==length(xbe))
    lxbe = length(xbe);
else
    error('QDSP:FFTD:argchk','The input is not a vector');
    return;
end
%length(x) must be a power of 2. if not, wi will make it.
lengthlog = nextpow2(lxbe);%this will be the power
lengthx = 2^lengthlog;%this is the real length

%first argument cheking
if (nargin<2)
    fft_type='fft_freq'
else
    fft_type=fftdft;
end
%these are the global wariables, to not move huge data amount, only
%pointers. 
global wws;
wws=0;

%then we decide which fft_type was invoked
switch lower(fft_type)
    case 'fft_time'
        X=timedomainFFT_main(xbe,lengthlog,lengthx);    
    case 'fft_freq'
        X=freqdomainFFT_main(xbe,lengthlog,lengthx);    
    case 'dft'
        X=DFT_main(xbe,lengthx);        
end

function X=freqdomainFFT_main(x,lengthlog,lengthx)
%--------------------------------------------------------------------------
    %first we must prepare the coefficients
    global wws;
    if(strcmp(class(x),'qdata'))
        wws = makewws(lengthx,class(x),get(x,'name'));
    else
        wws = makewws(lengthx,class(x),'double');
    end
    ivege = lengthlog;%0..log2(hossz)
    kvege = 1;%0..2^i
    mvege = lengthx/2;%0..lengthx/(2^i)
    counter=0;
%    hugeX = x;
    for i = 0 : ivege-1;
        %eenyi db bf oszlop van
        %disp('iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii');
        for k = 0 : kvege-1
            %egy oszlopon belul ennyi blokk van
            %mindegyiknek ugyanaz a ww-je, csak a bemenete ms
            %disp('kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk');
            nk = fordit(k,ivege-1);
            fprintf('Column: %.f, butterfly: %.0f, nk: %.0f\n',i,k,nk)
            for m = 0 : mvege-1
                %vgrehajtjuk a bfeket
                kit = k * 2 * mvege + m;
                kivel = kit + mvege;
                %sout = sprintf('i:%d.oszlop k:%d.blokk m:%d.lepke kit=%d(%d)\tkivel=%d(%d)\t  wszama=%d',i,k,m,kit,kit+1,kivel,kivel+1,nk);
                %disp(sout);
                [x(1+kit),x(1+kivel)]=bf(x(1+kit),x(1+kivel),nk);
                counter = counter + 2;
            end
        end
 %       hugeX = cat(2,hugeX,x);
        kvege = kvege * 2;
        mvege = mvege / 2;
    end
    clear wws;
    counter
    if strcmp(class(x),'qdata')
        X = reverseorder(x);
    else
        X = digitrevorder(x,2);
    end
return;
%--------------------------------------------------------------------------
function X=timedomainFFT_main(x,lengthlog,lengthx)
    %first we must prepare the coefficients
    error('eztne');
    global wws;
     if(strcmp(class(x),'qdata'))
        wws = makewws(lengthx,class(x),get(x,'name'));
    else
        wws = makewws(lengthx,class(x),'double');
    end
   %then we bitreverse the input
    if strcmp(class(x),'qdata')
        X = reverseorder(x);
    else
        X = digitrevorder(x,2);
    end

%               0   1   2   3   4   5   6   7
%  level   1
%  step    1                                    0    0
%  increm  2                                   W  = W
%  k = 0 (m=0..7)                               1    8
%               <--->   <--->   <--->   <--->   
%               m=0     m=2     m=4     m=6
%  level   2
%  step    2
%  increm  4                                    0        0
%  k = 0        <------->       <------->      W    =   W    
%               m=0;            m=4;            2    1   8    1
%  k = 1            <------->       <------->       W    =   W
%                   m=1;            m=5;             2        8
%  level   3                                         
%  step    4
%  increm  8                                    0       0
%  k = 0        <--------------->              W    =  W 
%               m=0;                            3    1  8    1
%  k = 1            <--------------->               W    =  W   
%                   m=1;                             3    2  8    2
%  k = 2                <--------------->                W    =  W   
%                       m=2;                              3    3   8  3  
%  k = 3                    <--------------->                 W   =  W
%                           m=3;                               3      8
%                                                              
%    
    ivege = lengthlog;%0..log2(hossz)
    kvege = lengthx/2;%0..hossz/2
    mvege = 1;
    counter=0;
    %hugeX = x;
    for i = 0 : ivege-1
        for k = 0 : kvege-1
            for m = 0 : mvege-1
                kit = k * 2 * mvege + m;
                kivel = kit + mvege;
                nk = m * kvege;
                
                %T = wws(nk+1) * x(kivel+1);
                %x(kivel+1) = x(kit+1) - T;                
                %x(kit+1) = x(kit+1) + T;
                [x(1+kit),x(1+kivel)]=bf(x(1+kit),x(1+kivel),nk);
                %sout = sprintf('i:%d.oszlop k:%d.blokk m:%d.lepke kit=%d(%d)\tkivel=%d(%d)\t  wszama=%d',i,k,m,kit,kit+1,kivel,kivel+1,m*kvege);
                %disp(sout);
            end
        end
        %hugeX = cat(2,hugeX,x);
        kvege = kvege / 2;
        mvege = mvege * 2;
    end
    X = x;
    clear wws;
return
%--------------------------------------------------------------------------
function X=DFT_main(x,lengthx)
    if strcmp(class(x),'qdata')
        X = qdata(zeros(size(x)),get(x,'name'));
    else
        X = zeros(size(x));
    end     
    if(strcmp(class(x),'qdata'))
        wws = makewws2(lengthx,class(x),get(x,'name'));
    else
        wws = makewws2(lengthx,class(x),'double');
    end

    for k = 0 : lengthx-1
        for n = 0 : lengthx-1
            X(k+1) = X(k+1) + x(n+1) .* wws(1+mod(k*n,lengthx));
        end
    end
    clear wws;
return
%--------------------------------------------------------------------------

%--------------------------------------
%kiegszt fggvnyek az implementlsokhoz
%--------------------------------------
function mire = fordit(kit,bitszam)
if(bitszam==0) mire =0; return; end;
if(bitszam<0 | kit<0) error ('XXX'); end;
mire = 0;
if(kit==0)mire =0;return; end;
for bi = 1 : bitszam
     aa = bitget(kit,bitszam-bi+1);
     mire=bitset(mire,bi,aa);
end
return
%--------------------------------------------------------------------------
function [c,d]=bf(a,b,nk)
global wws;
%This is what we should edit when changing to BFP objects;
c = a + wws(nk+1).*b;
d = a - wws(nk+1).*b;
return;
%--------------------------------------------------------------------------
function ki=w(nk,N,name)
%This is what we should edit when changing to BFP objects
%ki=exp(-j*2*pi*nk/N);
%ki = cos(2.*qpi.*nk./N) + j.*sin(2.*qpi.*nk./N);
qpi = qdata(qpi_str(qdata),name);
melyik = 2 * nk / N;
ki = expj(melyik .* qpi);
return;

function ki=wd(nk,N)
%This is what we should edit when changing to BFP objects;
%ki=exp(-j*2*pi*nk/N);
%ki = cos(2.*qpi.*nk./N) + j.*sin(2.*qpi.*nk./N);
melyik = 2 * nk / N;
ki = exp(-j* melyik * pi);
return;
%--------------------------------------------------------------------------
function wsor = makewws(wwsN,type,name)
if strcmp(type,'qdata')
    wsor = qdata(zeros(wwsN/2,1),name);
    for i=0:(wwsN/2)-1
        wsor(i+1)=w(i,wwsN,name);
    end
else
    wsor = zeros(wwsN/2,1);
    for i=0:(wwsN/2)-1
        wsor(i+1)=wd(i,wwsN);
    end
end;

function wsor = makewws2(wwsN,type,name)
if strcmp(type,'qdata')
    wsor = qdata(zeros(wwsN,1),name);
    for i=0:(wwsN)-1
        wsor(i+1)=w(i,wwsN);
    end
else
    wsor = zeros(wwsN/2,1);
    for i=0:(wwsN)-1
        wsor(i+1)=wd(i,wwsN);
    end
end;
%--------------------------------------------------------------------------
