function [s,err] = qtest(varargin)
%QTEST             Check consistency of a quantizer
%input, modifiers, default generation supported

% Copyright (C) 2004 by JUHASZ Laszlo

q = celldisint(varargin);
s = []; err = '';
if isempty(q), return, end
persistent propnames types mayempty ors values possvals

% THE SETS OF PROPNAMES AND POSSIBLE VALUES MUST BE DISJUNCT!!!
if isempty(propnames)
   % 'propnames' contains the property names in a row-cell
   propnames = {'name' 'notes' 'rights' 'type' 'coding' 'precision' 'leadingbit'...
         'fractionbits' 'nansignal' 'overflow' 'ofsignal' 'underflow' 'ufsignal'...
         'roundingdir' 'treathalf' 'expcoding' ... %'expbits' 
         'expbias' 'emax' 'emin' 'maximum' 'minimum' 'dither', 'qnoise'...
         'operation'};
   
   % 'types' contains the types of properties in a row cell. Its size accords
   % to 'propnames's size
   types = {    'char' 'char'  'char'   'char' 'char'   'double'   'char'...
         'double'  'char'      'char'     'char'     'char'      'char'...
         'char'     'char'      'char'    ... %'double' 
         'double' 'double' 'double' 'vector' 'vector' 'char' 'char' ...
         'char'};
   
   % 'mayempty' contains the possible empty properties
   mayempty = [   0       1       0        0       0         0           1 ...
         0          0            0         0           1           0 ...
         0          0            1         ... %1           
         1         1     1     1   1   0   0 ...
         1];
   
   % check length of 'types' and 'mayempty'
   if length(types)~=length(propnames) | length(mayempty)~=length(propnames)
      error('Internal error 1')
   end
   
   % 'ors' contains the original structure
   for k = 1:length(propnames)
      switch types{k}
      case 'char'
         eval(['ors.',propnames{k},'='''';'])
      case {'double','vector'}
         eval(['ors.',propnames{k},'=[];'])
      otherwise error('Internal error 2')
      end
   end
   
   % 'values' contains the possible property names in a column-cell, where each
   % element is a row-cell. Empty elements indicate properties that doesn't have
   % a finite set of values (e.g. numbers).
   values = {...
         {};...                                                         %  1. name
         {};...                                                         %  2. notes
         {'read-only' 'writable'};...                                   %  3. rights
         {'fixed-point' 'floating-point'};...                                       %  4. type
         {'unsign' 'ones-comp' 'twos-comp' 'sign-mag'};...              %  5. coding
         {};...                                                         %  6. precision
         {'hidden' 'shown'};...                                         %  7. leadingbit
         {};...                                                         %  8. fraction
         {'neglect+nan' 'count+nan' 'warning+count+nan' 'error'};...    %  9. nansignal
         {'inf' 'clip' 'modular' 'triang'};...                          % 10. overflow
         {'neglect' 'count' 'warning+count' 'error'};...                % 11. ofsignal
         {'flush' 'gradual'};...                                        % 12. underflow
         {'neglect' 'count' 'warning+count' 'error'};...                % 13. ufsignal
         {'fix' 'floor' 'ceil' 'toinf' 'round' 'trunc'};...             % 14. roundingdir
         {'toeven' 'toodd' 'fix' 'floor' 'ceil' 'toinf' 'trunc'};...            % 15. treathalf
         {'ones-comp' 'twos-comp' 'sign-mag' 'biased'};...              % 16. expcoding
         %{};...                                                         % 17. expbits
         {};...                                                         % 18. expbias
         {};...                                                         % 19. emax
         {};...                                                         % 20. emin
         {};...                                                         %maximum
         {};...                                                         %minimum
         {'off' 'uniform' 'dig-uniform' 'triangular' 'dig-triangular'};... %dither
         {'off' 'mean' 'var+mean' 'cov+var+mean'};...                   % 21. qnoise
         {'quantize' 'PQN'};...                                         % 22. operation
      };
   
   % 'possvals' contains the same values only once
   possvals = unique(celldisint(values));
   % check propnames and values
   if ~isempty(intersect(propnames,possvals))
      error('Internal error 3')
   end
end

% for QTEST
if length(q)==1
   t = q{1};                  % temp var
   if ischar(t) & strcmp(t,'get=internal=values')
      s = {ors,values}; return
   end
end

% look for input struct
if isa(q{end},'struct')
   s = q{end};
   q(end) = [];
   if isempty(s) | prod(size(s))~=1
      err = 'Empty or not scalar ''default'' struct.'; return
   end
   t = fieldnames(s);
   for k = 1:length(propnames)
      if ~strcmp(t{k},propnames{k})
         err = 'Invalid fieldname in ''default'' struct.'; return
      end
   end
else  % no 'default' struct
   s = ors;
end

if ~isempty(q)
  if isequal(length(q),2)&strcmp(q{1},'name'), q=[q,{'double'}]; end
  if isequal(length(q),2)&~strcmp(q{1},'name')&~any(strmatch(q{1},propnames))
    if strncmpi(q{2},'double',5), q=[{'name'},q(1),{'p'},{53}];
		elseif strncmpi(q{2},'BigFP',5)|strncmpi(q{2},'BFP',3), q=[{'name'},q(1),{'p'},{110}];
    elseif strncmpi(q{2},'single',1), q=[{'name'},q(1),{'p'},{24},{'emax'},{127},{'emin'},{-126}];
    elseif strncmpi(q{2},'8bit',1), q=[{'name'},q(1),{'type'},{'fixed-point'},{'p'},{7}];
    elseif strncmpi(q{2},'16bit',2), q=[{'name'},q(1),{'type'},{'fixed-point'},{'p'},{15}];
    elseif strncmpi(q{2},'32bit',2), q=[{'name'},q(1),{'type'},{'fixed-point'},{'p'},{31}];
    end
  elseif isequal(length(q),3)&strcmp(q{1},'name')
    if strncmpi(q{3},'double',1), q=[q(1:2),{'p'},{53}];
		elseif strncmpi(q{2},'BigFP',5)|strncmpi(q{2},'BFP',3), q=[{'name'},q(1:2),{'p'},{110}];
    elseif strncmpi(q{3},'single',1), q=[q(1:2),{'p'},{24},{'expbits'},{8}];
    elseif strncmpi(q{3},'8bit',1), q=[{'name'},q(1:2),{'type'},{'fixed-point'},{'B'},{8}];
    elseif strncmpi(q{3},'16bit',1), q=[{'name'},q(1:2),{'type'},{'fixed-point'},{'B'},{16}];
    elseif strncmpi(q{3},'32bit',1), q=[{'name'},q(1:2),{'type'},{'fixed-point'},{'B'},{32}];
    elseif strncmpi(q{3},'64bit',1), q=[{'name'},q(1:2),{'type'},{'fixed-point'},{'B'},{64}];
    end
  end
  % check formats
  [q,qerr] = str2int(q);
  if ~isempty(qerr), err = qerr; return, end
  q = celldisint(q);
   
   % check the even number of inputs
   if rem(length(q),2)
     if isstr(q{1})&~strncmpi(q(1),'name',2)&~any(strmatch(q{1},[propnames,{'B'}]))&...
         any(strmatch(q{2},[propnames,{'B'}]))
       q=[{'name'},q];
     else
       err = 'The number of inputs must be even.'; return
     end
   end
   
   % 'qmap' contains the category of elements of 'q'.
   % 0 - yet undefined, positive - propname or corresponding value
   qmap = zeros(length(q),10);
   
   % collect possibilities
   for k = 1:length(q)
      pn = [];
      if rem(k,2)                            % odd positions (1,3,5,...)
         % check propname format
         if ~ischar(q{k})
            err = ['Property name cannot be numeric (',num2str(q{k}),').']; return
         end
         % inserting possible propnames
         for m = 1:length(propnames)
            if strncmp(q{k},propnames{m},length(q{k})), pn = [pn,m]; end
         end
         % if no occurrence
         if isempty(pn)&strncmpi(q{k},'Bits',1), q{k+1}=q{k+1}-1; pn=6; end
         if isempty(pn)
            err = ['Invalid property name ''',q{k},'''.']; return
         end
         pn = unique(pn);
         for m = 1:length(pn)
            pos = sum(~~qmap(k,:))+1;
            qmap(k,pos) = pn(m);
         end
      else                                   % even positions (2,4,6,...)
         % inserting possible values
         for m = 1:length(values)
            t = values{m};
            for n = 1:length(t)
               % occurence with value of a propname
               if strncmp(q{k},t{n},length(q{k})), pn = [pn,m]; end
            end
         end
         pn = unique(pn);
         pos = sum(~~qmap(k,:))+1;
         qmap(k,pos:pos+length(pn)-1) = pn;
      end
   end
   
   % inserting values of 'free' propnames (without determined value)
   for k = 1:2:size(qmap,1)
      t = qmap(k,:); t = t(~~t);
      % 't' contains the possible propnames
      for m = 1:length(t)
         if isempty(values{t(m)}) & isa(q{k+1},types{t(m)})
            % if not option-like value and value at next position has good type
            % write to the possible properties this values
            pos = sum(~~qmap(k+1,:))+1;
            qmap(k+1,pos) = t(m);
         end
      end
      t = qmap(k+1,:); t = t(~~t);
      % if list of possible values is still empty
      if isempty(t)
         if ischar(q{k+1})
            err = ['Invalid property value ''',q{k+1},'''.']; return
         else
            err = ['Invalid property value ',num2str(q{k+1}),'.']; return
         end
      end
   end
   
   % check qmap
   for k = 1:2:size(qmap,1)
      t = qmap(k,:); t = t(~~t);             % possible property names
      u = qmap(k+1,:); u = u(~~u);           % possible values
      c = intersect(t,u);
      switch length(c)
      case 0
         % no correlation
         if ischar(q{k+1})
            err = ['Invalid property name - value pair ''',q{k},''' - ''',...
                  q{k+1},'''.']; return
         else
            err = ['Invalid property name - value pair ''',q{k},''' - ',...
                  num2str(q{k+1}),'.']; return
         end
      case 1
         % only one correlation
         t = 0*qmap(1,:); t(1) = c;
         qmap(k,:) = t; qmap(k+1,:) = t;
      otherwise
         % ambiguous correlation
         if length(c)>1
            if ischar(q{k+1})
               err = ['Ambiguous property name - property value pair ''',q{k},...
                     ''' - ''',q{k+1},'''.']; return
            else
               err = ['Ambiguous property name - property value pair ''',q{k},...
                     ''' - ',num2str(q{k+1}),'.']; return
            end
         end
      end
   end
   
   % check ambiguous values and fill up struct with completed values
   for k = 2:2:size(qmap,1)
      t = qmap(k,:); t = t(~~t);          % possible property values
      if ischar(q{k})                     % char input - need to be completed
         u = values{t};
         occ = [];
         for m = 1:length(u)
            if strncmp(q{k},u{m},length(q{k})), occ = [occ,m]; end
         end
         switch length(occ)
         case 0            % no occurrence or not option-like value
            eval(['s.',propnames{t},'=q{k};'])
         case 1            % one occurrence
            eval(['s.',propnames{t},'=u{occ};'])
         otherwise
            err = ['Ambiguous value ''',q{k},''' for property ''',propnames{t},'''.'];
            return
         end
      else                                % numeric input - no need to be completed
         eval(['s.',propnames{t},'=q{k};'])
      end
   end %for k
end

% check struct
% re-check needed because the incoming struct can contain errors
for k = 1:length(propnames)
   t = eval(['s.',propnames{k}]);
   u = values{k};
   % call the default function
   if isempty(t)
      t = eval(['def_',propnames{k},'(s)']);
   end
   % check type
   if ~isa(t,types{k})&~strcmp(types{k},'vector')
      err = [propnames{k},' must be the type of ',types{k},'.']; return
   end
   % check emptiness
   if ~mayempty(k) & isempty(t)
      err = [propnames{k},' must be nonempty.']; return
   end
   % check value
   if ~isempty(t)
      switch class(t)
      case 'char'
         % general char check
         if ndims(t)~=2 | prod(size(t))~=length(t) | size(t,1)~=1
            err = [propnames{k},': Must be row string.']; return
         end
         %t = lower(t);
         t = t(find(t=='_' | t=='+' | t=='-' | (t>='a' & t<='z') | (t>='A' & t<='Z') | (t>='0' & t<='9')));
         % option-like value
         if ~isempty(u)
            occ = [];
            for m = 1:length(u)
               if strcmp(t,u{m}), occ = [occ,m]; end
            end
            if isempty(occ)
               if ischar(t)
                  err = ['Invalid property name - property value pair ''',...
                        propnames{k},''' - ''',t,'''.']; return
               else
                  err = ['Invalid property name - property value pair ''',...
                        propnames{k},''' - ',num2str(t),'.']; return
               end
            end
            if length(occ)>1
               if ischar(t)
                  err = ['Ambiguous property name - property value pair ''',...
                        propnames{k},''' - ''',t,'''.']; return
               else
                  err = ['Ambiguous property name - property value pair ''',...
                        propnames{k},''' - ',num2str(t),'.']; return
               end
            end
         end
      case 'double'
         % general double check
         %if prod(size(t))~=1
         if size(t,2)~=1
            err = [propnames{k},': Must be scalar or vector.']; return
         end
         if ~all(isfinite(t))
            err = [propnames{k},': Must be finite.']; return
         end
         if (rem(t,1)~=0)&~strcmp(propnames{k},'maximum')&~strcmp(propnames{k},'minimum')
            err = [propnames{k},': Must be integer.']; return
         end
      end
   end
   % specific check
   terr = eval(['check_',propnames{k},'(t,s)']);
   if ~isempty(terr)
     err = [propnames{k},': ',terr]; return
   end
   % write back to struct
   eval(['s.',propnames{k},'=t;'])
end %for k
1;