function [y,err] = str2int(varargin)
%STR2INT           Transform string to number or keep it as string

% Copyright (C) 2003,2004 by JUHASZ Laszlo

x = varargin;
y = [];
err = '';

if isempty(x), return, end

if length(x)==1 & isempty(x{1})
   y = x;
   return
end

if isa(x,'cell')
   if size(x,1)~=1
      err = 'Not row cell.';
      return
   end
   x = celldisint(x);
   % if scalar cell
   if prod(size(x))==1
      x = x{1};
   else
      z = {};
      for k = 1:length(x)
         [t,terr] = str2int(x{k});
         if ~isempty(terr)
            err = [num2str(k),'. position: ',terr];
            return
         end
         z = {z{:},t};
      end
      y = z;
      return
   end
end

if isnumeric(x)
   if prod(size(x))~=1
      err = 'Not scalar number.';
      return
   end
   if ~isfinite(x)
      err = 'Nonfinite number.';
      return
   end
   if rem(x,1)
      err = 'Not integer number.';
      return
   end
   y = x;
elseif ischar(x)
   % check format
   if ndims(x)~=2 | prod(size(x))~=length(x) | size(x,1)~=1
      err = 'Not row string.';
      return
   end
   % trying to transform with 'sscanf'
   [a,count,errmsg] = sscanf(x,'%g');
   if count~=1 | ~isempty(errmsg) | ~isfinite(a) | rem(a,1)
      % keep only 'good' characters
      %x = lower(x);
      x = x(x=='_' | x=='+' | x=='-' | (x>='a' & x<='z') |(x>='A' & x<='Z') | (x>='0' & x<='9'));
      y = x;
   else
      y = a;
   end
else
   err = 'Invalid type.';
   return
end
