function test_qdsp_calls
%TEST_QDSP_CALLS  Execute shown examples

disp('Test calls in book ...')
qclear all

help qdsp, qdspstart

%roundfloat
testroundfloat_all

%qdata
qdata(1+eps) %1.000...0001e0d
qdata((1+eps)*2^(-1022)) %1.000...1 2^(1)
qdata('1.111b') %double prec 
qdata('1.111b','') %fixed-point

%@data\lsb
qclear all
Q=qmake('Qd','double'); Qs=qmake('Qs','single');
qmake('Qf','16-bit');
one=qdata(1,Q); two=qdata(2,Q); one_sing=qdata(1,'Qs');
[mag,lsbit]=lsb(one)  %is [eps = 2^(-52) = 2.2204e-016,0]
[mag,lsbit]=lsb(two)  %is [2*eps = 2*2^(-52) = 4.4409e-016,0]
[mag,lsbit]=lsb(one_sing+2^(-23))  %is [eps = 2^(-23) = 1.1921e-007,1]
[mag,lsbit]=lsb(qdata(0,'Qd'))  %is [realmin*eps = 2^(-1022-52) = 4.9407e-324,0]
%[mag,lsbit]=lsb(qdata(2^16-1,'Qf')) %is [1,1]
qclear
qmake('Qs','prec',24);
for ii=1:8
  x=qdata(1+ii*eps), [mag,val]=lsb(x);
  if ~isequal(rem(ii,2),val), error('lsb error'), end
  x=qdata(1+ii*2^(-23),'Qs'), [mag,val]=lsb(x);
  if ~isequal(rem(ii,2),val), error('lsb error'), end
end

%qdata
x=qdata('1001 /4/ 01 b')   %1001111101 (binary)
qmake('Q10','B',11)
if ~strcmp(num2bin(qdata(x,'Q10')),'010 0111 1101')
  error('qdata1 failed')
end
x=qdata('.bb\3\fx @ -15d') %0.bb000f (hexadecimal) * 2^(-15 (decimal))
x=qdata('j34.o')           %j*[34 (octal)]
if ~isequal(double(x),j*(3*8+4))
  error('qdata3 failed')
end
x=qdata('1001 /4/ 01 b')   %1001111101 (binary)
if ~strcmp(num2bin(qdata(x,'Q10')),'010 0111 1101')
  error('qdata4 failed')
end

%KOCZKA
test_num2bin_k

%WEB
qdspstart %hints how to start
Q=qmake('Qsw','single'); %IEEE single prec
a=qdata(1.22,Q); b=qdata(2.1,Q); %define numbers
c=a*b; %perform quantized multiplication 
get(c) %show number and associated quantizers
x=qdata(1,'BigFP') %110-bit number
y=randn(x) %normally distributed 110-bit number
y=rand(x) %uniformly distributed 110-bit number

%BOOK
%Appendix on Matlab-based simulation
qclear all

x=1; q=1;
xq = q*round(x/q);
roundrand([1,.5:4.5]);
%
p=52; x=1+eps;
[f,e]=log2(x); dxp = sign(x).*pow2(e+52-p); xqfl = (x+dxp)-dxp;
if ~isequal(xqfl,1), error('Floating test error b1a'), end
p=52; x=1+2*eps;
[f,e]=log2(x); dxp = sign(x).*pow2(e+52-p); xqfl = (x+dxp)-dxp;
if ~isequal(xqfl,1+2*eps), error('Floating test error b2a'), end
%
if ~isequal(roundfloat([-15,-2,-1,7:15],3),[-16,-2,-1,7,8,8,10,12,12,12,14,16])
  error('roundfloat error')
end
%
Q = qstruct('p',24); %define a quantizer
a = roundq(1.1,Q); b = roundq(2.34,Q); %rounded input numbers
c = roundq(a*b,Q); %operation and subsequent roundoff
num2bin(c,Q) %show contents of binary number
%
Q_PQN=qstruct(Q,'Operation','PQN'); %replace quantization by PQN
c2 = roundq(a*b,Q_PQN); %operation and addition of PQN
num2bin(c2,Q_PQN) %show contents of binary number
%
double_rounding
%
%Section L.3
qmake('name','Qs','type','floating-point','precision',24);
qmake('name','Qd','type','floating-point','precision',53);
qmake('name','Q_PQN','precision',24,'operation','PQN');

u=0.25*sin([1:100]/100*2*pi*5);
Q='Qs';
%Define coefficients and inputs as qdata objects:
b1=qdata(0.6,Q); a1=qdata(0.8,Q); a2=qdata(0.21,Q);
uq=qdata(u,Q); y=qdata(zeros(size(u)),Q);
%
for k=3:length(uq)
   y(k)=-a1*y(k-1)-a2*y(k-2)+b1*uq(k-1);
end

qmake('name','Qmem','type','fixed-point','Bits',16,'fract',15);
qmake('name','Qacc','type','fixed-point','Bits',40,'fract',30);
u=0.25*sin([1:100]/100*2*pi*5);
%Define coefficients and inputs as qdata objects:
b1=qdata(0.6,'Qmem'); a1=qdata(0.8,'Qmem'); a2=qdata(0.21,'Qmem');
uq=qdata(u,'Qmem'); y=qdata(zeros(size(u)),'Qmem');
%Increase precision of coefficients to the accumulator:
b1=qdata(b1,'Qacc'); a1=qdata(a1,'Qacc'); a2=qdata(a2,'Qacc');
%
%This is the simulation cycle:
for k=3:length(uq)
  y(k)=-a1*y(k-1)-a2*y(k-2)+b1*uq(k-1);
end

if exist('\fixedpoint\fixedpoint\quantizer')
  test_fptb_examples %call file of examples
end
