function testqdsp_rand(k)
%TESTQDSP_RAND  Test QDSP basics with random numbers

disp('QDSP basics with random numbers ...')
if nargin==0, k=[]; end, if isempty(k), k=5; end
if isstr(k), k=str2num(k); end

for ii=1:200
  x1=randn(1,1); x2=randn(1,1); x3=randn(1,1);
  Q='double';
  x1q=qdata(x1,Q); x2q=qdata(x2,Q); x3q=qdata(x3,Q);
  if ~isequal(x1,double(x1q))|~isequal(x2,double(x2q))|~isequal(x3,double(x3q))
    error('IEEE_double error')
  end
  if ~isequal(x1+x2,double(x1q+x2q))
    ii
    xstr=num2bin(x1+x2)
    xq=x1q+x2q
    error('IEEE_double addition error')
  end
  if ~isequal(x1-x2,double(x1q-x2q))
    ii
    xstr=num2bin(x1-x2)
    xq=x1q-x2q
    error('IEEE_double subtraction error')
  end
  if ~isequal(x1*x2,double(x1q*x2q))
    ii
    xstr=num2bin(x1*x2)
    xq=x1q*x2q
    error('IEEE_double multiplication error')
  end
  if ~isnan(x1/x2)&~isequal(x1/x2,double(x1q/x2q))
    ii
    xstr=num2bin(x1/x2)
    xq=x1q/x2q
    error('IEEE_double division error')
  end
end
%
for ii=1:k
  x1=randn(1,1); x2=randn(1,1); x3=randn(1,1);
  x4=abs(randn(1,1)); x5=randn(1,1); x6=randn(1,1);
  for Q={'double','single','BigFP','Q16bit','Q32bit'};
    Q=Q{1}; 
    if ~isquant(Q)
      Q=qmake('name',Q,Q(2:end));
    end
    x1q=qdata(x1,Q);
    x2q=qdata(x2,Q);
    x3q=qdata(x3,Q);
    x4q=qdata(x4,Q);
    x5q=qdata(x5,Q);
    x6q=qdata(x6,Q);
    y=x1+x2*x3+x5/(x4+1)-x6;
    yq=x1q+x2q*x3q+x5q/(x4q+1)-x6q;
    if strcmp(Q,'double')
      if ~isequal(x1,double(x1q))|~isequal(x2,double(x2q))|~isequal(x3,double(x3q))|...
          ~isequal(x4,double(x4q))|~isequal(x5,double(x5q))|~isequal(x6,double(x6q))
        error('IEEE_double error')
      end
      yqlim=0;
    elseif strcmp(Q,'BigFP')
      yqlim=double(lsb(y)+...
        lsb(x1+x2*x3+x5/(x4+1))+lsb(x1+x2*x3)+...
        lsb(x1)+lsb(x2*x3)+lsb(x2)*abs(x3)+lsb(x3)*abs(x2)+...
        lsb(x5/(x4+1))+abs(x5)*3*lsb(1/(x4+1))+lsb(x5)*abs(1/(x4+1))+lsb(x6))/1.95;
    else %fixed-point
      yqlim=double(lsb(yq)+...
        lsb(x1q+x2q*x3q+x5q/(x4q+1))+lsb(x1q+x2q*x3q)+...
        lsb(x1q)+lsb(x2q*x3q)+lsb(x2q)*abs(x3q)+lsb(x3q)*abs(x2q)+...
        lsb(x5q/(x4q+1))+abs(x5q)*3*lsb(1/(x4q+1))+lsb(x5q)*abs(1/(x4q+1))+lsb(x6q))/1.95;
    end
    %y=x1+x2*x3+x5/(x4+1)-x6;
    if abs(double(yq)-y)>yqlim
      yq,ystr=['      ',num2bin(y)],yqstr=['      ',num2bin(double(yq))]
      error(['error is too large for quantizer ''',Q,''''])
    end
  end %for Q
end %for ii=1:k