function testroundfloat(x,p,xq,typ,fb)
%TESTROUNDFLOAT
%
%      testroundfloat(x,p,xq,typ,fb)
%        x = unquantized number
%        p = precision (B-1 for fixed-point)
%        xq = quantized number
%        typ = 'fixed-point' or 'floating-point'
%        fb = fractionalbits (for fixed-point)

if nargin<3, return, end
if nargin<4, typ=''; end, if isempty(typ), typ='floating-point'; end
if nargin<5, fb=[]; end, if isempty(fb), fb=0; end

if strcmp(typ,'floating-point')
  for ii=1:4 %command line,roundfloat,rounding,qdsp
    if ii==1
      [f,e]=log2(x); dxp = sign(x).*pow2(max(e,-1021)+52-p); xqfl = (x+dxp)-dxp;
      str='command line';
    elseif ii==2
      xqfl=roundfloat(x,p);
      str='function roundfloat';
    elseif ii==3
      Q=qstruct('Q','precision',p);
      xqfl=roundq(x,Q);
      str='function roundoff';
    elseif ii==4
      qclear Qfl
      Qfl=qmake('Qfl','precision',p);
      xqflobj=qdata(x,Qfl); xqfl=double(xqflobj);
    str='QDSP';
    end
    fprintf(['Floating-point quantization, ',str,', p = %.0f, numbers before/after:\n'],p)
    num2bin(x), num2bin(xqfl)
    if ~isequal(xqfl,xq), error('Floating test error'), end
  end
elseif strcmp(typ,'fixed-point')
  B=p+1; q=2^(-fb);
  for ii=1:4 %command line,roundrand,rounding,qdsp
    Q=qstruct('Q','type','fixed-point','Bits',B,'fractionbits',fb);
    if ii==1
      xqc=q*round(x/q);
      str='command line';
    elseif ii==2
      xqc=q*roundrand(x/q);
      str='function roundrand';
    elseif ii==3
      xqc=roundq(x,Q);
      str='function roundoff';
    elseif ii==4
      qclear Q2
      Q2=qmake('Q2','type','fixed-point','Bits',B,'fractionbits',fb);
      xqc=qdata(x,Q2);
      str='QDSP';
    end
    fprintf(['Fixed-point quantization, ',str,', B = %.0f, numbers before/after:\n'],B)
    if ismember(ii,[1:3])
      num2bin(x), num2bin(xqc,Q)
    else
      num2bin(x), xqc
    end
    if ~isequal(double(xqc),xq), error('Fixed-point test error'), end
  end
else error('typ is not allowed')
end