function qcopy(varargin)
%QCOPY     Copy full quantizer to new name.
%   QCOPY SOURCE DEST makes a copy of quantizer named SOURCE with name DEST.
%   It is also possible to give the number of source quantizer instead of SOURCE.
%   If second argument is missing it results in a quantizer named copyof_SOURCE.
%
%   Examples:
%     qcopy quantizer_a quantizer_b
%     qcopy 1 quantizer_b
%     qcopy quantizer_a       (results in a quantizer named copyof_quantizer_a)
%     qcopy 1                 (results in a quantizer named copyof_quantizer_a)
%
%   See also: QMAKE, QSAVE, QGET, QSET

%   Copyright (C) 2003 by JUHASZ Laszlo

error(nargchk(1,2,nargin))
[x,err] = str2int(varargin); error(err)
x = celldisint(x);

% get the quantizer
[t,err] = quantizers('get',x{1}); error(err)

if length(x)==1
   % only one argument
   t.name = ['copyof_',t.name];
else
   % two arguments
   t.name = x{2};
end

% create new
[t,err] = quantizers('set','-new',t); error(err)
