function qload(varargin)
%QLOAD     Load quantizers from disk.
%   QLOAD FNAME  retrieves the quantizers from the MAT-file 'fname.mat'.
%
%   QLOAD, by itself, loads from the file named 'qdsp.mat'.
%
%   QLOAD FNAME X Y Z ... loads just the specified quantizers.  The
%   wildcard '*' can be used to load variables that match a pattern.
%   (not implemented!)
%
%   Use the functional form of QLOAD, such as QLOAD('fname'), when the
%   file name is stored in a string or when an output argument is
%   requested.
%   
%   See also QSAVE.

% (C) 2003, Juhasz Laszlo

if isempty(varargin)
   try
      load('qdsp','quantizer_variables')
   catch
      error(lasterr)
   end
   for k = 1:length(quantizer_variables)
      [null,err] = quantizers('set','-new',quantizer_variables(k)); error(err)
   end
   return
end

[x,err] = str2int(varargin); error(err)
x = celldisint(x);

if length(x)==1
   try
      load(x{1},'quantizer_variables')
   catch
      error(lasterr)
   end
   for k = 1:length(quantizer_variables)
      [null,err] = quantizers('set','-new',quantizer_variables(k)); error(err)
   end
   return
end

% saving out filename
fname = x{1};
x(1) = [];

try
   load(fname,'quantizer_variables')
catch
   error(lasterr)
end
for k = 1:length(x)
   switch class(x{k})
   case 'char'
      for m = 1:length(quantizer_variables)
         if strcmp(x{k},quantizer_variables(m).name)
            [null,err] = quantizers('set','-new',quantizer_variables(m)); error(err)
         end
      end
   case 'double'
      [null,err] = quantizers('set','-new',quantizer_variables(x{k})); error(err)
   otherwise error('Internal error')
   end
end
