function qsave(varargin)
%QSAVE     Save quantizers to disk. 
%   QSAVE FNAME saves all quantizers to the binary 'MAT-file' named fname.mat.
%   The data may be retrieved with QLOAD. Omitting the filename causes QSAVE
%   to use the default filename 'qdsp.mat'.
%
%   QSAVE FNAME X  saves only the quantizer with name/number X.
%   QSAVE FNAME X Y Z  saves X, Y, and Z. The wildcard '*' can be used to
%   save only those variables that match a pattern (not implemented!).
%
%   Use the functional form of QSAVE, such as
%   QSAVE('fname','quantizer_name_1',quantizer_number_2)
%   when the filename or quantizer names/numbers are stored in strings/variables.
%
%   See also QLOAD.

% Copyright (C) 2003 by JUHASZ Laszlo

if isempty(varargin)
   [quantizer_variables,err] = quantizers('get','-all'); error(err)
   save('qdsp','quantizer_variables')
   return
end

[x,err] = str2int(varargin); error(err)
x = celldisint(x);

if length(x)==1
   [quantizer_variables,err] = quantizers('get','-all'); error(err)
   save(x{1},'quantizer_variables')
   return
end

% saving out filename
fname = x{1};
x(1) = [];

for k = 1:length(x)
   [t,err] = quantizers('get',x{k}); error(err)
   quantizer_variables(k) = t;
end
save(fname,'quantizer_variables')
