function [F,xF,x2F]=cdf_gauss(x,sigm,mu,xc)
%CDF_GAUSS  Gaussian Cumulative Distribution Function
%
%       [F,xF,x2F]=cdf_gauss(x,sigma,mu,xc)
%
%       F: CDF
%       xF: integral of (x-xc)*f(x)
%       x2F: integral of (x-xc)^2*f(x)
%          mu can be a vector, then x must be scalar

if nargin<4, xc=[]; end, if isempty(xc), xc=0; end
if nargin<3, mu=[]; end, if isempty(mu), mu=0; end
if nargin<2, sigm=[]; end, if isempty(sigm), sigm=1; end
if (length(x)>1)&(length(mu)>1), error('x and mu are vectors the same time'), end
F=(erf((x-mu)/sqrt(2)/sigm)+1)/2;
if nargout>1
  fx=1/sqrt(2*pi)/sigm*exp(-(x-mu).^2/sigm^2/2);
  xF=-sigm^2*fx+mu.*F; xF0=xF;
  if ~isequal(xc,0), xF=xF-xc*F; end
end
if nargout>2
  if length(x)==1, x2F=-sigm^2*x*fx+sigm^2*F+mu.*xF0;
  else x2F=-sigm^2*x.*fx+sigm^2*F+mu*xF0;
  end
  if ~isequal(xc,0), x2F=x2F-2*xc*xF0+xc^2*F; end
end
