function str=num2bin(xprime,runmod,expmod)
%NUM2BIN  Contents of a number as binary string
%
%   str=num2bin(xprime,runmod)
%
%   When Ebias is defined as a field of runmod, the biased exponent is shown.
%
%   Examples:
%     num2bin(1)
%     num2bin(qfloat(realmin,'list'));
%     num2bin(0,struct('Precision',24,'Emin',-26,'Ebias',27,'lb','shown'))
%     num2bin(0,'single')
%     num2bin(1.75,struct('Bits',6,'FractionBits',2)) %+001.11
%     num2bin(-1.75,struct('Bits',6,'FractionBits',4,'coding','t')) %1110.01
%     num2bin(3.125,'fixed-point') %+11.001 %shortest fixed-point string with sign
%     %Matlab 7.x only:
%     num2bin(1+double(eps('single')),'single')
%         %this returns +(1).0000 0000 0000 0000 0000 001 2^[+0]

%	Copyright (c) I. Kollar, 1995-2005, all rights reserved
%	Last modified: 17-Feb-2006

%expmod: 'd' or decimal' to represent the exponent as a digital number
%        'b' or 'binary' to represent the exponent as biased binary number (default)

FractionBits=[]; Bits=[]; Precision=[]; if nargin<2, runmod=''; end
shortfixedpoint=0;
if nargin<3, expmod=''; end, if isempty(expmod), expmod='binary'; end
if nargin>1,
  if isstr(runmod)&strncmp(runmod,'fixed-point',3)
    if rem(xprime,1)==0, Q=qstruct('bits',54,'fractionbits',0,'coding','s');
    else fb=54-1-max(0,ceil(log2(abs(xprime)))); Q=qstruct('bits',54,'fractionbits',fb,'coding','s');
    end
    lb='s'; shortfixedpoint=1;
  elseif isstr(runmod)&(strncmp(runmod,'32-',3)|strncmp(runmod,'Q32',3))
    Q=qstruct('Q32');
    lb='s';
  elseif isstr(runmod)&strncmp(runmod,'segm',4)
    Q=qstruct('segments');
    [f,e]=log2(abs(xprime(1))); xprime=pow2(xprime,22-e);
    newrxp=fix(xprime(end)); remxprime=xprime(end)-newrxp; xprime(end)=newrxp;
    lb='s';
  elseif isstr(runmod)&(strncmp(runmod,'16-',3)|strncmp(runmod,'Q16',3))
    Q=qstruct('Q16');
    lb='s';
  else%if isstruct(runmod)|isstr(runmod)
    if isempty(runmod)
      Q=qstruct('double');
    else
      Q=qstruct(runmod);
    end
  end
else
  Q=qstruct('IEEE_double','double');
end

if ~isempty(Q) 
  fn=fieldnames(Q);
  for ii=1:length(fn)
    if strcmpi(fn{ii},'type'), typ=Q.Type;
    else eval([fn{ii},'=getfield(Q,fn{ii});'])
      if strcmp(fn{ii},'Name'), name=Name;
      elseif strcmp(fn{ii},'Precision'), p=Precision;
      elseif strcmp(fn{ii},'Bits'), B=Bits;
      elseif strcmp(fn{ii},'FractionBits'), fb=FractionBits;
      elseif strcmp(fn{ii},'RoundingDir'), rdir=RoundingDir;
      elseif strcmp(fn{ii},'TreatHalf'), treath=TreatHalf;
      elseif strcmp(fn{ii},'Coding'), coding=Coding;
      elseif strcmp(fn{ii},'OverFlow'), ovf=OverFlow;
      elseif strcmp(fn{ii},'Minimum'), xnmin=Minimum;
      elseif strcmp(fn{ii},'Maximum'), xpmax=Maximum;
      elseif strcmp(fn{ii},'ExpMinimum'), Emin=ExpMinimum;
      elseif strcmp(fn{ii},'ExpMaximum'), Emax=ExpMaximum;
      elseif strcmp(fn{ii},'ExpBias'), Ebias=ExpBias;
      elseif strcmp(fn{ii},'Operation'), op=Operation;
      elseif strcmp(fn{ii},'LeadingBit'), lb=LeadingBit;
      elseif strcmp(fn{ii},'UnderFlow'), ufl=UnderFlow;
          
      elseif strcmpi(fn{ii},'Dither'), dith=Dither; 
      elseif strcmpi(fn{ii},'ToD'), tod=ToD;
      elseif strcmpi(fn{ii},'Number_of_DitherBits'), L=Number_of_DitherBits;
      elseif strcmpi(fn{ii},'Amplitude_of_Sinusoid_Dither'), A=Amplitude_of_Sinusoid_Dither;  
      elseif strcmpi(fn{ii},'Ratio_of_Frequencies') 
          RoFn=Ratio_of_Frequencies_numerator;  
          RoFd=Ratio_of_Frequencies_denumerator; 
      elseif strcmpi(fn{ii},'Phase'), ph=Phase;
      elseif strcmpi(fn{ii},'Variance_of_Gaussian_Dither'), gvar=Variance_of_Gaussian_Dither;
      elseif strcmpi(fn{ii},'DFunction'), dfun=Dfunction;
      elseif strcmpi(fn{ii},'Sequenceperm'), seqp=Sequenceperm;
       elseif strcmpi(fn{ii},'Other'), other=Other;    
          
      else error(['Unknown field ''',fn{ii},''''])
      end
    end
  end
end

if ~exist('coding'), coding=''; end, if isempty(coding), coding='s'; end
if ~exist('lb'), lb='h'; end
if ~exist('Precision'), Precision=[]; end
if ~exist('Emin'), Emin=-1022; end
if ~exist('Ebias'), Ebias=-Emin+1; end
if ~isempty(Bits)&~isempty(Precision)&~isequal(Bits,Precision+~strncmp(coding,'u',1))
  error('Both Precision and Bits are given, inconsistently with coding')
elseif ~isempty(Bits)&isempty(Precision)
  Precision=Bits-~strncmp(coding,'u',1);
end
if isempty(Precision), Precision=53; end

if ~isempty(xprime)
  if strcmp(typ,'floating-point')&~shortfixedpoint
    if ~isequal(xprime,qfloat(xprime,qstruct('Precision',Precision,'Emin',Emin,'lb',lb)))
      if ~strcmp(Q.Operation,'PQN')
        if isequal(Emin,Emax)
        else error('Number is not obtained by floating-point quantization')
        end
      end
    end
  elseif strcmp(typ,'fixed-point')&...
      ~isequal(xprime,qfix(xprime,qstruct('Bits',Bits,'FractionBits',FractionBits)))
    if ~strcmp(Q.Operation,'PQN')
      warning('Number is not obtained by fixed-point quantization')
    end
  end
end

xprimev=xprime; iv=0;
for xprime=xprimev(:)'
  iv=iv+1;
  if strcmp(typ,'floating-point') %floating-point
    if ~isempty(Ebias)&~isequal(Ebias,0)&(Ebias<=-Emin)
      error(sprintf('Ebias = %.0f is not larger than -Emin = %.0f',Ebias,-Emin))
    end
    [f,e]=log2(xprime);
    f=f*2; e=e-1;
    if e<Emin, f=pow2(f,e-Emin); e=Emin; end
    if e>Emax, f=pow2(f,e-Emax); e=Emax; end
    if isequal(f,0), e=Emin; end
    xstr=''; fa=abs(f);
    if xprime>=0, xstr='+'; else xstr='-'; end
    if isnan(fa), xstr=[xstr,'NaN'];
    elseif ~isfinite(fa), xstr=[xstr,'Inf'];
    else
      blength=Precision; if strcmp(Q.Operation,'PQN'), blength=53; end
      for ii=1:blength
        if ii==Precision+1, xstr=[xstr,' ('];
        elseif (ii>4)&(rem(ii,4)==2), xstr=[xstr,' '];
        end
        if ii==1
          if fa>0, excessintbits=floor(log2(fa));
          else excessintbits=0;
          end
          blmod=0;
          for ib=1:excessintbits
            if strcmp(lb,'h'), error('Mantissa >= 2 with hidden leading bit'), end
            xstr=[xstr,num2str(fa>=pow2(excessintbits-ib+1))];
            fa=fa-pow2(excessintbits-ib+1); blmod=blmod+1;
          end
          if strcmp(lb,'h'), xstr=[xstr,'(']; end
        end
        if fa>=pow2(1-ii)
          xstr=[xstr,'1'];
          fa=fa-pow2(1-ii);
          if ii==1, ebin=e; end
        else
          xstr=[xstr,'0'];
          if (ii==1)&strcmp(UnderFlow,'g'), ebin=e-1; end
        end
        if ii==1
          if strcmp(lb,'h'), xstr=[xstr,')']; end
          %if strcmp(lb,'s'), xstr=[xstr,'(']; end
          xstr=[xstr,'('];
          xstr=[xstr,'.'];
          xstr=[xstr,')'];
        end
        if isequal(ii,blength-blmod), break, end
      end %for ii
      if strcmp(Q.Operation,'PQN'), xstr=[xstr,')']; end
      %end of significand
      %
      if shortfixedpoint
        ind=[findstr(xstr,'('),findstr(xstr,' '),findstr(xstr,')')]; if ~isempty(ind), xstr(ind)=''; end
        ind=findstr(xstr,'.');
        if ebin>0
          if length(xstr)-ind<ebin, xstr=[xstr,setstr('0'*ones(1,ebin-length(xstr)+ind))]; end
          xstr=[xstr(1:ind-1),xstr(ind+1:ind+ebin),'.',xstr(ind+ebin+1:end)];
        elseif ebin<0
          xstr=[xstr(1),setstr('0'*ones(1,-ebin-1)),xstr(2:end)];
          ind=findstr(xstr,'.');
          xstr=[xstr(1),'.',xstr(2:ind-1),xstr(ind+1:end)];
        end
        while strcmp(xstr(end),'0')&((length(xstr)>3)|any(findstr(xstr,'1'))), xstr(end)=''; end
        if strcmp(xstr(end),'.'), xstr(end)=''; end
      else
        %Exponent:
        %if strncmpi(expmod,'b',1)
        ebin=ebin+Ebias;
        if isequal(Emin,Emax)
          Ebits=0; ebits=0;
        else
          Ebits=ceil(log2(abs(Emin)))+1;
        end
        if ebin~=0, Ebits=max(ceil(log2(abs(ebin))),Ebits); end
        xstr=sprintf([xstr,' (*2^)']);
        if ebin<-1
          warning('Biased exponent is negative')
          xstr=[xstr,'-']; ebin=-ebin;
        end
        %xstr0=0;
        for ii=1:Ebits
          if (ii>4)&(rem(ii,4)==1), xstr=[xstr,' ']; end
          if ebin>=pow2(Ebits-ii), xstr=[xstr,'1']; ebin=ebin-pow2(Ebits-ii);
          else xstr=[xstr,'0'];
          end
          %xstr0=0;
        end
        %xstr=sprintf([xstr,' (*2^[%.0fd])\n'],esaved);
        %else %decimal
        xstr=sprintf([xstr,' [%.0fd]'],e);
        %end
      end
    end %shortfixedpoint
  else %fixed-point
    xstr=' '; xneg=xprime<0;
    if strncmp(coding,'u',1)
    elseif xprime>=0
      if strncmp(coding,'t',1)|strncmp(coding,'o',1), xstr='0';
      else xstr='+';
      end
    else %xneg
      if strncmp(coding,'t',1)|strncmp(coding,'o',1), xstr='1';
      else xstr='-';
      end
    end
    [lsbmag,lsbit]=lsb(xprime,qstruct('Bits',Bits,'FractionBits',FractionBits));
    if strncmp(coding,'t',1)&xneg, xprime=abs(xprime); xprime=xprime-lsbmag;
    else xprime=abs(xprime);
    end
    for ii=1:Precision
      if xor(rem(xprime,pow2(lsbmag,ii))==0,(~xneg|strncmp(coding,'s',1)|strncmp(coding,'u',1)))
        xstr=[xstr(1),'1',xstr(2:end)];
      else
        xstr=[xstr(1),'0',xstr(2:end)];
      end
      if rem(xprime,pow2(lsbmag,ii))>0
        xprime=xprime-pow2(lsbmag,ii-1);
      end
      if FractionBits==ii, xstr=[xstr(1),'(.)',xstr(2:end)]; end
    end
    ind=findstr(xstr,' '); if ~isempty(ind), xstr(ind)=''; end
  end
  if shortfixedpoint
    while any(find(xstr=='.'))&strcmp(xstr(end),'0')&(length(xstr)>3), xstr(end)=''; end
    if strcmp(xstr(end),'.'), xstr(end)=''; end
  end
  if nargout>0
    if iv==1, str=xstr; else str=[str;xstr]; end
    if (iv==length(xprimev))&isstr(runmod)&strncmp(runmod,'segm',4)&remxprime
      str=[str;num2bin(remxprime,'Q32')];
    end
  else
    fprintf(['',xstr,'\n'])
    if (iv==length(xprimev))&isstr(runmod)&strncmp(runmod,'segm',4)&remxprime
      fprintf(['',num2bin(remxprime,'Q32'),'\n'])
    end
  end
end
%
%End of num2bin