function [cf,cf1,cf2,cf3]=cfrayleigh(u,alpha,mu)
%CFRAYLEIGH Characteristic function and derivatives of a Gaussian variable.
%
%	[cf,cf1,cf2,cf3]=cfrayle(u,alpha,mu)
%
%	Definitions:
%	f(x) = x/alpha^2*exp(-x^2/(2*alpha^2))
%		x>=0, alpha>0
%	F(u) = ??? Not known - cfrayle does not work yet!
%
%	Output arguments:
%       cf = values of the characteristic function at u
%       cf1 = first derivative of the characteristic function at u
%       cf2 = second derivative of the characteristic function at u
%       cf3 = third derivative of the characteristic function at u
%
%	Input arguments:
%	u = values of the input argument (may be a vector or an array)
%	alpha = standard deviation (default value: 1)
%	mu = additional mean value (default value: zero)

%	See also:

%       Copyright (c) I. Kollar, 1994
%       All rights reserved.
%       Last modified: 30-Jul-1994

if nargin<3, mu=[]; end, if isempty(mu), mu=0; end
if length(mu)>1, error('mu is not a scalar'), end
if imag(mu)~=0, error('mu is not real'), end
if nargin<2, alpha=[]; end, if isempty(alpha), alpha=1; end
if length(alpha)>1, error('alpha is not a scalar'), end
if imag(alpha)~=0, error('alpha is not real'), end
if alpha<=0, error('alpha is not positive'), end
if any(any(imag(u)~=0)), error('u is not real'), end
%
error('Analytic calculation of the cf of the Rayleigh distribution not ready')
cf=[];
cf1=[]; cf2=[]; cf3=[];
if nargout>=2
  cf1=[];
end
if nargout>=3
  cf2=[];
end
if nargout>=4
  cf3=[];
end
%
%Combine with mu if it differs from zero
if mu~=0
  [cf,cf1,cf2,cf3]=cfmu(u,mu,cf,cf1,cf2,cf3);
end %mu~=0
%
%End of cfrayleigh
