function [fn,propsugg]=qprops(k)
%QPROPS  Display properties of a quantizer

qprop={'Name' '' 'name of quantizer'
  'Type' '' 'type of quantizer (''fixed-point'' or ''floating-point'')'
  'RoundingDirection' 'rdir'   ['direction of rounding, ''r'', ''f'', ''c'' ''x'', ''i'', ''t''\n',...
  '      for round, floor, ceil, fix, toinf, trunc']
  'TreatHalf' 'treath'   ['way of treating *.5 values for round: ''f'', ''c'', ''x'', ''e'', ''o'', ''r''\n',...
  '      for floor, ceil, fix, toinf, toeven, toodd, rand']
  'Coding' '' ['coding of mantissa, ''s'', ''o'', ''t''\n',...
  '      for sign-magnitude, one''s complement, two''s complement']
  'Dither' '' 'adding dither, ''on'' or ''off'''
  'Operation' '' 'way of operation, ''quantize'' or ''PQN'' to add PQN instead'
  'Bits' '' 'number of bits of the number, sign included'
  'Minimum'  'xnmin'  'minimum representable (negative) number'
  'Maximum'  'xpmax'  'maximum representable number'
  'FractionBits' 'fb' 'number of fractional bits'
  'OverFlow'  'ovf'  ['overflow handling, ''c'', ''m'', ''t'', ''i'', ''n''\n',...
  '      for clip, modular, triang, inf, none']
  'Precision' '' ['precision, number of bits of the significand, sign not\n',...
  '      included, but hidden bit is included']
  'LeadingBit' 'lb' ['leading bit representation, ''h'' or ''hidden'' for hidden,\n',...
  '      ''s'' or ''shown'' for shown']
  'UnderFlow'  'ufl' 'underflow, ''g'' for gradual, ''f'' for flush'
  'ExpMinimum'  'emin'  'minimum value of exponent (minimum: -1022)'
  'ExpMaximum'  'emax'  'maximum value of exponent (maximum: 1023)'
  'ExpBias'  'ebias'  'bias in exponent representation'
  };

fixi=strmatch('Bits',qprop(:,1));
floati=strmatch('Precision',qprop(:,1));
ditheri=strmatch('Dither',qprop(:,1));
typeofditheri=size(qprop,1)+1;

if nargin==0, k=0; end
if ~isequal(k,0)
  qprop=[qprop;{...
    'Type_of_Dither', 'ToD', 'Type of dither: ''u1'', ''u2'', ...'
    'Number_of_DitherBits', 'L', 'Number of bits in dither'
    'Amplitude', 'A', 'Dither amplitude'
    'Phase', '', 'Dither phase'
    %elseif strncmpi(fn{ii},'Var',3)|strncmpi(fn{ii},'Variance_of_Gaussian_Dither',8)
    %elseif strncmpi(fn{ii},'RoF',3)|strcmpi(fn{ii},'Ratio_of_Frequencies')     
    %elseif strncmpi(fn{ii},'DFunction',4)|strncmpi(fn{ii},'DFunction_given_by_User',9)
    %elseif strncmpi(fn{ii},'Sequenceperm',3)
    }];
end
if isnumeric(k)
  if k==0
    fni={'Quantizer properties and values:'};
    for ii=1:size(qprop,1)
      if ii==fixi, fni=[fni;{'Fixed-point quantizers only:'}];
      elseif ii==floati, fni=[fni;{'Floating-point quantizers only:'}];
      end
      if isempty(qprop{ii,2})
        fni=[fni;{[qprop{ii,1},' = ',qprop{ii,3}]}];
      else
        fni=[fni;{[qprop{ii,1},' or ',qprop{ii,2},' = ',qprop{ii,3}]}];
      end
    end %for ii
    if nargout==0
      for ii=1:size(fni,1)
        if ~strcmp(':',fni{ii}(end)), fprintf('  '), end
        fprintf([fni{ii},'\n'])
      end
    else
      fn=fni;
    end
  elseif any(ismember(k,[1,2,3])), fni=qprop(:,k);
  else error('Error in qprops')
  end
elseif isstr(k)
  if ~strcmpi(k,'Dither')
    ind=strmatch(lower(k),lower(qprop(:,1)));
    if isempty(ind), ind=strmatch(lower(k),lower(qprop(:,2))); end
    if length(ind)==0, error(['Property ''',k,''' not found'])
    elseif length(ind)>1, error(['Ambiguous property ''',k,''''])
    else fn=qprop{ind,1}; propsugg=qprop{ind,3};
    end
  else %Dither
    fni={'Dither properties and values:'};
    for ii=[ditheri,typeofditheri:size(qprop,1)]
      if isempty(qprop{ii,2})
        fni=[fni;{[qprop{ii,1},' = ',qprop{ii,3}]}];
      else
        fni=[fni;{[qprop{ii,1},' or ',qprop{ii,2},' = ',qprop{ii,3}]}];
      end
    end %for ii
    if nargout==0
      for ii=1:size(fni,1)
        if ~strcmp(':',fni{ii}(end)), fprintf('  '), end
        fprintf([fni{ii},'\n'])
      end
    else
      fn=fni;
    end    
  end
end
