function Qout=qstruct(varargin)
%QSTRUCT Form structure of quantizer properties
%
%   Qout=qstruct(qdef,prop1,prop1val,prop2,prop2val)
%
%   Input arguments (optional):
%     qdef = structure of properties (fields) with values
%     propi,propival = property/value pairs
%     properties and values: see 'qstructget'
%
%   Defaults:
%     'double', 'single', '16-bit', 'Q16', '32-bit', 'Q32'
%     IEEE double precision: p=53, lb='hidden', rdir='round',
%         treath='toeven', coding='s', ufl='gradual', Emin=-1022, Emax=1023
%     IEEE single precision: p=24, lb='hidden', rdir='round',
%         treath='toeven', coding='s', ufl='gradual', Emin=-126, Emax=127
%   Examples:
%     qstruct('Qd','double') %IEEE double precision
%     qstruct('Qs','single') %IEEE single precision
%     Q=qstruct('Q',struct('Precision',18)) %floating-point with precision 18
%     qstruct('Q16','16-bit') %16-bit fixed point, two's compl, modular overflow, integer
%     qstruct('Q32','32-bit') %32-bit fixed point, two's compl, modular overflow, integer
%     qstruct('16-bit-sm','Bits',16,'coding','s') %16-bit fixed point, sign-mag, modular overflow, integer

%	Copyright (c) I. Kollar, 1995-2006, all rights reserved
%	Last modified: 22-Sep-2006

%     qstruct('Qsegm','segments') %23-bit fixed point, two's compl, modular overflow, integer

if 0, dither=0; end %bypass earlier error in parser

typ='';
fb=[]; lb=''; rdir=''; treath=''; coding=''; ufl=''; Emin=[]; Emax=[]; Ebias=[];
expcoding='';
xnmin=[]; xpmax=[]; p=[]; B=[]; ovf=''; op='quantize'; Name='';
dith='';
other.Sdith=[];
other.Probab=[];
other.Hmtimes=[];
other.SumHmtimes=[];
other.NumberOfUsage=[];

%
if nargin==0
  Qout=qstruct('double'); return
  %qstructget; return
end
if (nargin>=1)&isstr(varargin{1}) %maybe binary string
  if ~isempty(varargin{1})&all(ismember(varargin{1},'01+-. ()[]<>@eE^')) & ...
      ~strncmpi(varargin{1},'32-',3)&~strncmpi(varargin{1},'16-',3)&~strncmpi(varargin{1},'8-',2)
    [val,Qout]=bin2num(varargin{:});
    return
  end
end
if (nargin>=2)&isstr(varargin{1})&isstr(varargin{2}) %maybe binary string
  if all(ismember(varargin{2},'01+-. ()[]<>@eE^'))
    [val,Qout]=bin2num(varargin{2:end});
    Qout.Name=varargin{1};
    return
  end
end

keyw={'double','single','16-bit','32-bit','segments','s','d'};
%Handle name
if isstr(varargin{1})&(strncmpi(varargin{1},'Bits',1)|strncmpi(varargin{1},'FractBits',2))
  varargin=[{'name','','type','fixed-point'},varargin]; qdef='16-bit';
elseif isstr(varargin{1})&strcmp(varargin{1},'p')
  varargin=[{'name','','type','floating-point'},varargin];
elseif isstr(varargin{1})&(length(varargin)>=3)&isstr(varargin{3})&...
    (strncmpi(varargin{3},'Bits',1)|strncmpi(varargin{3},'FractBits',2))
  varargin=[{'name','','type','fixed-point'},varargin]; qdef='16-bit';
elseif (length(varargin)>=1)&isstr(varargin{1})&any(strmatch(varargin{1},keyw,'exact'))
  %csak 1-2 arg ***????
  if length(varargin)==1, qdef=varargin{1};
  else %>1
    varargin{1}=qstruct(varargin{1});
    if length(varargin)==2
      varargin{1}=setfield(varargin{1},'Name',varargin{2});
    else
      for ii=3:2:length(varargin)
        varargin{1}=setfield(varargin{1},varargin{ii-1},varargin{ii});
      end
      if ii>=3, varargin{1}=qstruct(varargin{1}); end
    end
    if (length(varargin)>3)&(rem(length(varargin),2)==0)
      error(sprintf('Invalid number of input parameters for standard quantizer: %.0f',length(varargin)))
    end
    varargin(2:end)=[];
  end
end
%isnamegiven=0;
%if isstr(varargin{1})&strncmpi(varargin{1},'name',3), isnamegiven=1;
%else isnamegiven=0;
%end
%if isnamegiven, varargin(1)=[]; end
if length(varargin)>0
  isstrvin=0; isstructvin=0;
  for ii=2:2:length(varargin)
    if isstr(varargin{ii})
      if (length(varargin)==1)&any(strmatch(varargin{2},keyw,'exact')), isstrvin=1;
      elseif length(varargin)>ii, isstrvin=1;
      elseif (length(varargin)==2)&isstr(varargin{2})&any(strmatch(varargin{2},keyw,'exact'))
        isstrvin=1;
      else isstrvin=0; break,
      end
    end
  end
  if ((length(varargin)==2)|(length(varargin)==3))&isstruct(varargin{2}), isstructvin=1; end
  if isstruct(varargin{1}) %quantizer given as structure, accepted
    qdef=varargin{1};
  else
    if isstr(varargin{1}) %maybe name
      namegiven=0;
      if isstrvin|isstructvin
        if strncmpi(varargin{1},'name',3)
          namegiven=1; varargin(1)=[];
        end
        Name=varargin{1}; varargin(1)=[];
        inde=strmatch(Name,keyw,'exact');
        ind=strmatch(Name,keyw);
        if any(inde)&~namegiven
          Q=qstruct(Name); varargin=[{Q},varargin]; Name=''; namegiven=0;
        elseif any(ind)&~isempty(Name)&~strcmp(Name,'Q')&~strcmp(Name,'Q1')&~strcmp(Name,'Q16')&...
            ~strcmp(Name,'Q8')&~strcmp(Name,'Q32')&~strcmp(Name,'Q3')
          warning(['Potentially misleading quantizer name ''',Name,''' (similar to keyword ''',keyw{ind},''')'])
          namegiven=1;
        end
        if isstrvin
          if any(strmatch(Name,keyw,'exact'))
            if length(varargin)>1, qdef=struct(varargin{:}); end
            namegiven=0;
          else
            if length(varargin)==1, qdef=qstruct(varargin{1});
            elseif isstruct(varargin{1})
              for ii=3:length(varargin)
                varargin{1}=setfield(varargin{1},varargin{ii-1},varargin{ii});
              end
              qdef=varargin{1};
            elseif isstrvin
              if rem(length(varargin),2)==0, qdef=struct(varargin{:});
              elseif rem(length(varargin),2)==1, Q=varargin(1:end-1); qdef=struct(Q{:});
              end
            end
            namegiven=1;
          end
        elseif isstructvin
          qdef=varargin{1};
        end
      %elseif strmatch(varargin{1},keyw)
      %else
      %  error(['qdef = ''',varargin{1},''' ???'])
      end
    else
      if isnumeric(varargin{1})&(length(varargin{1})==1), p=varargin{1}; end
      %error('Quantizer name (first argument) is not a string')
    end
  end
end

if (length(varargin)==2)&isstruct(varargin{1})&isstr(varargin{2})&strcmp(varargin{2},'test')
  %test consistency
  Q=varargin{1};
  pmax=-log2(eps)+1; %53 for IEEE double precision
  if strncmp(Q.Type,'float',2)
    if rem(Q.Precision,1)~=0, error('p is not integer'), end
    if strncmp(Q.LeadingBit,'h',1), hbn=1;
    elseif strncmp(Q.LeadingBit,'s',1), hbn=0;
    else error(['lb=''',Q.LeadingBit,''' is not allowed'])
    end
    if Q.Precision>pmax, error(sprintf('Precision p is larger than %.0f',pmax)), end
    if Q.Precision<2, error('Bit number of significand is not positive'), end
    %
    Eminlim=round(log2(realmin)); Emaxlim=round(log2(realmax))-1;
    if Q.ExpMinimum<Eminlim
      error(sprintf('ExpMinimum=%.3g is too small for this computer, limit: %.0f',...
        Q.ExpMinimum,Eminlim))
    end
    if Q.ExpMaximum>Emaxlim
      error(sprintf('ExpMaximum=%.3g is too large for this computer, limit: %.0f',...
        Q.ExpMaximum,Emaxlim))
    end
    if Q.ExpMaximum<Q.ExpMinimum, error('ExpMaximum is smaller than ExpMinimum'), end
  elseif strncmp(Q.Type,'fixed',2) %fixed-point
    if length(Q.Bits)>1, error('Bits is not scalar'), end
    if Q.Bits<=2-~strncmp(coding,'unsigned',1), error('bit number is not positive'), end
    if rem(Q.Bits,1)~=0, error('Bits is not integer'), end
    if isfield(Q,'Precison'), p=Q.Precision; else p=Q.Bits-1; end
    if ~isempty(Q.Bits)&~isempty(p)&~isequal(Q.Bits,p+~strncmp(coding,'u',1))
      error('Both Precision and Bits are given, inconsistently with coding')
    end
    if ~isequal(rem(Q.Bits,1),0), error(sprintf('Bits = %.5g is not integer',Q.Bits)), end
    if Q.Bits>54, error(sprintf('Bits = %.f is larger than 54',Q.Bits)), end
    if Q.FractionBits<0, error('FractionBits is negative'), end
    if Q.FractionBits>Q.Bits-~strncmp(Q.Coding,'u',1)
      error('FractionBits is larger than number of binary bits')
    end
    if rem(Q.FractionBits,1)~=0, error('FrationBits is not integer'), end
    if strncmp(Q.Coding,'two''s complement',1)|strncmp(Q.Coding,'one''s complement',1)|...
        strncmp(Q.Coding,'sign-magnitude',1)|strncmp(Q.Coding,'unsigned',1)
    else error(['coding=''',Q.Coding,''' is not allowed'])
    end
    if strncmp(Q.Coding,'t',1),
      xnminl=-pow2(Q.Bits-Q.FractionBits-1);
    else
      xnminl=-pow2(pow2(Q.Bits-1)-1,-Q.FractionBits);
    end
    xpmaxl=pow2(pow2(Q.Bits-1)-1,-Q.FractionBits);
    if Q.Minimum<xnminl, error('Minimum is too small for the given bit number'), end
    if Q.Maximum>xpmaxl, error('Maximum is too large for the given bit number'), end
    if Q.Maximum<Q.Minimum, error('Maximum < Minimum'), end
  else
    error(['Type is invalid as ''',Q.Type,''''])
  end
  if nargout>0, Qout=1; end, return
end
%
%if length(varargin)<1
if ~exist('qdef'), qdef=''; end
%else
%  qdef=varargin{1}; varargin(1)=[];
%end
%

if isempty(qdef)
  if isempty(p), qdef='double'; else qdef=p; end
end
if isstr(qdef)
  if isempty(qdef)
  elseif strcmpi(qdef,'s')|strcmpi(qdef,'single')
    p=24; Emin=-126; Emax=127; Ebias=-Emin+1;
  elseif strcmpi(qdef,'d')|strcmpi(qdef,'double')|strcmpi(qdef,'list')
    p=53; Emin=-1022; Emax=1023; Ebias=-Emin+1;
  elseif strncmpi(qdef,'32-bit',3)|strncmpi(qdef,'Q32bit',3)
    B=32; %p=B-1;
  elseif strncmpi(qdef,'segments',4)
    B=23; coding='s';
  elseif strncmpi(qdef,'16-bit',3)|strncmpi(qdef,'Q16bit',3)
    B=16; %p=B-1;
  elseif strncmpi(qdef,'8-bit',2)|strncmpi(qdef,'Q8bit',2)
    B=8; %p=B-1;
  elseif strcmpi(qdef,'list')
  else
    if (length(varargin)>0)&(rem(length(varargin),2)==1), varargin=[{qdef},varargin];
    else error(['qdef = ''',qdef,''' not allowed'])
    end
  end
elseif isstruct(qdef)
  fn=fieldnames(qdef);
  for ii=1:length(fn)
    %eval([fn{ii},'=getfield(qdef,fn{ii});'])
    if strcmp(fn{ii},'p')|strcmp(fn{ii},'prec')|strcmpi(fn{ii},'precision')
      p=getfield(qdef,fn{ii}); B=p+1; if isempty(typ), typ='floating-point'; end
    elseif strncmpi(fn{ii},'Type',2)
      typ=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Name',3)
      Name=getfield(qdef,fn{ii});
    elseif strcmp(fn{ii},'lb')|strcmpi(fn{ii},'leadingbit')
      lb=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'rdir')|strncmpi(fn{ii},'roundingdirection',11)
      rdir=getfield(qdef,fn{ii});
    elseif strcmp(fn{ii},'treath')|strncmpi(fn{ii},'treathalf',6)
      treath=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'coding',1)
      coding=getfield(qdef,fn{ii});
    elseif strcmp(fn{ii},'ufl')|strcmpi(fn{ii},'underflow')
      ufl=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emin')|strncmpi(fn{ii},'Expminimum',6)
      Emin=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emax')|strncmpi(fn{ii},'Expmaximum',6)
      Emax=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Ebias')|strncmpi(fn{ii},'Expbias',6)
      Ebias=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'B')|strncmpi(fn{ii},'bits',4)
      B=getfield(qdef,fn{ii}); p=B-1; if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'xnmin')|strncmpi(fn{ii},'minimum',3)
      xnmin=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'xpmax')|strncmpi(fn{ii},'maximum',3)
      xpmax=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'ovf')|strncmpi(fn{ii},'overflow',5)
      ovf=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strncmp(fn{ii},'fb',2)|strncmpi(fn{ii},'fractionbits',5)
      fb=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strncmp(fn{ii},'Operation',2)
      op=getfield(qdef,fn{ii});
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif strncmpi(fn{ii},'Dither',2)
      dith=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'ToD',3)
      tod=getfield(qdef,fn{ii});
    elseif strncmp(fn{ii},'L',1)|strncmpi(fn{ii},'NoB',3)|strncmpi(fn{ii},'Number_of_DitherBits',6)
      L=getfield(qdef,fn{ii});
    elseif strncmp(fn{ii},'A',1)|strncmpi(fn{ii},'Amplitude',9)
      A=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Phase',2)
      ph=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Var',3)|strncmpi(fn{ii},'Variance_of_Gaussian_Dither',8)
      gvar=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'RoF',3)|strcmpi(fn{ii},'Ratio_of_Frequencies')
      rof=getfield(qdef,fn{ii});
      if (length(rof) ~= 2) error('Size of vector [numerator, denominator] is wrong.');
      else rofn=rof(1); rofd=rof(2);
      end
    elseif strncmpi(fn{ii},'DFunction',4)|strncmpi(fn{ii},'DFunction_given_by_User',9)
      dfun=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Sequenceperm',2)
      seqp=getfield(qdef,fn{ii});
      %    elseif strncmpi(fn{ii},'Other',5)
      %         error('It is not allowed to determine "Other" dither-properties.');
    else
      if ~(strncmpi(fn{ii},'Other',5)) error(['Unknown qdef field ''',fn{ii},'''']); end
    end
  end
elseif isnumeric(qdef)
  if rem(qdef,1)~=0, error('p is not an integer'), end
  Qout=qstruct(struct('p',qdef));
  return
end
%
if length(varargin)>=2
  pm=0;
  if isstr(varargin{1}), fn=varargin(1:2:end);
  elseif isstruct(varargin{1}), fn=varargin(2:2:end); pm=1;
  end
  for ii=1:length(fn)
    %eval([fn{ii},'=getfield(qdef,fn{ii});'])
    if strcmp(fn{ii},'p')|strcmp(fn{ii},'prec')|strcmpi(fn{ii},'precision')
      p=varargin{2*ii+pm}; if isempty(typ), typ='floating-point'; end
      if ~isempty(B)&~isequal(p,B-1)
        error(sprintf('Both  p = %.0f  and  B = %.0f  are given, and are inconsistent',p,B))
      end
    elseif strncmpi(fn{ii},'Type',2)
      typ=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'Name',3)
      Name=varargin{2*ii+pm};
    elseif strcmp(fn{ii},'lb')|strcmpi(fn{ii},'leadingbit')
      lb=varargin{2*ii+pm}; if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'rdir')|strncmpi(fn{ii},'roundingdirection',11)
      rdir=varargin{2*ii+pm};
    elseif strcmp(fn{ii},'treath')|strncmpi(fn{ii},'treathalf',6)
      treath=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'coding',1)
      coding=varargin{2*ii+pm};
    elseif strcmp(fn{ii},'ufl')|strcmpi(fn{ii},'underflow')
      ufl=varargin{2*ii+pm}; if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emin')|strncmpi(fn{ii},'Expminimum',6)
      Emin=varargin{2*ii+pm}; if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emax')|strncmpi(fn{ii},'Expmaximum',6)
      Emax=varargin{2*ii+pm}; if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Ebias')|strncmpi(fn{ii},'Expbias',6)
      Ebias=varargin{2*ii+pm}; if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'B')|strncmpi(fn{ii},'bits',4)
      B=varargin{2*ii+pm}; if isempty(typ), typ='fixed-point'; end
      if ~isempty(p)&~isequal(p,B-1)
        error(sprintf('Both  p = %.0f  and  B = %.0f  are given, and are inconsistent',p,B))
      end
    elseif strcmpi(fn{ii},'xnmin')|strncmpi(fn{ii},'minimum',3)
      xnmin=varargin{2*ii+pm}; if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'xpmax')|strncmpi(fn{ii},'maximum',3)
      xpmax=varargin{2*ii+pm}; if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'ovf')|strncmpi(fn{ii},'overflow',5)
      ovf=varargin{2*ii+pm}; if isempty(typ), typ='fixed-point'; end
    elseif strncmp(fn{ii},'fb',2)|strncmpi(fn{ii},'fractionbits',5)
      fb=varargin{2*ii+pm}; if isempty(typ), typ='fixed-point'; end
    elseif strncmpi(fn{ii},'Operation',2)
      op=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'Dither',2)
      dith=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'ToD',3)
      tod=varargin{2*ii+pm};
    elseif strncmp(fn{ii},'L',1)|strncmpi(fn{ii},'NoB',3)|strncmpi(fn{ii},'Number_of_DitherBits',6)
      L=varargin{2*ii+pm};
    elseif strncmp(fn{ii},'A',1)|strncmpi(fn{ii},'Amplitude',9)
      A=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'phase',2)
      ph=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'Var',3)|strncmpi(fn{ii},'Variance_of_Gaussian_Dither',8)
      gvar=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'RoF',3)|strcmpi(fn{ii},'Ratio_of_Frequencies')
      rof=varargin{2*ii+pm};
      if (length(rof) ~= 2), error('Size of vector [numerator, denominator] is wrong.');
      else rofn=rof(1); rofd=rof(2);
      end
    elseif strncmpi(fn{ii},'DFunction',4)|strncmpi(fn{ii},'DFunction_given_by_User',9)
      dfun=varargin{2*ii+pm};
    elseif strncmpi(fn{ii},'Sequenceperm',3)
      seqp=varargin{2*ii+pm};
      %     elseif strncmpi(fn{ii},'Other',5)
      %         error('It is not allowed to determine "Other" dither-properties.');
    else
      if ~(strncmpi(fn{ii},'Other',5)) error(['Unknown qdef field ''',fn{ii},'''']); end
    end
  end
end
%
if isempty(typ)
  if ~isempty(B)|~isempty(xnmin), typ='fixed-point';
  elseif ~isempty(p)|~isempty(ufl)|~isempty(lb)|~isempty(Emin), typ='floating-point';
  else error('Cannot determine type of quantizer')
  end
end

if strncmp(typ,'float',2)
  if ~exist('p'), p=[]; end, if isempty(p), p=53; end
  fb=p-1;
  %
  if ~exist('lb'), lb=''; end, if isnan(lb), lb=''; end
  if isempty(lb), lb='h'; end %hidden leading bit
  if ~exist('rdir'), rdir=''; end, if isnan(rdir), rdir=''; end
  if isempty(rdir), rdir='r'; end %round
  if ~exist('treath'), treath=''; end, if isnan(treath), treath=''; end
  if isempty(treath), treath='e'; end %round to even
  if ~exist('coding'), coding=''; end, if isempty(coding), coding='s'; end %sign-magnitude
  if ~exist('ufl'), ufl=''; end, if isnan(ufl), ufl=''; end
  if isempty(ufl), ufl='g'; end %gradual underflow
  %
  Eminlim=round(log2(realmin)); Emaxlim=round(log2(realmax))-1;
  %For IEEE double precision, Emin=-1022, Emax=1023
  if ~exist('Emin'), Emin=[]; end, if isnan(Emin), Emin=[]; end
  if isempty(Emin), Emin=Eminlim; end
  if ~exist('Emax'), Emax=[]; end, if isnan(Emax), Emax=[]; end
  if isempty(Emax), Emax=Emaxlim; end
  if ~exist('expcoding'), expcoding=''; end, if isnan(expcoding), expcoding=''; end
  if isempty(expcoding), expcoding='biased'; end
  if ~exist('Ebias'), Ebias=[]; end, if isnan(Ebias), Ebias=[]; end
  if isempty(Ebias)&strcmp(expcoding,'biased'), Ebias=-Emin+1; end
else %fixed-point
  if isempty(B), B=16; end
  if isnan(fb), fb=[]; end
  if isempty(fb), fb=0; end %decimal point at right
  if isnan(rdir), rdir=''; end
  if isempty(rdir), rdir='r'; end %rounding
  if isnan(treath), treath=''; end
  if isempty(treath), treath='e'; end %round to even
  if isnan(coding), coding=''; end
  if isempty(coding), coding='t'; end %two's complement
  if isnan(ovf), ovf=''; end
  if isempty(ovf), ovf='m'; end %modular
  if isnan(xnmin), xnmin=[]; end
  if strncmp(coding,'t',1), xnminl=-pow2(B-fb-1); else xnminl=-pow2(pow2(B-1)-1,-fb); end
  if isempty(xnmin), xnmin=xnminl; end
  if isnan(xpmax), xpmax=[]; end
  xpmaxl=pow2(pow2(B-1)-1,-fb);
  if isempty(xpmax), xpmax=xpmaxl; end
end
typ=lower(typ);


if ~strcmpi(dith,'on')
  dith='off';
  Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
    'TreatHalf',treath,'Coding',coding,'Dither',dith);
else
  dith='on';
  if (~exist('L','var'))
    L=3;     %% Default Number of DitherBits
  end
  if (L==0)
    error('Number of DitherBits is 0.');
  end
  if (~exist('tod','var'))
    error('Type of Dither is missing.');
  end
  if ~exist('seqp','var') seqp='disabled';
  else if strncmpi(seqp,'enable',2)
      seqp='enabled';
    else if strncmpi(seqp,'disable',3)
        seqp='disabled';
      else error('Sequence-property is wrong.');
      end
    end
  end


  if strcmp(tod,'u1')|strcmp(tod,'u2')|strcmp(tod,'u3')|strncmpi(tod,'uniform',2)
    tod=lower(tod);
    if (~strcmp(tod,'u1'))
      if (~strcmp(tod,'u2'))
        if (~strcmp(tod,'u3'))
          tod='u2';
        end
      end
    end

    if strncmpi(typ,'fixed',2)
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));

    else
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    if (exist('A','var'))  warning('Amplitude is unnecessary by uniform dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary by uniform dither.'); end
    if (exist('gvar','var'))  warning('Variance is unnecessary by uniform dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary by uniform dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denum.) is unnecessary by uniform dither.'); end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,'Sequenceperm',seqp,'Other',other);

  end

  if strncmpi(tod,'triangular',1)
    if (~strcmp(tod,'t1'))
      if (~strcmp(tod,'t2'))
        if (~strcmp(tod,'t3'))
          tod='t2';
        end
      end
    end

    if strncmpi(typ,'fixed',2)
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    if (exist('A','var')) warning('Amplitude is unnecessary by triangular dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary by triangular dither.'); end
    if (exist('gvar','var'))  warning('Variance is unnecessary by triangular dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary by triangular dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denum.) is unnecessary by triangular dither.'); end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,'Sequenceperm',seqp,...
      'Other',other);

  end

  if strncmpi(tod,'Gaussian',2)
    tod='Gaussian';
    if ~exist('gvar','var') error('Variance of Gaussian Dither is missing.'); end

    if strncmpi(typ,'fixed',2)
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith=sdith;
      other.Probab=probab;
      other.Hmtimes=hmtimes;
      other.SumHmtimes=sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,...
      'Variance_of_Gaussian_Dither',gvar,'Sequenceperm',seqp,'Other',other);

    if (exist('A','var'))  warning('Amplitude is unnecessary by Gaussian dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary by Gaussian dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary by Gaussian dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denum.) is unnecessary by Gaussian dither.'); end

  end

  if strncmpi(tod,'Sinusoid',3)
    tod='sinusoid';
    if ~exist('A','var') error('Amplitude of Sinusoid Dither is missing.'); end
    if (A == 0) error('Amplitude of sinosoid dither is 0.'); end
    if ~exist('rofn','var') error('The numerator of frequency-ratio of sinusoidal dither is missing.'); end
    if (rofn == 0) error('Ratio of frequencies (numerator) is 0.'); end
    if ~exist('rofd','var') error('The denumerator of frequency-ratio of sinusoid dither is missing.'); end
    if (rofd == 0) error('Ratio of frequencies (denumerator) is 0.'); end
    if ~exist('ph','var') error('Phase of Sinusoid Dither is missing.'); end

    if strncmpi(typ,'fixed',2)
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,...
      'Amplitude_of_Sinusoid_Dither',A,'Ratio_of_Frequencies',[rofn,rofd],...
      'Phase',ph,'Sequenceperm',seqp,'Other',other);

    if (exist('gvar','var'))  warning('Variance is unnecessary by sinusoidal dither.'); end

  end

  if strncmpi(tod,'user',2)
    tod='userfunction';
    if ~exist('dfun','var') error('Dither function is missing.'); end

    if strncmpi(typ,'fixed',2)
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value,dfun);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value,dfun);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end
    
    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod, 'Dfunction',dfun,...
      'Number_of_DitherBits',L,'Sequenceperm',seqp,'Other',other);

    if (exist('A','var'))  warning('Amplitude is unnecessary for user defined dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary for user defined dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary for user defined dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denom.) is unnecessary for user defined dither.'); end
  end

end

typ=lower(typ);
if strncmp(typ,'float',2)
  Q.LeadingBit=lb; Q.UnderFlow=ufl; Q.ExpMinimum=Emin; Q.ExpMaximum=Emax; Q.ExpBias=Ebias;
elseif strncmp(typ,'fix',2)
  Q.Bits=B; Q.Minimum=xnmin; Q.Maximum=xpmax; Q.OverFlow=ovf;
end
Q.Operation=op;

qstruct(Q,'test'); %test consistency

%make list
if nargout==0, qstructget(Q), else Qout=Q; end
%
%End of qstruct
