function testroundoff(varargin)
%TESTROUNDOFF  Test roundoff function M-files
%
%        TESTROUNDOFF
%        TESTROUNDOFF QUICK %no problem solutions

%        TESTQDSP

if (length(varargin)==1)&strcmp(varargin{1},'help'), help roundoff/private, return, end

if (nargin>=1)
  if isstr(varargin{1})&exist(['roundoff',filesep,'private',filesep,varargin{1},'.m'])
    if (length(varargin)==1)&strcmp(varargin{1},'testqmake'), testqmake, return, end
    if nargout==0, feval(varargin{:});
    elseif nargout==1, out1=feval(varargin{:}); varargout={out1};
    else error('Unrecognized call')
    end
    return
  elseif strcmp(varargin{1},'quick')
    mode=varargin{1};
  else
    error('Invalid call')
  end
end

if ~runique, error('Name collision, see info in command window'), end
mode='';

disp('  First step: reset random generators to make test repeatable')
rand('seed',0), randn('seed',0)
c0=clock;

%ULP
if length(which('ulp','-all'))>1
  wulp=which('ulp','-all');
  if any(findstr(['roundoff',filesep,'ulp.m'],wulp{1})) | ...
      any(findstr(['roundoff',filesep,'ulp.m'],wulp{2}))
    if any(findstr(['@qdata',filesep,'ulp.m'],wulp{1})) | ...
        any(findstr(['@qdata',filesep,'ulp.m'],wulp{2}))
    else
      warning('Several ulp functions are available')
      wulp
    end
  else
    disp('rmpathfr(''qdsp'') ???')
    error('Several ulp functions are available')
  end
end

%BIN2NUM
bin2num('+(1).0000 0000 0000 0000 0000 001 *2^[+0]','single-roundoff')-1 %2^(-23) = 1.1921e-007
if ~isequal(ans,2^(-23)), error('test error'), end
bin2num('+(0)(.)0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 (*2^)0000 0000 000') %0
if ~isequal(ans,0), error('test error'), end
bin2num('+1.0000 0000 0000 000 e [0000 0002]') %4
if ~isequal(ans,4), error('test error'), end
bin2num('+(1).0000 0000 0000 000 e 0000 0000 000',struct('expc','t')) %1
if ~isequal(ans,1), error('test error'), end
bin2num('1111',struct('Coding','t')) %-1
if ~isequal(ans,-1), error('test error'), end

%COMPRESS
x=[0:0.01:128]; cla, hold off
plot(x,compress(x,1,'asinh'),'k'), hold on
plot(x,compress(x,1,'u'),'r')
plot(x,compress(x,1,'lower'),'g')
plot(x,compress(x,1,'mu-law')/16,'c')
plot(x,compress(x,1,'A-law')/16,'m')
plot(x,compress(x,1,'p'),'b')
hold off, zoom on, shg

%EXPAND
y=[0:0.01:8]; cla, hold off
plot(y,expand(y,1,'asinh'),'k'), hold on
plot(y,expand(y,1,'u'),'r')
plot(y,expand(y,1,'lower'),'g')
plot(y,expand(y*16,1,'mu-law'),'c')
plot(y,expand(y*16,1,'A-law'),'m')
plot(y,expand(y,1,'p'),'b')
hold off, zoom on, shg
close all, drawnow

%LSB
[mag,lsbit]=lsb(1)  %is [eps = 2^(-52) = 2.2204e-016,0]
if ~isequal(lsbit,0), error('test error'), end
if ~isequal(mag,eps), error('test error'), end
[mag,lsbit]=lsb(2)  %is [2*eps = 2*2^(-52) = 4.4409e-016,0]
if ~isequal(lsbit,0), error('test error'), end
if ~isequal(mag,2*eps), error('test error'), end
[mag,lsbit]=lsb(1+2^(-23),24)  %is [eps = 2^(-23) = 1.1921e-007,1]
if ~isequal(lsbit,1), error('test error'), end
if ~isequal(mag,2^(-23)), error('test error'), end
[mag,lsbit]=lsb(0)  %is [realmin*eps = 2^(-1022-52) = 4.9407e-324,0]
if ~isequal(lsbit,0), error('test error'), end
if ~isequal(mag,realmin*eps), error('test error'), end
[mag,lsbit]=lsb(2^16-1,qstruct('FractionBits',0)) %is [1,1]
if ~isequal(lsbit,1), error('test error'), end
if ~isequal(mag,1), error('test error'), end
%
Qi=0;
for Q={qstruct('double'),qstruct('32-bit'),qstruct('32-bit')}
  Qi=Qi+1; Q=Q{1};
  if Qi==1
    lsb0=realmin*eps;
    xri=[1,1+eps,1+2*eps,1.5,1,-1-eps,-1-2*eps,-1.5,1,1-eps,1-2*eps,0,inf,nan];
    lsbmagxi=[eps,eps,eps,eps,eps,eps,eps,eps,eps,eps/2,eps/2,lsb0,inf,nan];
    lsbvali=[0,eps,0,0,0,eps,0,0,0,eps,0,0,inf,nan]; %14
  else
    lsb0=1;
    xri=[112,113,114,122,1,-1,-6,-7,2^31-1,-2^31,0];
    lsbmagxi=[1,1,1,1,1,1,1,1,1,1,1];
    lsbvali =[0,1,0,0,1,1,0,1,1,1,0]; %11
    if Qi==3
      Q=qstructset(Q,'fractbit',8,'B',32);
      lsb0=lsb0/2^8;
      xri=xri/2^8;
      lsbmagxi=lsbmagxi/2^8;
      lsbvali=lsbvali/2^8;
    end
  end
  for mod={'','complex'}
    mod=mod{1};
    for ri=1:length(xri)
      for ii=1:length(xri)
        for xi=1:3
          if xi==1, x=xri(ri);
            if ~strcmp(mod,'complex'), lsbmagx=lsbmagxi(ri);
            else lsbmagx=lsbmagxi(ri)+j*lsb0;
            end
          elseif xi==2, x=xri(ri)+j*xri(ii);
            if ~isequal(xri(ii),0)|strcmp(mod,'complex'), lsbmagx=lsbmagxi(ri)+j*lsbmagxi(ii);
            else lsbmagx=lsbmagxi(ri);
            end
          elseif xi==3,
            x=j*xri(ii);
            if ~isequal(xri(ii),0)|strcmp(mod,'complex'), lsbmagx=lsb0+j*lsbmagxi(ii);
            else lsbmagx=lsb0;
            end
          end
          [lsbmag,lsbval]=lsb(x,Q,mod);
          if ~isfinite(lsbmag)&~isfinite(lsbmagx)
          elseif isequal(lsbmag,lsbmagx)
          else
            fprintf('Qi = %.0f, ri = %.0f, ii = %.0f, xi = %.0f, x = %.4g + j*%.4g\n',Qi,ri,ii,xi,real(x),imag(x))
            lsbmag,lsbmagx
            error('Wrong lsb value')
          end
        end
      end %for ii
    end %for ri
  end %for mod
end %for Q
%
%NEXTAFTER
nextafter(1,2)-1  %is eps = 2^(-52) = 2.2204e-016
if ~isequal(ans,2^(-52)), error('test error'), end
nextafter(1,0)-1  %is eps/2 = 2^(-52-1) = 1.1104e-016
if ~isequal(ans,-2^(-52-1)), error('test error'), end
nextafter(1,2,24)-1  %is eps = 2^(-23) = 1.1921e-007
if ~isequal(ans,2^(-23)), error('test error'), end

%NUM2BIN
str=num2bin(1)
if ~isequal(1,bin2num(str)), error('test error'), end
str=num2bin(roundq(realmin,'list'))
if ~isequal(realmin,bin2num(str)), error('test error'), end
str=num2bin(0,struct('p',24,'Emin',-26,'Ebias',27,'lb','shown'))
if ~isequal(0,bin2num(str,struct('p',24,'Emin',-26,'Ebias',27,'lb','shown'))), error('test error'), end
str=num2bin(0,'single')
if ~isequal(0,bin2num(str,'single-roundoff')), error('test error'), end
try, str=num2bin(1+double(eps('single')),'single')
catch, str=num2bin(1+double(2^(-23)),'single')
end
%this returns +(1).0000 0000 0000 0000 0000 001 2^[0d]
if ~strcmp(str,'+(1)(.)0000 0000 0000 0000 0000 001 (*2^)0111 1111 [0d]'), error('test error'), end

%NUM2BIN
str=num2bin(1)
if ~isequal(1,bin2num(str)), error('test error'), end
str=num2bin(roundq(realmin,'list'));
if ~isequal(realmin,bin2num(str)), error('test error'), end
str=num2bin(0,struct('p',24,'Emin',-26,'Ebias',27,'lb','shown'))
if ~isequal(0,bin2num(str,struct('p',24,'Emin',-26,'Ebias',27,'lb','shown'))), error('test error'), end
str=num2bin(0,'single')
if ~isequal(0,bin2num(str,'single-roundoff')), error('test error'), end
str=num2bin(1.75,struct('Bits',6,'FractionBits',2)) %+001.11
if ~isequal(1.75,bin2num(str,struct('Bits',6,'FractionBits',2))), error('test error'), end
str=num2bin(-1.75,struct('Bits',6,'FractionBits',4,'Coding','t')) %1110.01
if ~isequal(-1.75,bin2num(str,struct('Bits',6,'FractionBits',4,'Coding','t'))), error('test error'), end
str=num2bin(3.125,'fixed-point') %+11.001 %shortest fixed-point string with sign
if ~isequal(3.125,bin2num(str)), error('test error'), end
ver=version; 
if ver(1)>='7'
  %Matlab 7.x only:
  str=num2bin(1+double(eps('single')),'single','dig')
  %this returns +(1).0000 0000 0000 0000 0000 001 2^[0d]
  if ~strcmp(str,'+(1)(.)0000 0000 0000 0000 0000 001 (*2^)0111 1111 [0d]'), error('test error'), end
end

%ROUNDOFF (QFIX)
roundq(1.75,qstruct('Bits',4,'FractionBits',0)) %2
if ~isequal(ans,2), error('test error'), end
qfix remark
 
%ROUNDOFF (QFLOAT)
roundq(1+2^(-24),'s')-1  %returns 0
if ~isequal(ans,0), error('test error'), end
roundq(1+2^(-23),'s')-1  %returns 2^(-23)=1.1921e-007
if ~isequal(ans,2^(-23)), error('test error'), end
roundq(realmin/2,qstruct('ufl','f'),'list')  %returns 0
if ~isequal(ans,0), error('test error'), end
roundq(realmin/2,qstruct('ufl','g'))  %returns  realmin/2=1.1125e-308
if ~isequal(ans,2^(-1022-1)), error('test error'), end
qfloat remark

%QSTRUCTGET
qstructget %all properties
qstructget('d') %double precision
qstructget('single') %single precision
qstructget(qstruct('Bits',4,'FractionBits',0))

%QSTRUCT
qstruct('double') %IEEE double precision
qstruct('single') %IEEE single precision
Q=qstruct('p',18) %floating-point with precision 18
Q16=qstruct('Qf1','16-bit') %16-bit fixed point, two's complement, modular overflow
Q32=qstruct('Qf2','32-bit') %32-bit fixed point, two's complement, modular overflow
qstruct('16-bit-sm','Bits',16,'coding','s')

%QSTRUCTSET
Q=qstructset(Q,'p',20) %change precision to 20 
Q=qstruct('double','prec',50) %IEEE double but 50-bit precision
Q2=qstructset(Q,'Precision',20) %change precision in Q to 20

%ULP
ulp(1)  %returns eps/2 = 2^(-53) = 1.1102e-016
if ~isequal(ans,2^(-53)), error('test error'), end
ulp(2)  %returns eps = 2^(-52) = 2.2204e-016
if ~isequal(ans,2^(-52)), error('test error'), end
ulp(1.55,24)  %returns 1.1921e-007 = 2^(-23)
if ~isequal(ans,2^(-23)), error('test error'), end
ulp(roundq(1.55,24),24)  %returns 1.1921e-007 = 2^(-23)
if ~isequal(ans,2^(-23)), error('test error'), end
log2(ulp(roundq(2.55,24),24))  %returns -22
if ~isequal(ans,-22), error('test error'), end
ulp(realmin/1.1,struct('underflow','flush to zero'))  %returns 2^(-1022)/2 = 1.1125e-308
if ~isequal(ans,2^(-1023)), error('test error'), end
ulp(realmin)  %returns 2^(-1022-52) = 4.9407e-324
if ~isequal(ans,2^(-1022-52)), error('test error'), end

%ROUNDOFF, PQN
testpqn

%Examples
test_roundoff_calls(5)

%Dither
roundq('private','test_roundoff_dither');

%ROUNDFFT, ROUNDDFT
if ~strcmp(mode,'quick')
  test_fft_dft
end

%RESONATORS
test_tf2res %transfer function
test_resfiltq %filtering
test_resimp %impulse response

global yesinpacceptdef
yesinpacceptdef='yes'; pause off
qmoments
pause on, clear global yesinpacceptdef

if exist('testproblems.m')&~strcmp(mode,'quick')
  dbs=dbstack; tfound=0;
  for ii=1:length(dbs)
    if isfield(dbs(ii),'file')&strcmp(dbs(ii).file,'testproblems.m')
      tfound=1; %call from that function
    end
    %if strncmp(version,'5',1), tfound=1; end %do not call problems from Matlab 5
  end
  if ~tfound, testproblems, end %test problem solving
end
close all
disp('All tests for toolbox ''roundoff'' were finished OK')
et=etime(clock, c0);
fprintf('Elapsed time: %.0f min %.0f s\n',fix(et/60),rem(et,60))
%disp('End of all tests of quantization')
%
%End of testroundoff